package com.gic.store.dto;

import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.service.StoreStrategyApiService;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;

/**
 * 门店分组策略和门店启用策略
  * @ClassName: StoreStrategyDTO 
  * @Description:   门店分组策略和门店启用策略
  * @author guojuxing 
  * @date 2019/7/31 10:32 AM 
  */
public class StoreStrategyDTO implements Serializable {
    private static final long serialVersionUID = -4683081402308681202L;

    public StoreStrategyDTO() {

    }

    /**
     *
     */
    @NotNull(message = "主键不能为空", groups = { StoreStrategyApiService.ModifyStoreStrategy.class })
    private Integer strategyId;

    /**
     * 策略名称
     */
    @NotBlank(message = "策略名称不能为空", groups = { StoreStrategyApiService.SaveStoreStrategy.class,
            StoreStrategyApiService.ModifyStoreStrategy.class })
    private String strategyName;

    /**
     *
     */
    @NotNull(message = "企业ID不能为空", groups = { StoreStrategyApiService.SaveStoreStrategy.class })
    private Integer enterpriseId;

    /**
     * 门店域id
     */
    @NotNull(message = "门店域不能为空", groups = { StoreStrategyApiService.SaveStoreStrategy.class,
            StoreStrategyApiService.ModifyStoreStrategy.class })
    private Integer storeRegionId;

    /**
     * 门店分组条件，json数组
     */
    private String conditions;

    /**
     * 策略命中值；如果type为门店分组策略，对应分组id；否则为门店启用和不启用值
     */
    @NotBlank(message = "策略命中值不能为空", groups = { StoreStrategyApiService.SaveStoreStrategy.class,
            StoreStrategyApiService.ModifyStoreStrategy.class })
    private String targetValue;

    /**
     *
     */
    private Date createTime;

    /**
     *
     */
    private Date updateTime;

    /**
     * 状态值  1有效  0无效
     */
    private Integer deleteFlag;

    /**
     * 策略类型，1门店状态策略，   2门店分组策略
     */
    @NotNull(message = "策略类型不能为空", groups = { StoreStrategyApiService.SaveStoreStrategy.class })
    @Min(value = StoreGroupConstant.STORE_STRATEGY_TYPE, message = "最小值不能小于{value}", groups = {
            StoreStrategyApiService.SaveStoreStrategy.class })
    @Max(value = StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE, message = "最大值不能大于{value}", groups = {
            StoreStrategyApiService.SaveStoreStrategy.class })
    private Integer strategyType;

    /**
     * 排序字段
     */
    private Double sort;

    /**
     * 策略类型，1门店状态策略时候，可以多选，门店创建方式，1：后台新增   2：批量导入  3：订单生成  4：erp同步,可以多选 _1_2_格式
     */
    private String storeCreateType;

    /**
     * 排序值
     */
    private Integer seq;

    public Integer getStrategyId() {
        return strategyId;
    }

    public void setStrategyId(Integer strategyId) {
        this.strategyId = strategyId;
    }

    public String getStrategyName() {
        return strategyName;
    }

    public void setStrategyName(String strategyName) {
        this.strategyName = strategyName;
    }

    public Integer getEnterpriseId() {
        return enterpriseId;
    }

    public void setEnterpriseId(Integer enterpriseId) {
        this.enterpriseId = enterpriseId;
    }

    public Integer getStoreRegionId() {
        return storeRegionId;
    }

    public void setStoreRegionId(Integer storeRegionId) {
        this.storeRegionId = storeRegionId;
    }

    public String getConditions() {
        return conditions;
    }

    public void setConditions(String conditions) {
        this.conditions = conditions;
    }

    public String getTargetValue() {
        return targetValue;
    }

    public void setTargetValue(String targetValue) {
        this.targetValue = targetValue;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public Date getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Date updateTime) {
        this.updateTime = updateTime;
    }

    public Integer getDeleteFlag() {
        return deleteFlag;
    }

    public void setDeleteFlag(Integer deleteFlag) {
        this.deleteFlag = deleteFlag;
    }

    public Integer getStrategyType() {
        return strategyType;
    }

    public void setStrategyType(Integer strategyType) {
        this.strategyType = strategyType;
    }

    public Double getSort() {
        return sort;
    }

    public void setSort(Double sort) {
        this.sort = sort;
    }

    public Integer getSeq() {
        return seq;
    }

    public void setSeq(Integer seq) {
        this.seq = seq;
    }

    public String getStoreCreateType() {
        return storeCreateType;
    }

    public void setStoreCreateType(String storeCreateType) {
        this.storeCreateType = storeCreateType;
    }

    @Override
    public String toString() {
        return "StoreStrategyDTO{" +
                "strategyId=" + strategyId +
                ", strategyName='" + strategyName + '\'' +
                ", enterpriseId=" + enterpriseId +
                ", storeRegionId=" + storeRegionId +
                ", conditions='" + conditions + '\'' +
                ", targetValue='" + targetValue + '\'' +
                ", createTime=" + createTime +
                ", updateTime=" + updateTime +
                ", status=" + deleteFlag +
                ", strategyType=" + strategyType +
                ", sort=" + sort +
                ", storeCreateType='" + storeCreateType + '\'' +
                ", seq=" + seq +
                '}';
    }
}
