package com.gic.store.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.CityDTO;
import com.gic.store.dto.CountyDTO;
import com.gic.store.dto.ProvinceDTO;

import java.util.List;

/**
 *
 * @ClassName: ProvincesApiService
 * @Description:
 * @author zhiwj
 * @date 2020-09-09 16:04
 */
public interface ProvincesApiService {
   
   /**
    * 查询所有省份
    * @Description: 查询所有省份
    * @author taogs
    * @Date 13:57 2019/8/5
    * @Param 
    * @return 
   */
    ServiceResponse<List<ProvinceDTO>> selectAllProvince();

    
    /**
     * 查询所有城市
     * @Description: 查询所有城市
     * @author taogs
     * @Date 14:07 2019/8/5
     * @Param 
     * @return 
    */
    ServiceResponse<List<CityDTO>> selectAllCity();
    
    /**
     * 通过城市id查询城市
     * @Description: 通过城市id查询城市
     * @author taogs
     * @Date 11:10 2019/12/9
     * @param cityId
     * @return 
    */
    ServiceResponse<CityDTO> getCityByCityId(String cityId);

    /**
     * 通过省id查询城市
     * @Description: 通过省id查询城市
     * @author taogs
     * @Date 14:29 2019/12/4
     * @param provinceId
     * @return 
    */
    ServiceResponse<List<CityDTO>> selectCityByProvinceId(String provinceId);

  /**
   * 查询所有地区
   * @Description: 查询所有地区
     * @author taogs
     * @Date 14:07 2019/8/5
     * @Param 
     * @return 
    */
    ServiceResponse<List<CountyDTO>> selectAllCounty();

    
    /**
     *
     * 通过地区id查询
     * @Description: 通过地区id查询
     * @author taogs
     * @Date 11:11 2019/12/9
     * @param countyId
     * @return 
    */
    ServiceResponse<CountyDTO> getCountyByCountyId(String countyId);
    
    /**
     * 通过城市id查询地区
     * @Description: 通过城市id查询地区
     * @author taogs
     * @Date 14:38 2019/12/4
     * @param cityId
     * @return 
    */
    ServiceResponse<List<CountyDTO>> selectCountyByCityId(String cityId);

    
    /**
     * 通过城市id查询地区id
     * @Description: 通过城市id查询地区id
     * @author taogs
     * @Date 14:44 2019/12/4
     * @param cityId
     * @return 
    */
    ServiceResponse<List<String>> selectCountyIdByCityId(String cityId);
    
   /**
    * 根据省份名称查询code
    * @Description: 根据省份名称查询code
    * @author taogs
    * @Date 13:59 2019/8/5
    * @param search
    * @return 
   */
    ServiceResponse<List<ProvinceDTO>> queryProvinces(String search);

    
    /**
     * queryCity 根据城市名称查询code
     * @Title: queryCity
     * @Description:
     * @author zhiwj
     * @param search
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.CityDTO>>
     * @throws
     */
    ServiceResponse<List<CityDTO>> queryCity(String search);

    
    /**
     * 根据地区名称查询地区code
     * @Description: 根据地区名称查询地区code
     * @author taogs
     * @Date 14:08 2019/8/5
     * @param search
     * @return 
    */
    ServiceResponse<List<CountyDTO>> queryCounty(String search);

    /**
     * getProvinceById
     * getProvinceById
     * @Title: getProvinceById
     * @Description:
     * @author zhiwj
     * @param provinceId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.ProvinceDTO>
     * @throws
     */
    ServiceResponse<ProvinceDTO> getProvinceById(String provinceId);

    /**
     * getProvinceNameByAreaId
     * getProvinceNameByAreaId
     * @Title: getProvinceNameByAreaId
     * @Description:
     * @author zhiwj
     * @param aredId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> getProvinceNameByAreaId(String aredId);
    
    /**
     * 热门城市
     * @Title: listHotCity
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param brandId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.CityDTO>>
     * @throws
     */
    ServiceResponse<List<CityDTO>> listHotCity(Integer enterpriseId, Integer brandId);

    /**
     * listEnterpriseCity
     * listEnterpriseCity
     * @Title: listEnterpriseCity
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.CityDTO>>
     * @throws
     */
    ServiceResponse<List<CityDTO>> listEnterpriseCity(Integer enterpriseId);
}
