package com.gic.store.service;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.*;

import java.util.List;
import java.util.Map;

/**
 * @author zhiwj
 * @date 2019/6/24
 */
public interface StoreApiService {

    /**
     * 保存门店
     * @Title: saveOrUpdate
     * @Description: 保存门店
     * @author zhiwj
     * @param storeDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<StoreDTO> saveOrUpdate(StoreDTO storeDTO);

    /**
     * 通过订单创建门店
     * @Title: 通过订单创建门店
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeCode
     * @param storeName
     * @param regionId
     * @param orderNum
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveStoreForOrder(Integer enterpriseId, String storeCode, String storeName, Integer regionId, String orderNum);

    /**
     * 通过订单创建门店
     * @Title: saveStoreForOrder
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeCode
     * @param storeName
     * @param regionId
     * @param orderNum
     * @param storeBrandId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveStoreForOrder(Integer enterpriseId, String storeCode, String storeName, Integer regionId, String orderNum, Integer storeBrandId);

    /**
     * 同步pos会员创建门店
     * @Title: saveStoreForPosMember
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeCode
     * @param storeName
     * @param regionId
     * @param cardNo
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveStoreForPosMember(Integer enterpriseId, String storeCode, String storeName, Integer regionId, String cardNo);

    /**
     * 同步pos会员创建门店
     * @Title: saveStoreForPosMember
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeCode
     * @param storeName
     * @param regionId
     * @param cardNo
     * @param storeBrandId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveStoreForPosMember(Integer enterpriseId, String storeCode, String storeName, Integer regionId, String cardNo, Integer storeBrandId);

    /**
     * countByOverflowStatus
     * @Title: countByOverflowStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> countByOverflowStatus(Integer enterpriseId);

    /**
     * 分页门店列表
     * @Title: listStore
     * @Description: 分页门店列表 查es
     * @author zhiwj
     * @param storeSearchDTO
     * @param pageNum
     * @param pageSize
     * @param returnFileds
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Page<StoreDTO>> listStore(StoreSearchDTO storeSearchDTO, Integer pageNum, Integer pageSize, String returnFileds);

    /**
     * 分页门店列表
     * @Title: listStore
     * @Description: 分页门店列表 查es,查询门店所有字段
     * @author zhiwj
     * @param storeSearchDTO
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Page<StoreDTO>> listStore(StoreSearchDTO storeSearchDTO, Integer pageNum, Integer pageSize);

    /**
     * 管理员的权限门店数据
     * @param userId
     * @return
     */
    ServiceResponse<List<StoreDTO>> listStore(Integer userId);

    /**
     * 按id查询门店图片  key：storeId    value:门店图片列表
     * @Title: listStorePhotoByIds
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeIdList
     * @return com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.Integer,java.util.List<java.lang.String>>>
     * @throws
     */
    ServiceResponse<Map<Integer, List<String>>> listStorePhotoByStoreIds(Integer enterpriseId, List<Integer> storeIdList);

    /**
     * 按id查询门店图片  key：storeInfoIdList    value:门店图片列表
     * @Title: listStorePhotoByIds
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeInfoIdList
     * @return com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.Integer,java.util.List<java.lang.String>>>
     * @throws
     */
    ServiceResponse<Map<Integer, List<String>>> listStorePhotoByStoreInfoIds(Integer enterpriseId, List<Integer> storeInfoIdList);

    /**
     * 查询门店详情
     * @Title: getStoreById
     * @Description: 查询门店详情
     * @author zhiwj
     * @param enterpriseId
     * @param storeId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreDTO>
     * @throws
     */
    ServiceResponse<StoreDTO> getStoreById(Integer enterpriseId, Integer storeId);


    /**
     * getStoreByStoreInfoId  通过企业id和storeInfoId查询门店
     * @Title: getStoreByStoreInfoId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeInfoId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreDTO>
     * @throws
     */
    ServiceResponse<StoreDTO> getStoreByStoreInfoId(Integer enterpriseId, Integer storeInfoId);


    /**
     * 数据库查询
     * @Description: 数据库查询
     * @author taogs
     * @Date 14:54 2020/7/29
     * @Param enterpriseId
     * @param storeInfoId
     * @param enterpriseId
     * @return
    */
    ServiceResponse<StoreDTO> getStoreByStoreInfoIdFromDb(Integer enterpriseId, Integer storeInfoId);

    /**
     * getStoreByStoreCode
     * @Title: getStoreByStoreCode
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param regionId
     * @param storeCode
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreDTO>
     * @throws
     */
    ServiceResponse<StoreDTO> getStoreByStoreCode(Integer enterpriseId, Integer regionId, String storeCode);

    /**
     * 查询门店，包括营业时间数据
     * @Title: getStoreContainBusinessTimeByStoreCode
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param regionId
     * @param storeCode
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreDTO>
     * @throws
     */
    ServiceResponse<StoreDTO> getStoreContainBusinessTimeByStoreCode(Integer enterpriseId, Integer regionId, String storeCode);

    /**
     * 校验除 是否重复 外的所有条件
     * @Title: validateFieldExceptRepeat
     * @Description: 校验除 是否重复 外的所有条件
     * @author zhiwj
     * @param value
     * @param storeFieldDTO
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Void> validateFieldExceptRepeat(String value, StoreFieldDTO storeFieldDTO);

    /**
     * 溢出门店列表
     * @Title: listStoreOverFlow
     * @Description: 溢出门店列表
     * @author zhiwj
     * @param enterpriseId
     * @param page
     * @param search
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.StoreDTO>>
     * @throws
     */
    ServiceResponse<Page<StoreDTO>> listStoreOverFlow(Integer enterpriseId, Page<StoreDTO> page, String search);

    /**
     * bulkUpdateStore  门店批量操作
     * @Title: bulkUpdateStore
     * @Description:
     * @author zhiwj
     * @param storeIds
     * @param value
     * @param enterpriseId
     * @param type
     * @param operatorId
     * @param operatorName
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse bulkUpdateStore(String storeIds, String value, Integer enterpriseId, int type, Integer operatorId, String operatorName);

    /**
     * 查询门店
     * @Title: getByStoreNameAndStoreCode
     * @Description: 查询门店
     * @author zhiwj
     * @param enterpriseId
     * @param storeName
     * @param storeCode
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreDTO>
     * @throws
     */
    ServiceResponse<StoreDTO> getByStoreNameAndStoreCode(Integer enterpriseId, Integer regionId, String storeName, String storeCode);

    /**
     * 添加单个门店到es
     * @Title: addStoreToIndex
     * @Description: 添加单个门店到es
     * @author zhiwj
     * @param enterpriseId
     * @param storeId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Void> addStoreToIndex(Integer enterpriseId, Integer storeId);

    /**
     * 门店es刷新队列方法
     * @Title:
     * @Description: 门店es刷新队列方法
     * @author zhiwj
     * @param param
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Void> addStoreToIndexMq(String param);

    /**
     * 批量添加门店到索引
     * @Title: bulkAddStoreToIndex
     * @Description: 批量添加门店到索引
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Integer> bulkAddStoreToIndex(Integer enterpriseId);

    
    /**
     * 通过分组刷新门店缓存
     * @Description: 通过分组刷新门店缓存
     * @author taogs
     * @Date 14:25 2020/4/2
     * @param enterpriseId
     * @param storeGroupId
     * @return 
    */
    ServiceResponse<Integer> refreshStoreIndex(Integer enterpriseId, Integer storeGroupId);

    /**
     * 批量添加门店到索引
     * @Title: bulkAddStoreToIndex
     * @Description: 批量添加门店到索引
     * @author zhiwj
     * @param enterpriseId
     * @param storeIdList
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Void> bulkAddStoreToIndex(Integer enterpriseId, List<Integer> storeIdList);

    /**
     * queryStoreFromES
     * @Title: queryStoreFromES
     * @Description: 查询es
     * @author zhiwj
     * @param jsonObject
     * @param pageNum
     * @param pageSize
     * @param sortJson
     * @param returnFields
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Void> queryStoreFromEs(JSONObject jsonObject, Integer pageNum, Integer pageSize, JSONObject sortJson, String returnFields);

    /**
     * 查询门店数量 from es
     * @Description: 查询门店数量 from es
     * @author taogs
     * @Date 13:37 2019/9/25
     * @param storeSearchDTO
     * @return
    */
    ServiceResponse<Long> queryStoreCountFromEs(StoreSearchDTO storeSearchDTO);

    /**
     * deleteStoreFromEs  删除门店 from es
     * @Title: deleteStoreFromEs
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> deleteStoreFromEs(Integer enterpriseId, Integer storeId);

    /**
     * 门店选择器参数转换
     * @Description: 门店选择器参数转换
     * @author taogs
     * @Date 11:26 2019/8/23
     * @param searchJson
     * @return
    */
    ServiceResponse<JSONObject> parseStoreSelectJson(String searchJson, Integer enterpriseId);

    /**
     * listStoreByStoreInfoId
     * @Title: listStoreByStoreInfoId
     * @Description:
     * @author zhiwj
     * @param storeInfoId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreDTO>>
     * @throws
     */
    ServiceResponse<List<StoreDTO>> listStoreByStoreInfoId(Integer storeInfoId);

    /**
     * listStore
     * @Title: listStore
     * @Description:
     * @author zhiwj
     * @param search
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreDTO>>
     * @throws
     */
    ServiceResponse<List<StoreDTO>> listStore(String search, Integer enterpriseId);


    /**
     * 小程序附近门店
     * @Description: 小程序附近门店
     * @author taogs
     * @Date 19:41 2019/12/4
     * @param wxSearchStoreSearchDTO
     * @return
    */
    ServiceResponse<Page<StoreDTO>> pageNearbyStore(WxSearchStoreSearchDTO wxSearchStoreSearchDTO);

    /**
     * 小程序卡券适用门店
     * @Description: 小程序卡券适用门店
     * @author taogs
     * @Date 17:22 2019/12/24
     * @param wxSearchStoreSearchDTO
     * @return
    */
    ServiceResponse<Page<StoreDTO>> pageCoupApplyStore(WxSearchStoreSearchDTO wxSearchStoreSearchDTO);


    /**
     * 小程序开卡门店
     * @Description: 小程序开卡门店
    * @author taogs
    * @Date 18:38 2019/12/24
    * @param wxSearchStoreSearchDTO
    * @return
    */
    ServiceResponse<Page<StoreDTO>> pageOpenCardStore(WxSearchStoreSearchDTO wxSearchStoreSearchDTO);

    /**
     * storeQrcodeCallBack
     * @Title: storeQrcodeCallBack
     * @Description:
     * @author zhiwj
     * @param param
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> storeQrcodeCallBack(String param);

    /**
     * 导入门店
     * @Title: importStoreForOperation
     * @Description:
     * @author zhiwj
     * @param processBatchStoreDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> importStoreForOperation(ProcessBatchStoreDTO processBatchStoreDTO);

    /**
     * 更新门店分数
     * @Title: updateScore
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeId
     * @param storeScore
     * @param evaluateCount
     * @return void
     * @throws
     */
    ServiceResponse<Void> updateScore(Integer enterpriseId, Integer storeId, double storeScore, Integer evaluateCount);

    /**
     *getStoreIdByStoreNameAndStoreCodeAndRegionId
     * @Title: getStoreIdByStoreNameAndStoreCodeAndRegionId
     * @Description:
     * @author zhiwj
     * @param storeCode
     * @param regionId
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Object>
     * @throws
     */
    ServiceResponse<Integer> getStoreIdByStoreNameAndStoreCodeAndRegionId(String storeCode, Integer regionId, Integer enterpriseId);

    /**
     * 查看商户下是否存在门店
     * @Title: checkExistStore
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Boolean>
     * @throws
     */
    ServiceResponse<Boolean> checkExistStore(Integer enterpriseId);

    
    /**
     * 从队列中读取门店和导购数据写入数据库
     * @Description: 从队列中读取门店和导购数据写入数据库
     * @author taogs
     * @Date 17:03 2020/3/19
     * @param params
     * @return 
    */
    ServiceResponse<Void> saveStoreAndClerkForBulkOrder(String params);

    
    /**
     * 刷新门店缓存
     * @Description: 刷新门店缓存
     * @author taogs
     * @Date 20:29 2020/3/25
     * @param enterpriseId
     * @param storeId
     * @return 
    */
    void refreshCache(Integer enterpriseId, Integer storeId);

    /**
     * 批量转移至门店列表
     * @Title: tranOverFlow
     * @Description: 批量转移至门店列表
     * @author zhiwj
     * @param enterpriseId
     * @param storeIds
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> tranOverFlow(Integer enterpriseId, String storeIds);

    /**
     * listStoreFromDb
     * @Title: listStoreFromDb
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeSearchDbDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreDTO>>
     * @throws
     */
    ServiceResponse<List<StoreDTO>> listStoreFromDb(Integer enterpriseId, StoreSearchDbDTO storeSearchDbDTO);

    /**
     * hasUnCompleteStatusStore 商户下不启用的门店里是否存在有未完善的门店
     * @Title: hasUnCompleteStatusStore
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Boolean> true:有存在未完善的门店，  false：所有门店都已完善
     * @throws
     */
    ServiceResponse<Boolean> hasUnCompleteStatusStore(Integer enterpriseId);
}
