package com.gic.store.service;

import java.util.List;
import java.util.Map;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreFieldDTO;

/**
 * 自定义字段
 * @ClassName: StoreFieldApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/7/31 10:50 AM 
 */
public interface StoreFieldApiService {

    /**
     * 新增
     * @Title: save 
     * @Description: 新增
      * @author guojuxing
     * @param storeFieldDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     * 主键
      */
    ServiceResponse<Integer> save(StoreFieldDTO storeFieldDTO);

    /**
     * 上下移动排序
     * @Title: upOrDown 
     * @Description: 
      * @author guojuxing 
     * @param storeFieldId
     * @param isUp  是否向上移动
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> upOrDown(int storeFieldId, boolean isUp, Integer enterpriseId);

    /**
     * 置顶、置底，自定义序号
     * @Title: setStoreFieldSort 
     * @Description: 置顶、置底，自定义序号
      * @author guojuxing
     * @param storeFieldId
     * @param setValue  排序目标值
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String>  返回日志操作对象
      */
    ServiceResponse<String> setStoreFieldSort(int storeFieldId, int setValue, Integer enterpriseId);

    /**
     * delete
     * @Title: delete 
     * @Description:
      * @author guojuxing
     * @param storeFieldId  主键ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer>  返回日志操作对象
      */
    ServiceResponse<String> delete(int storeFieldId);

    /**
     * 删除全部
     * @Title: deleteAll 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> deleteAll(int enterpriseId);

    /**
     * 批量删除
     * @Title: deleteBatch 
     * @Description:
      * @author guojuxing
     * @param storeFieldIds  主键集合
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer>  返回日志操作对象
      */
    ServiceResponse<String> deleteBatch(List<Integer> storeFieldIds);

    /**
     * 编辑保存
     * @Title: edit 
     * @Description:
      * @author guojuxing
     * @param storeFieldDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> edit(StoreFieldDTO storeFieldDTO);

    /**
     * 自定义字段分页查询列表数据
     * 查询所有的有权限的域下面的字段（即有权限的自定义字段）
     * @Title: listStoreField 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 商户ID
     * @param search code或者自定义名称名称
     * @param pageNum 第几页
     * @param pageSize  每页大小
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.StoreFieldDTO>> 
      */
    ServiceResponse<Page<StoreFieldDTO>> listStoreField(int enterpriseId, String search, Integer pageNum,
            Integer pageSize);

    /**
     * 分页查询
     * 查询所有的有权限的域下面的字段（即有权限的自定义字段）
     * @Title: listStoreField 
     * @Description:
      * @author guojuxing
     * @param storeFieldIdList
     * @param enterpriseId
     * @param search
     * @param pageNum
     * @param pageSize 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.StoreFieldDTO>> 
      */
    ServiceResponse<Page<StoreFieldDTO>> listStoreField(List<Integer> storeFieldIdList, int enterpriseId, String search, Integer pageNum,
                                                        Integer pageSize);
    /**
     * 自定义字段ID map 域IDs
     * 查询所有的有权限的域下面的字段（即有权限的自定义字段）
     * @Title: storeFieldIdMapRegionId 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.Integer,java.lang.String>> 
      */
    ServiceResponse<Map<Integer, String>> storeFieldIdMapRegionId(Integer enterpriseId);

    /**
     * 自定义字段查询列表数据
     * 查询所有的有权限的域下面的字段（即有权限的自定义字段）
     * @Title: listStoreField 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 商户ID
     * @param search  code或者自定义名称名称
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreFieldDTO>> 
      */
    ServiceResponse<List<StoreFieldDTO>> listStoreField(Integer enterpriseId, String search);

    /**
     * 主键查询记录
     * @Title: getStoreField 
     * @Description:
      * @author guojuxing
     * @param storeFieldId  主键ID
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreFieldDTO> 
      */
    ServiceResponse<StoreFieldDTO> getStoreField(int storeFieldId);

    /**
     * 置底排序
     * @Title: sortBottom 
     * @Description:
      * @author guojuxing
     * @param storeFieldId  主键ID
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> sortBottom(int storeFieldId, Integer enterpriseId);

    /**
     *  置顶排序
     * @Title: sortTopping 
     * @Description:
      * @author guojuxing
     * @param storeFieldId  主键ID
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> sortTopping(int storeFieldId, Integer enterpriseId);

    /**
     * 根据域查询数据
     * @Title: listStoreFieldByRegionId 
     * @Description: 
      * @author guojuxing
     * @param regionId  门店域ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreFieldDTO>> 
      */
    ServiceResponse<List<StoreFieldDTO>> listStoreFieldByRegionId(Integer regionId);

    /**
     * 授权自定义字段
     * @Title: authStoreField 
     * @Description:
      * @author guojuxing
     * @param fromEnterpriseId
     * @param toEnterpriseId
     * @param regionIdList 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> authStoreField(Integer fromEnterpriseId, Integer toEnterpriseId, List<Integer> regionIdList);

    /**
     * 查询商户的own字段（去除被授权的字段）
     * @Title: listOwnStoreField 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreFieldDTO>> 
      */
    ServiceResponse<List<StoreFieldDTO>> listOwnStoreField(Integer enterpriseId);

    /**
     * 设置ERP映射
     * @Title: setErpMapperCode 
     * @Description:
      * @author guojuxing
     * @param storeFieldId
     * @param code
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> setErpMapperCode(Integer storeFieldId, String code);

    /**
     * 解除ERP关联
     * @Title: unBindErpMapperCode 
     * @Description: 
      * @author guojuxing
     * @param storeFieldId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> unBindErpMapper(Integer storeFieldId);

}
