package com.gic.store.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreTmpDTO;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: StoreImportApiService
 * @Description:  门店导入
 * @author zhiwj
 * @date 2019/7/31 14:56
 */
public interface StoreImportApiService {
    /**
     * 判断当前企业下是否有正在导入的门店
     * @Title: hasImportingStore
     * @Description: 判断当前企业下是否有正在导入的门店
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Boolean>
     * @throws
     */
    ServiceResponse<Boolean> hasImportingStore(Integer enterpriseId);

    /**
     * 删除实体
     * @Title: delStoreTmp
     * @Description: 删除实体
     * @author zhiwj 
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws 
     */
    ServiceResponse<Void> delStoreTmp(Integer enterpriseId);

    /**
     * 导入门店
     * @Title: importDataToStore
     * @Description: 导入门店
     * @author zhiwj
     * @param enterpriseId
     * @param userId
     * @param uuId
     * @param isReplace 如果门店code重复，是否替换更新
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws 
     */
    ServiceResponse<String> importDataToStore(Integer enterpriseId, String userId, String uuId, boolean isReplace);

    /**
     * 保存
     * @Title: save
     * @Description: 保存
     * @author zhiwj 
     * @param bean
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws 
     */
    ServiceResponse<Void> save(StoreTmpDTO bean);
    
    /**
     * 查询列表
     * @Title: listStoreTmp
     * @Description: 查询列表
     * @author zhiwj
     * @param enterpriseId
     * @param isSuccess
     * @param currentPage
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page>
     * @throws 
     */
    ServiceResponse<Page> listStoreTmp(Integer enterpriseId, boolean isSuccess, Integer currentPage, Integer pageSize);
    
    /**
     * 校验storeCode
     * @Title: validateStoreCode
     * @Description: 校验storeCode
     * @author zhiwj 
     * @param enterpriseId
     * @param regionId
     * @param storeCode
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Boolean> ture：不通过
     * @throws 
     */
    ServiceResponse<Boolean> validateStoreCode(Integer enterpriseId, Integer regionId, String storeCode);
    
    /**
     * 门店分组是否存在
     * @Title: isExistStoreGroup
     * @Description: 门店分组是否存在
     * @author zhiwj 
     * @param enterpriseId
     * @param storeGroupName
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> isExistStoreGroup(Integer enterpriseId, String storeGroupName);
    
    /**
     * 所有不存在的门店品牌
     * @Title: listBrandByNotExist
     * @Description: 所有不存在的门店品牌
     * @author zhiwj 
     * @param enterpriseId
     * @param brands
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<java.lang.String>>
     * @throws 
     */
    ServiceResponse<List<String>> listBrandByNotExist(Integer enterpriseId, String brands);

    /**
     * 查询导入列表
     * @Title: listStoreTmp
     * @Description: 查询导入列表
     * @author zhiwj 
     * @param enterpriseId
     * @param isRight
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreTmpDTO>>
     * @throws 
     */
    ServiceResponse<List<StoreTmpDTO>> listStoreTmp(Integer enterpriseId, Boolean isRight);

    /**
     * 查询所有导入列表
     * @Title: listAllStoreTmp
     * @Description: 查询所有导入列表
     * @author zhiwj 
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreTmpDTO>>
     * @throws 
     */
    ServiceResponse<List<StoreTmpDTO>> listAllStoreTmp(Integer enterpriseId);

    /**
     * 查询所有导入列表
     * @Title: listError
     * @Description: 查询所有导入列表
     * @author zhiwj 
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<java.util.List<java.lang.String>>>
     * @throws 
     */
    ServiceResponse<List<List<String>>> listError(Integer enterpriseId);
    
    /**
     * 查询导入列表
     * @Title: pageError
     * @Description: 查询导入列表
     * @author zhiwj 
     * @param enterpriseId
     * @param currentPage
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page>
     * @throws 
     */
    ServiceResponse<Page> pageError(Integer enterpriseId, Integer currentPage, Integer pageSize);

    /**
     * 查询title
     * @Title: listImportTitle
     * @Description: 查询title
     * @author zhiwj 
     * @param enterpriseId
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<java.util.List<java.lang.String>>>
     * @throws 
     */
    ServiceResponse<List<List<String>>> listImportTitle(Integer enterpriseId, Integer regionId);

    /**
     * 查询field是否是date
     * @Title: fieldIsDateType
     * @Description: 查询field是否是date
     * @author zhiwj 
     * @param enterpriseId
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.Integer,java.lang.Boolean>>
     * @throws 
     */
    ServiceResponse<Map<Integer, Boolean>> fieldIsDateType(Integer enterpriseId, Integer regionId);

    /**
     * 验证storeName
     * @Title: validateStoreName
     * @Description: 验证storeName
     * @author zhiwj 
     * @param enterpriseId
     * @param storeName
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Boolean>
     * @throws 
     */
    ServiceResponse<Boolean> validateStoreName(Integer enterpriseId, Integer regionId, String storeName);
    
    /**
     * 查询导入状态
     * @Title: getStoreBatchProcess
     * @Description: 查询导入状态
     * @author zhiwj 
     * @param enterpriseId
     * @param uuId
     * @param currentPage
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.String,java.lang.Object>>
     * @throws 
     */
    ServiceResponse<Map<String, Object>> getStoreBatchProcess(Integer enterpriseId, String uuId, Integer currentPage, Integer pageSize);
}
