package com.gic.store.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.ClerkPositionDTO;
import com.gic.store.dto.open.ClerkDTO;
import com.gic.store.dto.open.StoreGroupDTO;
import com.gic.store.dto.open.StoreOpenDTO;

import java.util.List;

/**
 * 门店、导购对外API接口
 * @ClassName: StoreOpenApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2020/2/25 3:48 PM 
 */
public interface StoreOpenApiService {

    /**
     * 获取gic端和对外API之间门店自定义字段的匹配数据
     * @Title: getApiFieldJson 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 商户ID
     * @param regionId 门店域ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> getApiFieldJson(Integer enterpriseId, Integer regionId);

    /**
     * 查询门店详情
     * @Title: getStoreDetail 
     * @Description:
     * @author guojuxing
     * @param appId
     * @param enterpriseId 商户ID
     * @param storeCode 门店代码
     * @param regionId 门店域ID
     * @param storeInfoId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> getStoreDetail(String appId, Integer enterpriseId, String storeCode, Integer regionId, Integer storeInfoId);

    /***
     * 门店新增接口
     * @Title: addStore
     * @Description:
     * @author zhiwj
     * @param appId
     * @param enterpriseId
     * @param storeOpenDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> addStore(String appId, Integer enterpriseId, StoreOpenDTO storeOpenDTO);


    /**
     * 门店修改接口
     * @Title: editStore
     * @Description:
     * @author zhiwj
     * @param appId
     * @param enterpriseId
     * @param storeOpenDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> editStore(String appId, Integer enterpriseId, StoreOpenDTO storeOpenDTO);

    /**
     * pageStore
     * @Title: pageStore
     * @Description:
     * @author zhiwj
     * @param appId
     * @param enterpriseId
     * @param regionId
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> pageStore(String appId, Integer enterpriseId, Integer regionId, Integer pageNum, Integer pageSize);

    /**
     * getClerkDetail
     * @Title: getClerkDetail
     * @Description:   查看导购详情
     * @author taogs
     * @param appId
     * @param enterpriseId
     * @param regionId
     * @param storeCode
     * @param clerkCode
     * @param storeInfoId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> getClerkDetail(String appId, Integer enterpriseId, Integer regionId, String storeCode, String clerkCode, Integer storeInfoId);

    /**
     * addClerk
     * @Title: addClerk
     * @Description:
     * @author zhiwj
     * @param appId
     * @param enterpriseId
     * @param clerkInfo
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> addClerk(String appId, Integer enterpriseId, ClerkDTO clerkInfo);

    /**
     * editClerk
     * @Title: editClerk
     * @Description:
     * @author zhiwj
     * @param appId
     * @param enterpriseId
     * @param clerkInfo
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> editClerk(String appId, Integer enterpriseId, ClerkDTO clerkInfo);

    /**
     * 分页查询导购列表
     * @Title: pageClerk 
     * @Description:
     * @author guojuxing
     * @param appId 应用ID
     * @param enterpriseId 商户ID
     * @param storeCode 门店code 如果为空，则查询所有域下的
     * @param regionId 门店域ID
     * @param storeInfoId
     * @param pageNum 第几页
     * @param pageSize 每页多少条
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> pageClerk(String appId, Integer enterpriseId, String storeCode, Integer regionId, Integer storeInfoId, Integer pageNum, Integer pageSize);


    /**
     * listClerkPosition
     * @Title: listClerkPosition
     * @Description:   查询导购角色列表
     * @author taogs
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.ClerkPositionDTO>>
     * @throws
     */
    ServiceResponse<List<ClerkPositionDTO>> listClerkPosition(Integer enterpriseId);
    /**
     * listRegion 门店域列表
     * @Title: listRegion
     * @Description:
     * @author zhiwj
     * @param appId
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> listRegion(String appId, Integer enterpriseId);

    /**
     * listBrand 门店品牌列表
     * @Title: listBrand
     * @Description:
     * @author zhiwj
     * @param appId
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> listBrand(String appId, Integer enterpriseId);

    /**
     * listField 域下自定义属性列表查询
     * @Title: listField
     * @Description:
     * @author zhiwj
     * @param appId
     * @param enterpriseId
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> listField(String appId, Integer enterpriseId, Integer regionId);

    /**
     * 查询门店分组数据
     * @Title: listStoreGroup 
     * @Description:
     * @author guojuxing
     * @param appId
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> listStoreGroup(String appId, Integer enterpriseId);

    /**
     * 新增分组
     * @Title: saveStoreGroup 
     * @Description:
     * @author guojuxing
     * @param appId
     * @param enterpriseId
     * @param storeGroupInfo 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> saveStoreGroup(String appId, Integer enterpriseId, StoreGroupDTO storeGroupInfo);

    /**
     * 编辑分组
     * @Title: editStoreGroup 
     * @Description:
     * @author guojuxing
     * @param appId
     * @param enterpriseId
     * @param storeGroupId
     * @param storeGroupName  
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> editStoreGroup(String appId, Integer enterpriseId, Integer storeGroupId, String storeGroupName);

    /**
     * 删除分组
     * @Title: deleteStoreGroup 
     * @Description:
     * @author guojuxing
     * @param appId
     * @param enterpriseId
     * @param storeGroupId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> deleteStoreGroup(String appId, Integer enterpriseId, Integer storeGroupId);
}
