package com.gic.store.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreRegionDTO;

import java.util.List;

/**
 * @ClassName: StoreRegionApiService
 * @Description: 门店域
 * @author zhiwj
 * @date 2019/7/31 15:08
 */
public interface StoreRegionApiService {
    
    /**
     * 保存门店域
     * @Title: saveOrUpdateStoreRegion
     * @Description: 保存门店域
     * @author zhiwj
     * @param enterpriseId
     * @param regionCode
     * @param regionName
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveOrUpdateStoreRegion(Integer enterpriseId, String regionCode, String regionName, Integer regionId);

    /**
     * 分页查询门店域列表
     * @Title: listStoreRegion
     * @Description: 分页查询门店域列表
     * @author zhiwj 
     * @param enterpriseId
     * @param search 门店域code or 门店域名称
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.StoreRegionDTO>>
     * @throws 
     */
    ServiceResponse<Page<StoreRegionDTO>> listStoreRegion(Integer enterpriseId, String search, Integer pageNum, Integer pageSize);

    /**
     * 查询全部的门店域
     * @Title: listStoreRegion
     * @Description: 查询全部的门店域
     * @author zhiwj 
     * @param enterpriseId
     * @param search
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreRegionDTO>>
     * @throws 
     */
    ServiceResponse<List<StoreRegionDTO>> listStoreRegion(Integer enterpriseId, String search);

    /**
     * listStoreRegion
     * @Title: listStoreRegion
     * @Description: 
     * @author zhiwj 
     * @param enterpriseId
     * @param search
     * @param ownType
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreRegionDTO>>
     * @throws 
     */
    ServiceResponse<List<StoreRegionDTO>> listStoreRegion(Integer enterpriseId, String search, Integer ownType);

    /**
     * 查询自有的门店域
     * @Title: listOwnerStoreRegion
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreRegionDTO>>
     * @throws
     */
    ServiceResponse<List<StoreRegionDTO>> listOwnerStoreRegion(Integer enterpriseId);

    /**
     * 查询门店域详情
     * @Title: getStoreRegionByRegion
     * @Description: 查询门店域详情
     * @author zhiwj 
     * @param enterpriseId
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreRegionDTO>
     * @throws 
     */
    ServiceResponse<StoreRegionDTO> getStoreRegionByRegion(Integer enterpriseId, Integer regionId);

    /**
     * listStoreRegionByIds 通过域id查询域列表
     * @Title: listStoreRegionByIds
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeRegionIdList
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreRegionDTO>>
     * @throws
     */
    ServiceResponse<List<StoreRegionDTO>> listStoreRegionByIds(Integer enterpriseId, List<Integer> storeRegionIdList);
    /**
     * 授权门店域给其他商户
     * @Title: authorizeRegionToOther
     * @Description:
     * @author zhiwj
     * @param ownerEnterpriseId 原商户
     * @param unionEnterpriseId 被授权商户
     * @param storeRegionIdList
     * @param authType 1表示重新授权 0正常授权
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> authorizeRegionToOther(Integer ownerEnterpriseId, Integer unionEnterpriseId, List<Integer> storeRegionIdList, Integer authType);

    /**
     * 收回授权
     * @Title: unAuthorizeRegion
     * @Description:
     * @author zhiwj
     * @param toEnterpriseId
     * @param storeRegionId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Void> unAuthorizeRegion(Integer toEnterpriseId, Integer storeRegionId);

    /**
     * 查询门店域
     * @Title: getStoreRegionByRegionName
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param regionName
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreRegionDTO>
     * @throws
     */
    ServiceResponse<StoreRegionDTO> getStoreRegionByRegionName(Integer enterpriseId, String regionName);

    /**
     * 门店域和erp关联接口
     * @Title: 门店域和erp关联接口
     * @Description:
     * @author taogs
     * @param enterpriseId
     * @param storeRegionId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<String> relationErp(Integer enterpriseId, Integer storeRegionId);

    /**
     * delRegion
     * @Title: delRegion
     * @Description: 删除门店域
     * @author taogs
     * @param enterpriseId
     * @param regionId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> delRegion(Integer enterpriseId, Integer regionId);
}