package com.gic.store.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreStrategyDTO;

import java.util.List;

/**
 * 门店策略和门店启用策略
 * @ClassName: StoreStrategyApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/7/4 9:44 AM
 */
public interface StoreStrategyApiService {

    @interface SaveStoreStrategy {
    }

    /**
     * 新增 策略数据
     * @Title: saveStoreStrategy 
     * @Description:
      * @author guojuxing
     * @param storeStrategyDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveStoreStrategy(StoreStrategyDTO storeStrategyDTO);

    /**
     * 删除分组（单条）
     * @Title: deleteStoreStrategy 
     * @Description:
      * @author guojuxing
     * @param strategyId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<String> deleteStoreStrategy(int strategyId);

    @interface ModifyStoreStrategy {
    }

    /**
     * 修改策略
     * @Title: editStoreStrategy 
     * @Description:
      * @author guojuxing
     * @param storeStrategyDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> editStoreStrategy(StoreStrategyDTO storeStrategyDTO);

    /**
     * 根据主键ID获取数据
     * @Title: getStoreStrategyById 
     * @Description:
      * @author guojuxing
     * @param strategyId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreStrategyDTO> 
      */
    ServiceResponse<StoreStrategyDTO> getStoreStrategyById(int strategyId);

    /**
     * 置底排序
     * @Title: sortBottom 
     * @Description:
      * @author guojuxing
     * @param strategyId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<String> sortBottom(int strategyId);

    /**
     * 置顶排序
     * @Title: sortTopping 
     * @Description:
      * @author guojuxing
     * @param strategyId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<String> sortTopping(int strategyId);

    /**
     * 上移、下移
     * @Title: upOrDown 
     * @Description:
      * @author guojuxing
     * @param strategyId
     * @param isUp  是否向上移动排序
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<String> upOrDown(int strategyId, boolean isUp);

    /**
     * 设置排序值
     * @Title: setStoreStrategySort 
     * @Description:
      * @author guojuxing
     * @param strategyId
     * @param setValue 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
      */
    ServiceResponse<String> setStoreStrategySort(int strategyId, int setValue);

    /**
     * 分页查询分组策略数据
     * @Title: listStoreField 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param search 策略名称
     * @param strategyType 策略类型，1门店状态策略，   2门店分组策略
     * @param pageNum 第几页
     * @param pageSize  每页大小
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.StoreStrategyDTO>> 
      */
    ServiceResponse<Page<StoreStrategyDTO>> listStoreField(int enterpriseId, String search, int strategyType,
            Integer pageNum, Integer pageSize);

    /**
     * 设置门店分组策略权重
     * @Title: setStoreGroupStrategyWeight 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param weight 
     * @return  com.gic.api.base.commons.ServiceResponse 
      */
    ServiceResponse<Void> setStoreGroupStrategyWeight(Integer enterpriseId, int weight);

    /**
     * 门店启用策略：人为操作更改过门店启用状态后，门店资料更新不再执行启用状态策略
     * @Title: setStoreEnableStrategyWeight 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param weight 1：勾选 0：不勾选
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> setStoreStatusEnableStrategyWeight(Integer enterpriseId, int weight);

    /**
     * 获取门店分组策略权重
     * @Title: getStoreGroupStrategyWeight 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse 
      */
    ServiceResponse<Integer> getStoreGroupStrategyWeight(Integer enterpriseId);

    /**
     * 获取门店启用策略：人为操作更改过门店启用状态后，门店资料更新不再执行启用状态策略
     * @Title: getStoreStatusEnableStrategyWeight 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> getStoreStatusEnableStrategyWeight(Integer enterpriseId);

    /**
     * 重置门店策略
     * @Title: resetStoreGroupStrategy 
     * @Description:
      * @author guojuxing
     * @param regionIdList 门店域集合
     * @param storeStatusList  门店状态集合 启用的、不启用的，可以多选，所以是集合
     * @return  com.gic.api.base.commons.ServiceResponse 
      */
    ServiceResponse<String> resetStoreGroupStrategy(List<Integer> regionIdList, List<Integer> storeStatusList);

}
