package com.gic.store.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreBrandDTO;
import com.gic.store.dto.StoreColumnSet;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreWidgetDTO;

import java.util.List;

/**
 *
 * @ClassName: StoreWidgetApiService
 * @Description:
 * @author zhiwj
 * @date 2020-09-09 16:08
 */
public interface StoreWidgetApiService {

    /**
     * 保存门店选择器内容
     * @Title: 保存门店选择器内容
     * @Description:
     * @author zhiwj
     * @param storeWidgetDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveStoreWidget(StoreWidgetDTO storeWidgetDTO);

    /**
     * 保存权限门店选择器（管理员和联合商户专用）
     * @Title: 保存权限门店选择器（管理员和联合商户专用）
     * @Description:
     * @author zhiwj
     * @param storeWidgetDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveAuthStoreWidget(StoreWidgetDTO storeWidgetDTO);


    /**
     * getStoreWidget 获取门店选择器内容
     * @Title: getStoreWidget
     * @Description:
     * @author zhiwj
     * @param id
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreWidgetDTO>
     * @throws
     */
    ServiceResponse<StoreWidgetDTO> getStoreWidget(Integer id);

    /**
     * 更新选择器的权限内容
     * @Title: updateAuth
     * @Description:
     * @author zhiwj
     * @param storeWidgetDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> updateAuth(StoreWidgetDTO storeWidgetDTO);

    
    /**
     * 通过门店控件id查询门店
     * @Title: listStoreByStoreWidgetId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.StoreDTO>>
     * @throws
     */
    ServiceResponse<Page<StoreDTO>> listStoreByStoreWidgetId(Integer enterpriseId, Integer storeWidgetId, int pageNum, int pageSize);

    /**
     * listStoreIdByStoreWidgetId
     * @Title: listStoreIdByStoreWidgetId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<java.lang.Integer>>
     * @throws
     */
    ServiceResponse<Page<Integer>> listStoreIdByStoreWidgetId(Integer enterpriseId, Integer storeWidgetId, int pageNum, int pageSize);

    /**
     * listStoreInfoIdByStoreWidgetId
     * @Title: listStoreInfoIdByStoreWidgetId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<java.lang.Integer>>
     * @throws
     */
    ServiceResponse<Page<Integer>> listStoreInfoIdByStoreWidgetId(Integer enterpriseId, Integer storeWidgetId, int pageNum, int pageSize);

    /**
     * listStoreIdByStoreWidgetId
     * @Title: listStoreIdByStoreWidgetId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<java.lang.Integer>>
     * @throws
     */
    ServiceResponse<List<Integer>> listStoreIdByStoreWidgetId(Integer enterpriseId, Integer storeWidgetId);
    /**
     * listStoreInfoIdByStoreWidgetId
     * @Title: listStoreInfoIdByStoreWidgetId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<java.lang.Integer>>
     * @throws
     */
    ServiceResponse<List<Integer>> listStoreInfoIdByStoreWidgetId(Integer enterpriseId, Integer storeWidgetId);

    /**
     * listBrandIdByStoreWidgetId
     * @Title: listBrandIdByStoreWidgetId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreBrandDTO>>
     * @throws
     */
    ServiceResponse<List<StoreBrandDTO>> listBrandIdByStoreWidgetId(Integer enterpriseId, Integer storeWidgetId);

    /**
     * 判断门店是否在门店控件里
     * @Title: exist
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @param storeId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse exist(Integer enterpriseId, Integer storeWidgetId, Integer storeId);

    /**
     * exist 判断门店是否在门店控件里
     * @Title: exist
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @param storeId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse exist(String enterpriseId, Integer storeWidgetId, Integer storeId);

    /**
     * existByStoreInfoId  判断门店是否在门店控件里
     * @Title: existByStoreInfoId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeWidgetId
     * @param storeInfoId
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse existByStoreInfoId(Integer enterpriseId, Integer storeWidgetId, Integer storeInfoId);

    /**
     * getStoreColumnSet
     * @Title: getStoreColumnSet
     * @Description:
     * @author zhiwj
 * @param enterpriseId
 * @param storeWidgetId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreColumnSet>
     * @throws
     */
    ServiceResponse<StoreColumnSet> getStoreColumnSet(Integer enterpriseId, Integer storeWidgetId);


    /**
     * listStoreWidget
     * @Title: listStoreWidget
     * @Description:   根据门店控件id集合查询门店控件详情
     * @author taogs
     * @param storeWidgetIdList
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreWidgetDTO>>
     * @throws
     */
    ServiceResponse<List<StoreWidgetDTO>> listStoreWidget(List<Integer> storeWidgetIdList);
}
