package com.gic.store.utils.storegroup;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreGroupDTO;

/**
 * 门店分组查询树结构工具
 * @ClassName: ChangeListToTree 
 * @Description:  
 * @author guojuxing 
 * @date 2019/7/12 3:10 PM
 */
public class ChangeListToTree {

    /**
     * 把集合数据成树结构
     * @Title: changeListToTree 
     * @Description:
      * @author guojuxing
     * @param storeGroupId
     * @param list 
     * @return  java.util.List<java.util.Map<java.lang.String,java.lang.Object>> 
      */
    public static List<Map<String, Object>> changeListToTree(int storeGroupId, List<StoreGroupDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (StoreGroupDTO t : list) {
                if (storeGroupId == t.getParentStoreGroupId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("parentGroupId", t.getParentStoreGroupId());
                    data.put("storeGroupId", t.getStoreGroupId());
                    data.put("storeGroupName", t.getStoreGroupName());
                    data.put("sort", t.getSort());
                    data.put("isDefault", t.getIsDefault());
                    data.put("groupLevel", t.getGroupLevel());
                    List<Map<String, Object>> children = changeListToTree(t.getStoreGroupId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }

    /**
     * 不包括全部门店的数据的树结构数据
     * @Title: changeListToTreeNotIncludeAllStore 
     * @Description:
      * @author guojuxing
     * @param storeGroupId
     * @param list 
     * @return  java.util.List<java.util.Map<java.lang.String,java.lang.Object>> 
      */
    public static List<Map<String, Object>> changeListToTreeNotIncludeAllStore(int storeGroupId,
            List<StoreGroupDTO> list) {
        List<Map<String, Object>> result = changeListToTree(storeGroupId, list);
        if (result.size() > 0) {
            for (Map<String, Object> obj : result) {
                if ((int) obj.get("groupLevel") == StoreGroupConstant.ALL_STORE_LEVEL) {
                    return (List<Map<String, Object>>) obj.get("children");
                }
            }
        }
        return result;
    }

}
