package com.gic.store.dao.mapper;

import com.gic.store.entity.TabClerk;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @author zhiwj
 * @ClassName: TabClerkMapper
 * @Description:
 * @date 2020-09-09 18:35
 */
public interface TabClerkMapper {
    /**
     * 根据主键删除
     *
     * @param clerkId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(String clerkId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabClerk record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabClerk record);

    /**
     * 根据主键查询
     *
     * @param clerkId 主键
     * @return 实体对象
     */
    TabClerk selectByPrimaryKey(Integer clerkId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabClerk record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabClerk record);

    /**
     * countByClerkCode
     *
     * @param storeId
     * @param clerkCode
     * @param clerkId
     * @return java.lang.Integer
     * @throws
     * @Title: countByClerkCode
     * @Description:
     * @author zhiwj
     */
    Integer countByClerkCode(@Param("storeId") Integer storeId, @Param("clerkCode") String clerkCode, @Param("clerkId") Integer clerkId);

    /**
     * countByPhoneNumber
     *
     * @param storeId
     * @param phoneNumber
     * @param clerkId
     * @return java.lang.Integer
     * @throws
     * @Title: countByPhoneNumber
     * @Description:
     * @author zhiwj
     */
    Integer countByPhoneNumber(@Param("storeId") Integer storeId, @Param("phoneNumber") String phoneNumber, @Param("clerkId") Integer clerkId);

    /**
     * listClerkByStoreInfoId
     *
     * @param storeClerkIds
     * @param storeInfoIds
     * @param search
     * @param clerkType
     * @param status
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     * @Title: listClerkByStoreInfoId
     * @Description:
     * @author zhiwj
     */
    List<TabClerk> listClerkByStoreInfoId(@Param("storeClerkIds") List<Integer> storeClerkIds,
                                          @Param("storeInfoIds") String storeInfoIds,
                                          @Param("search") String search,
                                          @Param("clerkType") Integer clerkType,
                                          @Param("status") Integer status);

    /**
     * 查询导购
     * @param storeClerkIds
     * @param storeInfoIds
     * @param search
     * @param clerkType
     * @param status
     * @param clerkIdList
     * @return
     */
    List<TabClerk> listClerkByStoreInfoIdAndClerkId(@Param("storeClerkIds") List<Integer> storeClerkIds,
                                          @Param("storeInfoIds") String storeInfoIds,
                                          @Param("search") String search,
                                          @Param("clerkType") Integer clerkType,
                                          @Param("status") Integer status,
                                          @Param("clerkIdList") List<Integer> clerkIdList);

    /**
     * getTotalClerk
     *
     * @param storeClerkIds
     * @param search
     * @param status
     * @return java.lang.Integer
     * @throws
     * @Title: getTotalClerk
     * @Description:
     * @author zhiwj
     */
    Integer getTotalClerk(@Param("storeClerkIds") List<Integer> storeClerkIds,
                          @Param("search") String search,
                          @Param("status") Integer status,
                          @Param("clerkIdList") List<Integer> clerkIdList);

    /**
     * getStoreInfoIdsBySearch
     *
     * @param enterpriseId
     * @param search
     * @return java.util.List<java.lang.Integer>
     * @throws
     * @Title: getStoreInfoIdsBySearch
     * @Description:
     * @author zhiwj
     */
    List<Integer> getStoreInfoIdsBySearch(@Param("enterpriseId") Integer enterpriseId, @Param("search") String search);

    /**
     * updateClerkStatus
     *
     * @param enterpriseId
     * @param clerkIds
     * @param status
     * @return java.lang.Integer
     * @throws
     * @Title: updateClerkStatus
     * @Description:
     * @author zhiwj
     */
    Integer updateClerkStatus(@Param("enterpriseId") Integer enterpriseId, @Param("clerkIds") List<Integer> clerkIds, @Param("status") Integer status);

    /**
     * getBySelective
     *
     * @param tabClerk
     * @return com.gic.store.entity.TabClerk
     * @throws
     * @Title: getBySelective
     * @Description:
     * @author zhiwj
     */
    TabClerk getBySelective(TabClerk tabClerk);

    /**
     * listClerkByIds
     *
     * @param clerkIdList
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     * @Title: listClerkByIds
     * @Description:
     * @author zhiwj
     */
    List<TabClerk> listClerkByIds(@Param("ids") List<Integer> clerkIdList);

    /**
     * getClerkByCode
     *
     * @param clerkCode
     * @param storeInfoId
     * @return com.gic.store.entity.TabClerk
     * @throws
     * @Title: getClerkByCode
     * @Description:
     * @author zhiwj
     */
    TabClerk getClerkByCode(@Param("clerkCode") String clerkCode, @Param("storeInfoId") Integer storeInfoId);

    /**
     * getClerkLeaderByStoreInfoId
     *
     * @param storeInfoId
     * @return com.gic.store.entity.TabClerk
     * @throws
     * @Title: getClerkLeaderByStoreInfoId
     * @Description:
     * @author zhiwj
     */
    TabClerk getClerkLeaderByStoreInfoId(@Param("storeInfoId") Integer storeInfoId);

    @MapKey("clerkPositionId")
    /**
     * countMapByPosition
     * @Title: countMapByPosition
     * @Description:
     * @author zhiwj
     * @param  enterpriseId
     * @param positionList
     * @return java.util.Map<java.lang.Integer, java.util.Map < java.lang.String, java.lang.Object>>
     * @throws
     */
    Map<Integer, Map<String, Object>> countMapByPosition(@Param("enterpriseId") Integer enterpriseId, @Param("ids") List<Integer> positionList);

    /**
     * updatePosition
     *
     * @param enterpriseId
     * @param clerkPositionId
     * @param positionName
     * @return void
     * @throws
     * @Title: updatePosition
     * @Description:
     * @author zhiwj
     */
    void updatePosition(@Param("enterpriseId") Integer enterpriseId,@Param("positionId") Integer clerkPositionId, @Param("positionName") String positionName);

    /**
     * listHaobanClerk
     *
     * @param enterpriseIdList
     * @param keyword
     * @param storeInfoIdList
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     * @Title: listHaobanClerk
     * @Description:
     * @author zhiwj
     */
    List<TabClerk> listHaobanClerk(@Param("enterpriseIdList") List<Integer> enterpriseIdList, @Param("keyword") String keyword, @Param("storeInfoIdList") List<Integer> storeInfoIdList);

    /**
     * listClerkByPositionId
     *
     * @param enterpriseId
     * @param positionId
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     * @Title: listClerkByPositionId
     * @Description:
     * @author zhiwj
     */
    List<TabClerk> listClerkByPositionId(@Param("enterpriseId") Integer enterpriseId, @Param("positionId") Integer positionId);

    /**
     * listAllClerkId
     *
     * @param enterpriseId
     * @return java.util.List<java.lang.Integer>
     * @throws
     * @Title: listAllClerkId
     * @Description:
     * @author zhiwj
     */
    List<Integer> listAllClerkId(@Param("enterpriseId") Integer enterpriseId);

    /**
     * listStoreInfoIdByStoreInfoId
     *
     * @param storeInfoIdList
     * @return java.util.List<java.lang.Integer>
     * @throws
     * @Title: listStoreInfoIdByStoreInfoId
     * @Description:
     * @author zhiwj
     */
    List<Integer> listStoreInfoIdByStoreInfoId(@Param("ids") List<Integer> storeInfoIdList);

    /**
     * listClerkIdByStoreInfoId
     *
     * @param storeInfoIdList
     * @return java.util.List<java.lang.Integer>
     * @throws
     * @Title: listClerkIdByStoreInfoId
     * @Description:
     * @author taogs
     */
    List<TabClerk> listClerkByStoreInfoIdList(@Param("ids") List<Integer> storeInfoIdList);
}