package com.gic.store.dao.mapper;

import com.gic.store.entity.TabStoreBrand;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 
 * @ClassName: TabStoreBrandMapper
 * @Description:  
 * @author zhiwj
 * @date 2020-09-09 18:36
 */
public interface TabStoreBrandMapper {
    /**
     * 根据主键删除
     *
     * @param storeBrandId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer storeBrandId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabStoreBrand record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabStoreBrand record);

    /**
     * 根据主键查询
     *
     * @param storeBrandId 主键
     * @return 实体对象
     */
    TabStoreBrand selectByPrimaryKey(Integer storeBrandId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabStoreBrand record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabStoreBrand record);

    /**
     * 查询符合条件的个数； 除id所有值取等于计算， id取不等于计算
     * @param enterpriseId
     * @param storeBrandCode
     * @param storeBrandName
     * @param storeBrandId
     * @return
     */
    int countBySelective(@Param("enterpriseId") Integer enterpriseId, @Param("storeBrandCode") String storeBrandCode, @Param("storeBrandName") String storeBrandName, @Param("storeBrandId") Integer storeBrandId);

    /**
     * 查列表
     * @param enterpriseId
     * @param search
     * @return
     */
    List<TabStoreBrand> listStoreBrand(@Param("enterpriseId") Integer enterpriseId, @Param("search") String search);

    /**
     * 查询比传入sort小的sort中最大的
     * @param enterpriseId
     * @param sort
     * @return
     */
    TabStoreBrand getPrevSort(@Param("enterpriseId") Integer enterpriseId, @Param("sort") Double sort);

    /**
     * 查询比传入sort大的sort中最小的
     * @param enterpriseId
     * @param sort
     * @return
     */
    TabStoreBrand getNextSort(@Param("enterpriseId") Integer enterpriseId, @Param("sort") Double sort);

    /**
     * 查询几条连续的记录
     * @param enterpriseId
     * @param start 开始的位置
     * @param size  查询几条条
     * @param storeBrandId 要除外的id
     * @return
     */
    List<TabStoreBrand> listTwoElementExceptId(@Param("enterpriseId") Integer enterpriseId, @Param("start") int start, @Param("size") int size, @Param("storeBrandId") Integer storeBrandId);

    /**
     * 批量逻辑删除
     * @param ids
     * @return
     */
    Integer deleteByIds(@Param("ids") List<Integer> ids);

    /**
     * listStoreBrandByIds
     * @Title: listStoreBrandByIds
     * @Description:
     * @author zhiwj
     * @param idList
     * @return java.util.List<com.gic.store.entity.TabStoreBrand>
     * @throws
     */
    List<TabStoreBrand> listStoreBrandByIds(@Param("ids") List<Integer> idList);

    /**
     * 查询品牌
     * @param brandIdList
     * @return
     */
    List<TabStoreBrand> listAllStoreBrandInfoByIds(@Param("ids") List<Integer> brandIdList);

    /**
     * getByStoreBrandName
     * @Title: getByStoreBrandName
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandName
     * @return com.gic.store.entity.TabStoreBrand
     * @throws
     */
    TabStoreBrand getByStoreBrandName(@Param("enterpriseId") Integer enterpriseId, @Param("storeBrandName") String storeBrandName);


}