package com.gic.store.dao.mapper;

import com.gic.store.entity.TabStoreBrandRef;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface TabStoreBrandRefMapper {
    /**
     * 根据主键删除
     *
     * @param id 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer id);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabStoreBrandRef record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabStoreBrandRef record);

    /**
     * 根据主键查询
     *
     * @param id 主键
     * @return 实体对象
     */
    TabStoreBrandRef selectByPrimaryKey(Integer id);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabStoreBrandRef record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabStoreBrandRef record);

    /**
     * 查询最小的排序号
     * @param enterpriseId
     * @return
     */
    Double getMinSort(@Param("enterpriseId") Integer enterpriseId);

    /**
     * 查询最大的排序号
     * @param enterpriseId
     * @return
     */
    Double getMaxSort(@Param("enterpriseId") Integer enterpriseId);

    /**
     * selectByEidAndBrandId
     * @Title: selectByEidAndBrandId
     * @Description:
     * @author taogs
     * @param enterpriseId
     * @param storeBrandId
     * @return com.gic.store.entity.TabStoreBrandRef
     * @throws
     */
    TabStoreBrandRef selectByEidAndBrandId(@Param("enterpriseId") Integer enterpriseId, @Param("storeBrandId") Integer storeBrandId);

    /**
     * selectByEidAndBrandIdList 
     * @Title: selectByEidAndBrandIdList
     * @Description: 
     * @author taogs 
 * @param enterpriseId
 * @param storeBrandIdList
     * @return java.util.List<com.gic.store.entity.TabStoreBrandRef>
     * @throws 
     */
    List<TabStoreBrandRef> selectByEidAndBrandIdList(@Param("enterpriseId") Integer enterpriseId, @Param("storeBrandIdList") List<Integer> storeBrandIdList);

    /**
     * updateSort
     * @Title: updateSort
     * @Description:  更新排序
     * @author taogs
     * @param enterpriseId
     * @param storeBrandId
     * @param sort
     * @return java.lang.Integer
     * @throws
     */
    Integer updateSort(@Param("enterpriseId") Integer enterpriseId, @Param("storeBrandId") Integer storeBrandId, @Param("sort") Double sort);

    Integer insertList(@Param("list") List<TabStoreBrandRef> list);

    /**
     * cancelBrandAuth
     * @Title: cancelBrandAuth
     * @Description:
     * @author taogs
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return int
     * @throws
     */
    int cancelBrandAuth(@Param("ownEnterpriseId") Integer ownEnterpriseId, @Param("unionEnterpriseId") Integer unionEnterpriseId);
}