package com.gic.store.dao.mapper;

import com.gic.store.entity.TabStoreGroup;
import com.github.pagehelper.Page;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 
 * @ClassName: TabStoreGroupMapper
 * @Description:  
 * @author zhiwj
 * @date 2020-09-09 18:37
 */
public interface TabStoreGroupMapper {
    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 主键ID
     */
    int insert(TabStoreGroup record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabStoreGroup record);

    /**
     * 根据主键查询
     *
     * @param storeGroupId 主键
     * @return 实体对象
     */
    TabStoreGroup selectByPrimaryKey(Integer storeGroupId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabStoreGroup record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabStoreGroup record);

    /**
     * 查询分组列表
     * @param record
     * @return
     */
    List<TabStoreGroup> listStoreGroup(TabStoreGroup record);

    /**
     * 查询最大排序值
     * @param enterpriseId
     * @param groupLevel
     * @return
     */
    int selectStoreGroupMaxSort(@Param("enterpriseId") Integer enterpriseId, @Param("groupLevel") Integer groupLevel);

    /**
     * 批量更新分组所有子集的分组链数据，当分组更新上级分组的时候
     * @param storeGroupId
     * @param fromStoreGroupId 分组链需要被替换的字符串
     * @param toStoreGroupId 分组链替换的字符串数据
     * @return
     */
    int updateStoreGroupChainByStoreGroupId(@Param("storeGroupId") Integer storeGroupId,
                                            @Param("fromStoreGroupId") Integer fromStoreGroupId,
                                            @Param("toStoreGroupId") Integer toStoreGroupId);



    /**
     * 获取大于该序号的，最小序号记录
     * @param enterpriseId
     * @param groupLevel
     * @param sort
     * @return
     */
    TabStoreGroup getStoreGroupWhenMinSort(@Param("enterpriseId") Integer enterpriseId, @Param("groupLevel") Integer groupLevel, @Param("sort") Integer sort);

    /**
     * 获取小于该序号的，最大序号记录
     * @param enterpriseId
     * @param groupLevel
     * @param sort
     * @return
     */
    TabStoreGroup getStoreGroupWhenMaxSort(@Param("enterpriseId") Integer enterpriseId, @Param("groupLevel") Integer groupLevel, @Param("sort") Integer sort);

    /**
     * 查询所有门店
     * @param enterpriseId
     * @param groupLevel
     * @return
     */
    TabStoreGroup selectAllStoreGroup(@Param("enterpriseId") Integer enterpriseId, @Param("groupLevel") Integer groupLevel);

    /**
     * 根据父级ID，统计下面是否有子节点
     * @param parentId
     * @return
     */
    int countByParentId(@Param("parentId") Integer parentId);

    /**
     * isRepeatName
     * @Title: isRepeatName
     * @Description:
     * @author zhiwj
     * @param storeGroupName
     * @param storeGroupId
     * @param enterpriseId
     * @return int
     * @throws
     */
    int isRepeatName(@Param("storeGroupName") String storeGroupName, @Param("storeGroupId") Integer storeGroupId, @Param("enterpriseId") Integer enterpriseId);

    /**
     * getStoreGroupByName
     * @Title: getStoreGroupByName
     * @Description:
     * @author zhiwj
     * @param storeGroupName
     * @param enterpriseId
     * @return com.gic.store.entity.TabStoreGroup
     * @throws
     */
    TabStoreGroup getStoreGroupByName(@Param("storeGroupName") String storeGroupName, @Param("enterpriseId") Integer enterpriseId);


    /**
     * 查询未分组门店 分组数据
     * @param enterpriseId
     * @return
     */
    TabStoreGroup selectUnGroupedStore(@Param("enterpriseId") Integer enterpriseId);

    /**
     * listStoreGroupByIds
     * @Title: listStoreGroupByIds
     * @Description:
     * @author zhiwj
     * @param storeGroupIdList
     * @param enterpriseId
     * @param level
     * @return java.util.List<com.gic.store.entity.TabStoreGroup>
     * @throws
     */
    List<TabStoreGroup> listStoreGroupByIds(@Param("storeGroupIdList") List<Integer> storeGroupIdList,
                                            @Param("enterpriseId") Integer enterpriseId,
                                            @Param("level") Integer level);

    /**
     * pageStoreGroupByIds
     * @Title: pageStoreGroupByIds
     * @Description:
     * @author zhiwj
     * @param storeGroupIdList
     * @param enterpriseId
     * @param parentId
     * @param search
     * @return com.github.pagehelper.Page<com.gic.store.entity.TabStoreGroup>
     * @throws
     */
    Page<TabStoreGroup> pageStoreGroupByIds(@Param("storeGroupIdList") List<Integer> storeGroupIdList,
                                            @Param("enterpriseId") Integer enterpriseId,
                                            @Param("parentId") Integer parentId,
                                            @Param("search") String search);
}