package com.gic.store.dao.mapper;

import com.gic.store.entity.TabStoreStrategy;
import org.apache.ibatis.annotations.Param;

import java.util.List;
/**
 * 
 * @ClassName: TabStoreStrategyMapper
 * @Description:  
 * @author zhiwj
 * @date 2020-09-09 18:38
 */
public interface TabStoreStrategyMapper {
    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabStoreStrategy record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabStoreStrategy record);

    /**
     * 根据主键查询
     *
     * @param strategyId 主键
     * @return 实体对象
     */
    TabStoreStrategy selectByPrimaryKey(Integer strategyId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabStoreStrategy record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabStoreStrategy record);

    /**
     * countRepeatStrategyName 
     * @Title: countRepeatStrategyName
     * @Description: 
     * @author zhiwj 
 * @param record
     * @return java.lang.Integer
     * @throws 
     */
    Integer countRepeatStrategyName(TabStoreStrategy record);

    /**
     * 获取下一个排序值
     * @param enterpriseId
     * @param sort
     * @param strategyType
     * @return
     */
    TabStoreStrategy selectNextByEnterpriseIdAndSort(@Param("enterpriseId") Integer enterpriseId, @Param("sort") Double sort,
                                                     @Param("strategyType") Integer strategyType);

    /**
     * 获取上一个排序值
     * @param enterpriseId
     * @param sort
     * @param strategyType
     * @return
     */
    TabStoreStrategy selectPrevByStoreFieldIdAndSort(@Param("enterpriseId") Integer enterpriseId, @Param("sort") Double sort,
                                                     @Param("strategyType") Integer strategyType);

    /**
     * 查询某字段最小排序值
     * @param enterpriseId
     * @param strategyType
     * @return
     */
    Double getMinSort(@Param("enterpriseId") Integer enterpriseId, @Param("strategyType") Integer strategyType);

    /**
     /* *查询某字段最大排序值
     *
     * @param enterpriseId
     * @param strategyType
     * @return
     */
    Double getMaxSort(@Param("enterpriseId") Integer enterpriseId, @Param("strategyType") Integer strategyType);

    /**
     * 查询几条连续的记录
     * @param strategyId
     * @param start
     * @param size
     * @param enterpriseId
     * @param strategyType
     * @return
     */
    List<TabStoreStrategy> listTwoElementExceptId(@Param("strategyId") Integer strategyId, @Param("start") int start,
                                               @Param("size") int size, @Param("enterpriseId") Integer enterpriseId,
                                                  @Param("strategyType") Integer strategyType);


    /**
     * 分页查询
     * @param enterpriseId
     * @param search
     * @param strategyType
     * @return
     */
    List<TabStoreStrategy> listStoreStrategy(@Param("enterpriseId") Integer enterpriseId, @Param("search") String search,
                                             @Param("strategyType") Integer strategyType);
}