package com.gic.store.service;

import com.gic.store.dto.ClerkDTO;
import com.gic.store.dto.ClerkSearchDTO;
import com.gic.store.entity.TabClerk;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: ClerkService
 * @Description: 导购service
 * @author zhiwj
 * @date 2019/7/31 11:16
 */
public interface ClerkService {
    /**
     * 导购代码是否重复
     * @Title: isRepeatByClerkCode
     * @Description: 导购代码是否重复
     * @author zhiwj
     * @param storeInfoId 门店id
     * @param clerkCode 导购代码
     * @param clerkId 导购id 如果传了，会剔除这个id去判断
     * @return boolean true:重复, false:不重复
     * @throws
     */
    boolean isRepeatByClerkCode(Integer storeInfoId, String clerkCode, Integer clerkId);

    /**
     * 导购手机号是否重复
     * @Title: isRepeatByClerkCode
     * @Description: 导购手机号是否重复
     * @author zhiwj
     * @param storeInfoId 门店id
     * @param phoneNumber 导购手机号
     * @param clerkId 导购id 如果传了，会剔除这个id去判断
     * @param nationCode 手机号区号
     * @return boolean true:重复, false:不重复
     * @throws
     */
    boolean isRepeatByPhoneNumber(Integer storeInfoId, String phoneNumber, Integer clerkId, String nationCode);

    /**
     * 导购名称不能重复
     * @Title: isRepeatByClerkName
     * @Description: 
     * @author zhiwj 
     * @param storeInfoId
     * @param clerkName
     * @param clerkId
     * @return boolean
     * @throws 
     */
    boolean isRepeatByClerkName(Integer storeInfoId, String clerkName, Integer clerkId);
    /**
     * 保存导购
     * @Title: saveByOwner
     * @Description: 保存导购
     * @author zhiwj 
     * @param clerkDTO clerk实体
     * @return java.lang.Integer 数据库影响行数
     * @throws
     */
    Integer save(ClerkDTO clerkDTO);
    
    /**
     * 编辑导购
     * @Title: update
     * @Description: 编辑导购
     * @author zhiwj 
     * @param clerkDTO clerk实体
     * @return java.lang.Integer 数据库影响行数
     * @throws 
     */
    Integer update(ClerkDTO clerkDTO);

    /**
     * update
     * @Title: update
     * @Description:
     * @author zhiwj
     * @param clerk
     * @return java.lang.Integer
     * @throws
     */
    Integer update(TabClerk clerk);

    /**
     * 查询下导购列表
     * @Title: listClerkByStoreInfoId
     * @Description: 查询下导购列表
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param storeInfoIds 门店ids
     * @param search 查询条件:clerkName or clerkCode
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws 
     */
    List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, List<Integer> storeInfoIds, String search);

    /**
     * listClerkByStoreInfoId
     * @Title: listClerkByStoreInfoId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeInfoIds
     * @param search
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     */
    List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, String storeInfoIds, String search);
    /**
     * listClerkByStoreInfoId
     * @Title: listClerkByStoreInfoId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeInfoIds
     * @param search
     * @param clerkType
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     */
    List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, List<Integer> storeInfoIds, String search, Integer clerkType);

    /**
     * listClerkByStoreInfoId
     * @param enterpriseId
     * @param storeInfoIds
     * @param search
     * @return
     */
    List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, List<Integer> storeInfoIds, ClerkSearchDTO search);

    /**
     * listClerkByStoreInfoId
     * @Title: listClerkByStoreInfoId
     * @Description: 
     * @author taogs 
 * @param enterpriseId
 * @param storeInfoIds
 * @param status
 * @param search
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws 
     */
    List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, List<Integer> storeInfoIds, Integer status, String search);
    /**
     * getTotalClerk
     * @Title: getTotalClerk
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeInfoIds
     * @param search
     * @return java.lang.Integer
     * @throws
     */
    Integer getTotalClerk(Integer enterpriseId, List<Integer> storeInfoIds, ClerkSearchDTO search);
    /**
     * 查询符合条件的门店id
     * @Title: getStoreInfoIdsBySearch
     * @Description: 查询符合条件的门店id
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param search 查询条件:clerkName or clerkCode
     * @return java.util.List<java.lang.Integer>
     * @throws 
     */
    List<Integer> getStoreInfoIdsBySearch(Integer enterpriseId, String search);

    /**
     * 批量修改导购状态
     * @Title: updateClerkStatus
     * @Description: 批量修改导购状态
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param clerkIdList 导购ids
     * @param status 状态
     * @return java.lang.Integer
     * @throws 
     */
    Integer updateClerkStatus(Integer enterpriseId, List<Integer> clerkIdList, Integer status);

    /**
     * listClerkByIds
     * @Title: listClerkByIds
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param clerkIds
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     */
    List<TabClerk> listClerkByIds(Integer enterpriseId, String clerkIds);

    /**
     * listClerkByIds
     * @Title: listClerkByIds
     * @Description:
     * @author zhiwj
     * @param clerkIdList
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     */
    List<TabClerk> listClerkByIds(List<Integer> clerkIdList);

    /**
     * getById
     * @Title: getById
     * @Description:
     * @author zhiwj
     * @param clerkId
     * @return com.gic.store.entity.TabClerk
     * @throws
     */
    TabClerk getById(Integer clerkId);

    /**
     * listClerkByStoreInfoId
     * @Title: listClerkByStoreInfoId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param search
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     */
    List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, String search);

    /**
     * getClerkByCode
     * @Title: getClerkByCode
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param clerkCode
     * @param storeInfoId
     * @return com.gic.store.entity.TabClerk
     * @throws
     */
    TabClerk getClerkByCode(Integer enterpriseId, String clerkCode, Integer storeInfoId);

    /**
     * getClerkLeaderByStoreInfoId
     * @Title: getClerkLeaderByStoreInfoId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeInfoId
     * @return com.gic.store.entity.TabClerk
     * @throws
     */
    TabClerk getClerkLeaderByStoreInfoId(Integer enterpriseId, Integer storeInfoId);

    /**
     * countMapByPosition
     * @Title: countMapByPosition
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param positionList
     * @return java.util.Map<java.lang.Integer,java.lang.Integer>
     * @throws
     */
    Map<Integer, Integer> countMapByPosition(Integer enterpriseId, List<Integer> positionList);

    /**
     * updatePosition
     * @Title: updatePosition
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param clerkPositionId
     * @param positionName
     * @return void
     * @throws
     */
    void updatePosition(Integer enterpriseId, Integer clerkPositionId, String positionName);

    /**
     * listHaobanClerk
     * @Title: listHaobanClerk
     * @Description:
     * @author zhiwj
     * @param enterpriseIdList
     * @param keyword
     * @param storeInfoIdList
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     */
    List<TabClerk> listHaobanClerk(List<Integer> enterpriseIdList, String keyword, List<Integer> storeInfoIdList);

    /**
     * listClerkByPositionId
     * @Title: listClerkByPositionId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param positionId
     * @return java.util.List<com.gic.store.entity.TabClerk>
     * @throws
     */
    List<TabClerk> listClerkByPositionId(Integer enterpriseId, Integer positionId);

    /**
     * listAllClerkId
     * @Title: listAllClerkId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return java.util.List<java.lang.Integer>
     * @throws
     */
    List<Integer> listAllClerkId(Integer enterpriseId);

    /**
     * listStoreInfoIdByStoreInfoId 
     * @Title: listStoreInfoIdByStoreInfoId
     * @Description: 
     * @author zhiwj 
     * @param enterpriseId
     * @param storeInfoIdList
     * @return java.util.List<java.lang.Integer>
     * @throws 
     */
    List<Integer> listStoreInfoIdByStoreInfoId(Integer enterpriseId, List<Integer> storeInfoIdList);

    /**
     * listStoreInfoIdByStoreInfoId
     * @Title: listStoreInfoIdByStoreInfoId
     * @Description: 查询导购list
     * @author taogs
     * @param storeInfoIdList
     * @return java.util.List<java.lang.Integer>
     * @throws
     */
    List<TabClerk> listClerkByStoreInfoId(List<Integer> storeInfoIdList);
}
