package com.gic.store.service;

import com.gic.store.dto.StoreBrandDTO;
import com.gic.store.entity.TabStoreBrand;
import com.github.pagehelper.Page;

import java.util.List;

/**
 * @author zhiwj
 * @date 2019/6/26
 */
public interface StoreBrandService {
    /**
     * save
     * @Title: save
     * @Description:
     * @author zhiwj
     * @param storeBrandDTO
     * @return java.lang.Integer
     * @throws
     */
    Integer save(StoreBrandDTO storeBrandDTO);

    /**
     * saveStoreBrandRef
     * @Title: saveStoreBrandRef
     * @Description: 保存品牌关联表
     * @author taogs
     * @param enterpriseId
     * @param storeBrandId
     * @param ownEnterpriseId
     * @param type
     * @return void
     * @throws
     */
    void saveStoreBrandRef(Integer enterpriseId, Integer storeBrandId, Integer type, Integer ownEnterpriseId);

    /**
     * saveStoreBrandRef
     * @Title: saveStoreBrandRef
     * @Description:  共享品牌
     * @author taogs 
     * @param enterpriseId
     * @param storeBrandIdList
     * @param ownEnterpriseId
     * @param type
     * @return void
     * @throws 
     */
    void saveStoreBrandRef(Integer enterpriseId, List<Integer> storeBrandIdList, Integer type, Integer ownEnterpriseId);

    /**
     * update
     * @Title: update
     * @Description:
     * @author zhiwj
     * @param storeBrandDTO
     * @return java.lang.Integer
     * @throws
     */
    Integer update(StoreBrandDTO storeBrandDTO);

    /**
     * countByStoreBrandCode
     * @Title: countByStoreBrandCode
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandCode
     * @param storeBrandId
     * @return int
     * @throws
     */
    int countByStoreBrandCode(Integer enterpriseId, String storeBrandCode, Integer storeBrandId);

    /**
     * countByStoreBrandName
     * @Title: countByStoreBrandName
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandName
     * @param storeBrandId
     * @return int
     * @throws
     */
    int countByStoreBrandName(Integer enterpriseId, String storeBrandName, Integer storeBrandId);

    /**
     * getByStoreBrandName
     * @Title: getByStoreBrandName
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandName
     * @return com.gic.store.entity.TabStoreBrand
     * @throws
     */
    TabStoreBrand getByStoreBrandName(Integer enterpriseId, String storeBrandName);

    /**
     * listStoreBrand
     * @Title: listStoreBrand
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param search
     * @param pageNum
     * @param pageSize
     * @return com.github.pagehelper.Page<com.gic.store.entity.TabStoreBrand>
     * @throws
     */
    Page<TabStoreBrand> listStoreBrand(Integer enterpriseId, String search, Integer pageNum, Integer pageSize);

    /**
     * listAllStoreBrand
     * @Title: listAllStoreBrand
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param search
     * @return java.util.List<com.gic.store.entity.TabStoreBrand>
     * @throws
     */
    List<TabStoreBrand> listAllStoreBrand(Integer enterpriseId, String search);

    /**
     * sortMoveUp
     * @Title: sortMoveUp
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandId
     * @return void
     * @throws
     */
    void sortMoveUp(Integer enterpriseId, Integer storeBrandId);

    /**
     * sortMoveDown
     * @Title: sortMoveDown
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandId
     * @return void
     * @throws
     */
    void sortMoveDown(Integer enterpriseId, Integer storeBrandId);

    /**
     * sortTopping
     * @Title: sortTopping
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandId
     * @return void
     * @throws
     */
    void sortTopping(Integer enterpriseId, Integer storeBrandId);

    /**
     * sortBottom
     * @Title: sortBottom
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandId
     * @return void
     * @throws
     */
    void sortBottom(Integer enterpriseId, Integer storeBrandId);

    /**
     * setStoreBrandSort
     * @Title: setStoreBrandSort
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandId
     * @param seq
     * @return void
     * @throws
     */
    void setStoreBrandSort(Integer enterpriseId, Integer storeBrandId, Integer seq);

    /**
     * deleteByIds
     * @Title: deleteByIds
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param delList
     * @return java.lang.Integer
     * @throws
     */
    Integer deleteByIds(Integer enterpriseId, List<Integer> delList);

    /**
     * listStoreBrandByIds
     * @Title: listStoreBrandByIds
     * @Description:
     * @author zhiwj
     * @param brandIds
     * @return java.util.List<com.gic.store.entity.TabStoreBrand>
     * @throws
     */
    List<TabStoreBrand> listStoreBrandByIds(String brandIds);

    /**
     * listStoreBrandByIds
     * @Title: listStoreBrandByIds
     * @Description:
     * @author zhiwj
     * @param brandIdList
     * @return java.util.List<com.gic.store.entity.TabStoreBrand>
     * @throws
     */
    List<TabStoreBrand> listStoreBrandByIds(List<Integer> brandIdList);

    /**
     * 查询品牌信息
     * @param brandIdList
     * @return
     */
    List<TabStoreBrand> listAllStoreBrandInfoByIds(List<Integer> brandIdList);

    /**
     * getById
     * @Title: getById
     * @Description:
     * @author zhiwj
     * @param storeBrandId
     * @return com.gic.store.entity.TabStoreBrand
     * @throws
     */
    TabStoreBrand getById(Integer storeBrandId);

    /**
     * updateRelByOperation
     * @Title: updateRelByOperation
     * @Description:
     * @author zhiwj
     * @param storeBrandId
     * @param hasRel
     * @return int
     * @throws
     */
    int updateRelByOperation(Integer storeBrandId, Integer hasRel);

    /**
     * cancelBrandAuth
     * @Title: cancelBrandAuth
     * @Description:  门店品牌取消授权
     * @author taogs
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return int
     * @throws
     */
    int cancelBrandAuth(Integer ownEnterpriseId, Integer unionEnterpriseId);
}
