package com.gic.store.service;

import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.entity.TabStoreField;
import com.github.pagehelper.Page;

import java.util.List;

/**
 * 自定义字段
 * @ClassName: StoreFieldService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/7/1 9:12 AM
 */
public interface StoreFieldService {

    /**
     * 新增
     * @Title: saveByOwner 
     * @Description:
      * @author guojuxing
     * @param storeFieldDTO 
     * @return  int  返回主键ID
      */
    int save(StoreFieldDTO storeFieldDTO);

    /**
     * 编辑保存
     * @Title: edit 
     * @Description:
      * @author guojuxing
     * @param storeFieldDTO 
     * @return  int 
      */
    int edit(StoreFieldDTO storeFieldDTO);

    /**
     * 自定义字段的code是否重复
     * @Title: isStoreFieldCodeRepeat 
     * @Description:
      * @author guojuxing
     * @param storeFieldCode
     * @param enterpriseId 
     * @return  boolean 
      */
    boolean isStoreFieldCodeRepeat(String storeFieldCode, int enterpriseId);

    /**
     * 查询下一个排序值的属性值
     * @Title: getNextSort 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param sort 
     * @return  com.gic.store.entity.TabStoreField 
      */
    StoreFieldDTO getNextSort(int enterpriseId, double sort);

    /**
     * 查询上一个排序值的属性值
     * @Title: getPrevSort 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param sort 
     * @return  com.gic.store.entity.TabStoreField 
      */
    StoreFieldDTO getPrevSort(int enterpriseId, double sort);

    /**
     * 根据主键查询
     * @Title: getStoreFieldById 
     * @Description:
      * @author guojuxing
     * @param storeFieldId 
     * @return  com.gic.store.entity.TabStoreField 
      */
    TabStoreField getStoreFieldById(int storeFieldId);

    /**
     * 更新序号
     * @param storeFieldId
     * @param sort
     * @return
     */
    int updateSortById(int storeFieldId, double sort);

    /**
     * 删除全部
     * @Title: deleteAll 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  int 
      */
    int deleteAll(int enterpriseId);

    /**
     * 批量删除
     * @Title: deleteBatch 
     * @Description:
      * @author guojuxing
     * @param storeFieldIds 
     * @return  int 
      */
    int deleteBatch(List<Integer> storeFieldIds);

    /**
     * 自定义序号排序
     * @Title: setStoreFieldSort 
     * @Description:
      * @author guojuxing
     * @param storeFieldId
     * @param enterpriseId
     * @param setValue 
     * @return  void 
      */
    void setStoreFieldSort(int storeFieldId, int enterpriseId, int setValue);

    /**
     * delete
     * @Title: delete 
     * @Description:
      * @author guojuxing
     * @param storeFieldId 
     * @return  int 
      */
    int delete(int storeFieldId);

    /**
     * 分页查询列表数据
     * @Title: listStoreField 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param search
     * @param pageNum
     * @param pageSize 
     * @param storeFieldIdList
     * @return  com.github.pagehelper.Page<com.gic.store.entity.TabStoreField> 
      */
    Page<StoreFieldDTO> listStoreField(int enterpriseId, String search, Integer pageNum, Integer pageSize,
            List<Integer> storeFieldIdList);

    /**
     * 查询列表数据
     * @Title: listStoreField 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param search 
     * @param storeFieldIdList
     * @return  java.util.List<com.gic.store.entity.TabStoreField> 
      */
    List<StoreFieldDTO> listStoreField(Integer enterpriseId, String search, List<Integer> storeFieldIdList);

    /**
     * 获取最小排序值的sort值
     * @Title: getMinSort 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  double 
      */
    double getMinSort(int enterpriseId);

    /**
     * code是否重复
     * @Title: isRepeatCode 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param storeFieldId
     * @param storeFieldCode 
     * @return  boolean 
      */
    boolean isRepeatCode(Integer enterpriseId, Integer storeFieldId, String storeFieldCode);

    /**
     * 名称是否重复
     * @Title: isRepeatName 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param storeFieldId
     * @param storeFieldName 
     * @return  boolean 
      */
    boolean isRepeatName(Integer enterpriseId, Integer storeFieldId, String storeFieldName);

    /**
     * 置底排序
     * @Title: sortBottom 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param storeFieldId 
     * @return  void 
      */
    void sortBottom(int enterpriseId, int storeFieldId);

    /**
     * 置顶排序
     * @Title: sortTopping 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param storeFieldId 
     * @return  void 
      */
    void sortTopping(int enterpriseId, int storeFieldId);

    /**
     * 根据域主键查询字段列表数据
     * @Title: listStoreFieldByRegionId 
     * @Description:
      * @author guojuxing
     * @param regionId 
     * @return  java.util.List<com.gic.store.entity.TabStoreField> 
      */
    List<StoreFieldDTO> listStoreFieldByRegionId(Integer regionId);

    /**
     * 根据企业ID查询列表
     * @Title: listStoreFieldByEnterpriseId 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  java.util.List<com.gic.store.entity.TabStoreField> 
      */
    List<StoreFieldDTO> listStoreFieldByEnterpriseId(Integer enterpriseId);

    /**
     * 批量查询
     * @Title: listByBatchId 
     * @Description:
      * @author guojuxing
     * @param storeFieldIds 
     * @return  java.util.List<com.gic.store.entity.TabStoreField> 
      */
    List<TabStoreField> listByBatchId(List<Integer> storeFieldIds);

    /**
     * 根据名称和企业ID获取字段实体
     * @Title: getStoreFieldByNameAndEnterpriseId 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param storeFieldName 
     * @return  com.gic.store.entity.TabStoreField 
      */
    StoreFieldDTO getStoreFieldByNameAndEnterpriseId(Integer enterpriseId, String storeFieldName);

    /**
     * 统计根据门店域ID
     * @Title: countStoreField 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param regionId 
     * @return  int 
      */
    int countStoreField(Integer enterpriseId, Integer regionId);

    /**
     * 查询自己域的自定义字段
     * @Title: listStoreField 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param storeFieldIdList 
     * @return  java.util.List<com.gic.store.dto.StoreFieldDTO> 
      */
    List<StoreFieldDTO> listOwnStoreField(Integer enterpriseId, List<Integer> storeFieldIdList);
}
