package com.gic.store.service;

import java.util.List;

import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreInfoDTO;
import com.gic.store.dto.StoreStrategyDTO;
import com.gic.store.entity.TabStoreStrategy;
import com.github.pagehelper.Page;

/**
 * 策略
 * @ClassName: StoreStrategyService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/7/31 1:48 PM 
 */
public interface StoreStrategyService {

    /**
     * 新增策略
     * @Title: saveStoreGroupStrategy 
     * @Description:
      * @author guojuxing
     * @param storeStrategyDTO 
     * @return  int 
      */
    int saveStoreGroupStrategy(StoreStrategyDTO storeStrategyDTO);

    /**
     * 是否策略名称重复
     * @Title: isRepeatGroupStrategyName 
     * @Description:
      * @author guojuxing
     * @param record 
     * @return  boolean 
      */
    boolean isRepeatGroupStrategyName(TabStoreStrategy record);

    /**
     * 根据主键查询
     * @param strategyId
     * @return
     */
    TabStoreStrategy getStoreStrategyById(Integer strategyId);

    /**
     * 查询下一个排序值的属性值
     * @Title: getNextSort 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param sort
     * @param strategyType 
     * @return  com.gic.store.entity.TabStoreStrategy 
      */
    TabStoreStrategy getNextSort(int enterpriseId, double sort, int strategyType);

    /**
     * 查询上一个排序值的属性值
     * @Title: getPrevSort 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param sort
     * @param strategyType 
     * @return  com.gic.store.entity.TabStoreStrategy 
      */
    TabStoreStrategy getPrevSort(int enterpriseId, double sort, int strategyType);

    /**
     * 自定义序号排序
     * @Title: setStoreStrategySort 
     * @Description:
      * @author guojuxing
     * @param strategyId
     * @param enterpriseId
     * @param setValue
     * @param strategyType 
     * @return  void 
      */
    void setStoreStrategySort(int strategyId, int enterpriseId, int setValue, int strategyType);

    /**
     * 获取最小排序值的sort值
     * @Title: getMinSort 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param strategyType 
     * @return  double 
      */
    double getMinSort(int enterpriseId, int strategyType);

    /**
     * 置底排序
     * @Title: sortBottom 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param strategyId
     * @param strategyType 
     * @return  void 
      */
    void sortBottom(int enterpriseId, int strategyId, int strategyType);

    /**
     * 置顶排序
     * @Title: sortTopping 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param strategyId
     * @param strategyType 
     * @return  void 
      */
    void sortTopping(int enterpriseId, int strategyId, int strategyType);

    /**
     * 删除分组策略（单条）
     * @Title: deleteStoreGroupStrategy 
     * @Description:
      * @author guojuxing
     * @param strategyId 
     * @return  int 
      */
    int deleteStoreGroupStrategy(int strategyId);

    /**
     * 修改策略
     * @Title: modifyStoreGroupStrategy 
     * @Description:
      * @author guojuxing
     * @param storeStrategyDTO 
     * @return  int 
      */
    int modifyStoreGroupStrategy(StoreStrategyDTO storeStrategyDTO);

    /**
     * 分页查询数据
     * @Title: listStoreStrategy 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param search
     * @param strategyType
     * @param pageNum
     * @param pageSize 
     * @return  com.github.pagehelper.Page<com.gic.store.entity.TabStoreStrategy> 
      */
    Page<TabStoreStrategy> listStoreStrategy(int enterpriseId, String search, int strategyType, Integer pageNum,
            Integer pageSize);

    /**
     * 查询策略列表数据
     * @Title: listStoreStrategy 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param strategyType 
     * @return  java.util.List<com.gic.store.entity.TabStoreStrategy> 
      */
    List<TabStoreStrategy> listStoreStrategy(int enterpriseId, int strategyType);

    /**
     * 更新排序值
     * @Title: updateSortById 
     * @Description:
      * @author guojuxing
     * @param strategyId
     * @param sort 
     * @return  void 
      */
    void updateSortById(int strategyId, double sort);

    /**
     * 验证是否命中，如果命中，返回的result值就是命中值，否则是空字符串
     * @Title: isHitStrategy 
     * @Description:
      * @author guojuxing
     * @param storeId
     * @param enterpriseId
     * @param strategyType 
     * @return  java.lang.String 
      */
    String isHitStrategy(int storeId, int enterpriseId, int strategyType);

    /**
     * 验证是否命中，如果命中，返回的result值就是命中值，否则是空字符串
     * @Title: isHitStrategy 
     * @Description:
      * @author guojuxing
     * @param storeDTO
     * @param strategyType  1:门店启用状态策略 2：分组策略
     * @return  java.lang.String 
      */
    String isHitStrategy(StoreDTO storeDTO, int strategyType);

}
