package com.gic.store.service.impl;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.GlobalInfo;
import com.gic.store.dao.mapper.TabStoreBrandMapper;
import com.gic.store.dao.mapper.TabStoreBrandRefMapper;
import com.gic.store.dto.StoreBrandDTO;
import com.gic.store.entity.TabStoreBrand;
import com.gic.store.entity.TabStoreBrandRef;
import com.gic.store.service.StoreBrandService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/6/26
 */
@Service
public class StoreBrandServiceImpl implements StoreBrandService {
    private static final Logger log = LogManager.getLogger(StoreBrandServiceImpl.class);
    @Autowired
    private TabStoreBrandMapper tabStoreBrandMapper;
    @Autowired
    private TabStoreBrandRefMapper tabStoreBrandRefMapper;

    @Override
    public Integer save(StoreBrandDTO copy) {
        Date date = new Date();
        TabStoreBrand storeBrand = new TabStoreBrand();
        storeBrand.setStoreBrandCode(copy.getStoreBrandCode());
        storeBrand.setStoreBrandName(copy.getStoreBrandName());
        storeBrand.setStoreBrandCategoryCode(copy.getStoreBrandCategoryCode());
        storeBrand.setStoreBrandCategory(copy.getStoreBrandCategory());
        storeBrand.setDeleteFlag(0);
        storeBrand.setCreateTime(date);
        storeBrand.setHasRel(GlobalInfo.DATA_STATUS_DELETE);
        tabStoreBrandMapper.insertSelective(storeBrand);
        TabStoreBrandRef tabStoreBrandRef = new TabStoreBrandRef();
        Double sort = tabStoreBrandRefMapper.getMinSort(copy.getEnterpriseId());
        tabStoreBrandRef.setSort(sort == null ? 1d : sort - 1d);
        tabStoreBrandRef.setEnterpriseId(copy.getEnterpriseId());
        tabStoreBrandRef.setFromEnterpriseId(copy.getEnterpriseId());
        tabStoreBrandRef.setStoreBrandId(storeBrand.getStoreBrandId());
        tabStoreBrandRef.setDeleteFlag(0);
        tabStoreBrandRef.setCreateTime(date);
        tabStoreBrandRef.setUpdateTime(date);
        tabStoreBrandRef.setType(copy.getType());
        tabStoreBrandRefMapper.insertSelective(tabStoreBrandRef);
        return storeBrand.getStoreBrandId();
    }

    @Override
    public void saveStoreBrandRef(Integer enterpriseId, Integer storeBrandId, Integer type, Integer ownEnterpriseId){
        TabStoreBrandRef existTab = tabStoreBrandRefMapper.selectByEidAndBrandId(enterpriseId, storeBrandId);
        if(existTab != null){
            return;
        }
        Date date = new Date();
        TabStoreBrandRef tabStoreBrandRef = new TabStoreBrandRef();
        Double sort = tabStoreBrandRefMapper.getMinSort(enterpriseId);
        tabStoreBrandRef.setSort(sort == null ? 1d : sort - 1d);
        tabStoreBrandRef.setEnterpriseId(enterpriseId);
        tabStoreBrandRef.setFromEnterpriseId(ownEnterpriseId);
        tabStoreBrandRef.setStoreBrandId(storeBrandId);
        tabStoreBrandRef.setDeleteFlag(0);
        tabStoreBrandRef.setCreateTime(date);
        tabStoreBrandRef.setUpdateTime(date);
        tabStoreBrandRef.setType(type);
        tabStoreBrandRefMapper.insertSelective(tabStoreBrandRef);
    }

    @Override
    public void saveStoreBrandRef(Integer enterpriseId, List<Integer> storeBrandIdList, Integer type, Integer ownEnterpriseId) {
        if(CollectionUtils.isNotEmpty(storeBrandIdList)){
            List<TabStoreBrandRef> list = new ArrayList<>();
            Double sort = tabStoreBrandRefMapper.getMinSort(enterpriseId);
            sort = sort == null ? 1d : sort - 1d;
            Date date = new Date();
            List<TabStoreBrandRef> existList = tabStoreBrandRefMapper.selectByEidAndBrandIdList(enterpriseId, storeBrandIdList);
            Map<String, Integer> existMap = existList.stream().collect(Collectors.toMap(ref -> ref.getEnterpriseId() + "-" + ref.getStoreBrandId(), TabStoreBrandRef::getId, (k1,k2)->k2));
            for(Integer storeBrandId : storeBrandIdList){
                if(MapUtils.isNotEmpty(existMap) && existMap.containsKey(enterpriseId + "-" + storeBrandId)){
                    continue;
                }
                TabStoreBrandRef tabStoreBrandRef = new TabStoreBrandRef();
                tabStoreBrandRef.setSort(sort);
                tabStoreBrandRef.setEnterpriseId(enterpriseId);
                tabStoreBrandRef.setFromEnterpriseId(ownEnterpriseId);
                tabStoreBrandRef.setStoreBrandId(storeBrandId);
                tabStoreBrandRef.setDeleteFlag(0);
                tabStoreBrandRef.setCreateTime(date);
                tabStoreBrandRef.setUpdateTime(date);
                tabStoreBrandRef.setType(type);
                list.add(tabStoreBrandRef);
                sort = sort - 1d;
            }
            /**新增需要共享的品牌***/
            if(CollectionUtils.isNotEmpty(list)){
                tabStoreBrandRefMapper.insertList(list);
            }
            /**删除本次***/
        }
    }

    @Override
    public Integer update(StoreBrandDTO copy) {
        TabStoreBrand storeBrand = new TabStoreBrand();
        storeBrand.setStoreBrandId(copy.getStoreBrandId());
        storeBrand.setStoreBrandCode(copy.getStoreBrandCode());
        storeBrand.setStoreBrandName(copy.getStoreBrandName());
        storeBrand.setStoreBrandCategoryCode(copy.getStoreBrandCategoryCode());
        storeBrand.setStoreBrandCategory(copy.getStoreBrandCategory());
        return tabStoreBrandMapper.updateByPrimaryKeySelective(storeBrand);
    }

    @Override
    public int countByStoreBrandCode(Integer enterpriseId, String storeBrandCode, Integer storeBrandId) {
        return tabStoreBrandMapper.countBySelective(enterpriseId, storeBrandCode, null, storeBrandId);
    }

    @Override
    public int countByStoreBrandName(Integer enterpriseId, String storeBrandName, Integer storeBrandId) {
        return tabStoreBrandMapper.countBySelective(enterpriseId, null, storeBrandName, storeBrandId);
    }

    @Override
    public TabStoreBrand getByStoreBrandName(Integer enterpriseId, String storeBrandName) {
        return tabStoreBrandMapper.getByStoreBrandName(enterpriseId, storeBrandName);
    }

    @Override
    public Page<TabStoreBrand> listStoreBrand(Integer enterpriseId, String search, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<TabStoreBrand> storeBrandList = tabStoreBrandMapper.listStoreBrand(enterpriseId, search);
        return (Page<TabStoreBrand>) storeBrandList;
    }

    @Override
    public List<TabStoreBrand> listAllStoreBrand(Integer enterpriseId, String search) {
        return tabStoreBrandMapper.listStoreBrand(enterpriseId, search);
    }

    @Override
    public void sortMoveUp(Integer enterpriseId, Integer storeBrandId) {
        TabStoreBrandRef tabStoreBrandRef = tabStoreBrandRefMapper.selectByEidAndBrandId(enterpriseId, storeBrandId);
        TabStoreBrand prevStoreBrand = tabStoreBrandMapper.getPrevSort(enterpriseId, tabStoreBrandRef.getSort());
        swapSort(tabStoreBrandRef, prevStoreBrand);
    }

    @Override
    public void sortMoveDown(Integer enterpriseId, Integer storeBrandId) {
        TabStoreBrandRef tabStoreBrandRef = tabStoreBrandRefMapper.selectByEidAndBrandId(enterpriseId, storeBrandId);
        TabStoreBrand nextStoreBrand = tabStoreBrandMapper.getNextSort(enterpriseId, tabStoreBrandRef.getSort());
        swapSort(tabStoreBrandRef, nextStoreBrand);
    }

    private void swapSort(TabStoreBrandRef tabStoreBrandRef, TabStoreBrand prevStoreBrand) {
        log.info("品牌排序：{},{}", JSON.toJSONString(tabStoreBrandRef), JSON.toJSONString(prevStoreBrand));
        if (prevStoreBrand != null) {
            tabStoreBrandRefMapper.updateSort(tabStoreBrandRef.getEnterpriseId(), tabStoreBrandRef.getStoreBrandId(), prevStoreBrand.getSort());
            tabStoreBrandRefMapper.updateSort(prevStoreBrand.getEnterpriseId(), prevStoreBrand.getStoreBrandId(), tabStoreBrandRef.getSort());
        }
    }

    @Override
    public void sortTopping(Integer enterpriseId, Integer storeBrandId) {
        Double sort = tabStoreBrandRefMapper.getMinSort(enterpriseId);
        tabStoreBrandRefMapper.updateSort(enterpriseId, storeBrandId, sort - 1d);
    }

    @Override
    public void sortBottom(Integer enterpriseId, Integer storeBrandId) {
        Double sort = tabStoreBrandRefMapper.getMaxSort(enterpriseId);
        if (sort != null) {
            tabStoreBrandRefMapper.updateSort(enterpriseId, storeBrandId, sort + 1d);
        }
    }

    @Override
    public void setStoreBrandSort(Integer enterpriseId, Integer storeBrandId, Integer seq) {
        if (seq <= 1) {
            // 相当于置顶
            sortTopping(enterpriseId, storeBrandId);
        } else {
            int start = seq - 2;
            int size = 2;

            List<TabStoreBrand> list = tabStoreBrandMapper.listTwoElementExceptId(enterpriseId, start, size, storeBrandId);
            if (CollectionUtils.isEmpty(list)) {
                // 置底
                sortBottom(enterpriseId, storeBrandId);
            } else {
                Double sort;
                if (list.size() == 1) {
                    // 置底
                    sort = list.get(0).getSort() + 1d;
                } else {
                    sort = (list.get(0).getSort() + list.get(1).getSort()) / 2d;
                }
                tabStoreBrandRefMapper.updateSort(enterpriseId, storeBrandId, sort);
            }
        }
    }

    @Override
    public Integer deleteByIds(Integer enterpriseId, List<Integer> delList) {
        if (CollectionUtils.isNotEmpty(delList)) {
            return tabStoreBrandMapper.deleteByIds(delList);
        }
        return 0;
    }

    @Override
    public List<TabStoreBrand> listStoreBrandByIds(String brandIds) {
        if (StringUtils.isBlank(brandIds)) {
            return null;
        }
        List<String> idStrList = Arrays.asList(brandIds.split(","));
        if (CollectionUtils.isNotEmpty(idStrList)) {
            List<Integer> idList = idStrList.stream().map(Integer::parseInt).collect(Collectors.toList());
            return tabStoreBrandMapper.listStoreBrandByIds(idList);
        }
        return null;
    }

    @Override
    public List<TabStoreBrand> listStoreBrandByIds(List<Integer> brandIdList) {
        if (CollectionUtils.isNotEmpty(brandIdList)) {
            return tabStoreBrandMapper.listStoreBrandByIds(brandIdList);
        }
        return null;
    }

    @Override
    public List<TabStoreBrand> listAllStoreBrandInfoByIds(List<Integer> brandIdList) {
        if (CollectionUtils.isNotEmpty(brandIdList)) {
            return tabStoreBrandMapper.listAllStoreBrandInfoByIds(brandIdList);
        }
        return null;
    }

    @Override
    public TabStoreBrand getById(Integer storeBrandId) {
        return tabStoreBrandMapper.selectByPrimaryKey(storeBrandId);
    }

    @Override
    public int updateRelByOperation(Integer storeBrandId, Integer hasRel) {
        TabStoreBrand tabStoreBrand = new TabStoreBrand();
        tabStoreBrand.setStoreBrandId(storeBrandId);
        tabStoreBrand.setHasRel(hasRel);
        return tabStoreBrandMapper.updateByPrimaryKeySelective(tabStoreBrand);
    }

    @Override
    public int cancelBrandAuth(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        return tabStoreBrandRefMapper.cancelBrandAuth(ownEnterpriseId, unionEnterpriseId);
    }
}
