package com.gic.store.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.store.dao.mapper.TabStoreFieldSelectMapper;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.entity.TabStoreFieldSelect;
import com.gic.store.service.StoreFieldSelectService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author guojx
 * @date 2019/7/1 2:48 PM
 */
@Service("storeFieldSelectService")
public class StoreFieldSelectServiceImpl implements StoreFieldSelectService{
    @Autowired
    private TabStoreFieldSelectMapper tabStoreFieldSelectMapper;

    @Override
    public int save(StoreFieldSelectDTO storeFieldSelectDTO) {
        TabStoreFieldSelect storeFieldSelect = EntityUtil.changeEntityNew(TabStoreFieldSelect.class, storeFieldSelectDTO);
        tabStoreFieldSelectMapper.insert(storeFieldSelect);
        return storeFieldSelect.getStoreFieldSelectId();
    }

    @Override
    public TabStoreFieldSelect getNextSort(int storeFieldId, double sort) {
        return tabStoreFieldSelectMapper.selectNextByStoreFieldIdAndSort(storeFieldId, sort);
    }

    @Override
    public TabStoreFieldSelect getPrevSort(int storeFieldId, double sort) {
        return tabStoreFieldSelectMapper.selectPrevByStoreFieldIdAndSort(storeFieldId, sort);
    }

    @Override
    public TabStoreFieldSelect getStoreFieldSelectById(int storeFieldSelectId) {
        return tabStoreFieldSelectMapper.selectByPrimaryKey(storeFieldSelectId);
    }

    @Override
    public int updateSortById(int storeFieldSelectId, double sort) {
        TabStoreFieldSelect fieldSelect = new TabStoreFieldSelect();
        fieldSelect.setStoreFieldSelectId(storeFieldSelectId);
        fieldSelect.setSort(sort);
        tabStoreFieldSelectMapper.updateByPrimaryKeySelective(fieldSelect);
        return 1;
    }

    @Override
    public void setStoreFieldSelectSort(int storeFieldSelectId, int storeFieldId, int setValue) {
        if (setValue <= 1) {
            // 相当于置顶
            sortTopping(storeFieldId, storeFieldSelectId);
        } else {
            int start = setValue - 2;
            int size = 2;

            List<TabStoreFieldSelect> list = tabStoreFieldSelectMapper.listTwoElementExceptId(storeFieldId, start, size, storeFieldSelectId);
            if (CollectionUtils.isEmpty(list)) {
                // 置底
                sortBottom(storeFieldId, storeFieldSelectId);
            } else {
                Double sort;
                if (list.size() == 1) {
                    // 置底
                    sort = list.get(0).getSort() + 1d;
                } else {
                    sort = (list.get(0).getSort() + list.get(1).getSort()) / 2d;
                }
                TabStoreFieldSelect fieldSelect = new TabStoreFieldSelect();
                fieldSelect.setStoreFieldSelectId(storeFieldSelectId);
                fieldSelect.setSort(sort);
                tabStoreFieldSelectMapper.updateByPrimaryKeySelective(fieldSelect);
            }
        }
    }

    @Override
    public int deleteBatch(List<Integer> storeFieldSelectIdList) {
        return tabStoreFieldSelectMapper.deleteBatch(storeFieldSelectIdList);
    }

    @Override
    public int edit(int storeFieldSelectId, String storeFieldSelectName, String storeFieldSelectCode) {
        TabStoreFieldSelect tabStoreFieldSelect = new TabStoreFieldSelect();
        tabStoreFieldSelect.setStoreFieldSelectId(storeFieldSelectId);
        tabStoreFieldSelect.setStoreFieldSelectName(storeFieldSelectName);
        tabStoreFieldSelect.setStoreFieldSelectCode(storeFieldSelectCode);
        tabStoreFieldSelect.setUpdateTime(new Date());
        return tabStoreFieldSelectMapper.updateByPrimaryKeySelective(tabStoreFieldSelect);
    }

    @Override
    public Page<TabStoreFieldSelect> listStoreFieldSelect(int storeFieldId, String search, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<TabStoreFieldSelect> list = tabStoreFieldSelectMapper.listStoreFieldSelect(storeFieldId, search);
        return (Page<TabStoreFieldSelect>) list;
    }

    @Override
    public List<TabStoreFieldSelect> listStoreFieldSelect(int storeFieldId, String search) {
        List<TabStoreFieldSelect> list = tabStoreFieldSelectMapper.listStoreFieldSelect(storeFieldId, search);
        if (list == null || list.size() < 1) {
            return new ArrayList<>();
        }
        return list;
    }


    @Override
    public void sortTopping(int storeFieldId, int storeFieldSelectId) {
        Double sort = getMinSort(storeFieldId);
        TabStoreFieldSelect fieldSelect = new TabStoreFieldSelect();
        fieldSelect.setStoreFieldSelectId(storeFieldSelectId);
        fieldSelect.setSort(sort - 1d);
        tabStoreFieldSelectMapper.updateByPrimaryKeySelective(fieldSelect);
    }

    @Override
    public double getMinSort(int storeFieldId) {
        Double sort = tabStoreFieldSelectMapper.getMinSort(storeFieldId);
        return sort == null ? 1d : sort;
    }

    @Override
    public boolean isRepeatCode(Integer storeFieldId, Integer storeFieldSelectId, String storeFieldSelectCode) {
        int count = tabStoreFieldSelectMapper.isRepeatCode(storeFieldId, storeFieldSelectId, storeFieldSelectCode);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public boolean isRepeatName(Integer storeFieldId, Integer storeFieldSelectId, String storeFieldSelectName) {
        int count = tabStoreFieldSelectMapper.isRepeatName(storeFieldId, storeFieldSelectId, storeFieldSelectName);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public List<TabStoreFieldSelect> listStoreFieldSelectByIds(List<Integer> storeFileIdSelectIdList) {
        return this.tabStoreFieldSelectMapper.listStoreFieldSelectByIds(storeFileIdSelectIdList);
    }

    @Override
    public void sortBottom(int storeFieldId, int storeFieldSelectId) {
        Double sort = getMaxSort(storeFieldId);
        TabStoreFieldSelect fieldSelect = new TabStoreFieldSelect();
        fieldSelect.setStoreFieldSelectId(storeFieldSelectId);
        fieldSelect.setSort(sort + 1d);
        tabStoreFieldSelectMapper.updateByPrimaryKeySelective(fieldSelect);
    }

    public double getMaxSort(int storeFieldId) {
        Double sort = tabStoreFieldSelectMapper.getMaxSort(storeFieldId);
        return sort == null ? 1d : sort;
    }

}
