package com.gic.store.service.impl;

import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dao.mapper.TabStoreRegionMapper;
import com.gic.store.dao.mapper.TabStoreRegionRelMapper;
import com.gic.store.dto.StoreRegionDTO;
import com.gic.store.entity.TabStoreRegion;
import com.gic.store.entity.TabStoreRegionRel;
import com.gic.store.service.StoreRegionService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/6/24
 */
@Service
@SuppressWarnings("unchecked")
public class StoreRegionServiceImpl implements StoreRegionService {

    @Autowired
    private TabStoreRegionMapper storeRegionMapper;
    @Autowired
    private TabStoreRegionRelMapper tabStoreRegionRelMapper;

    @Override
    public int countByRegionCode(Integer enterpriseId, String regionCode) {
        StoreRegionDTO storeRegion = new StoreRegionDTO();
        storeRegion.setEnterpriseId(enterpriseId);
        storeRegion.setRegionCode(regionCode);
        return storeRegionMapper.countBySelective(storeRegion);
    }

    @Override
    public int countByRegionName(Integer enterpriseId, String regionName, Integer regionId) {
        StoreRegionDTO storeRegion = new StoreRegionDTO();
        storeRegion.setEnterpriseId(enterpriseId);
        storeRegion.setRegionName(regionName);
        storeRegion.setRegionId(regionId);
        return storeRegionMapper.countBySelective(storeRegion);
    }

    @Override
    public Integer save(Integer enterpriseId, String regionCode, String regionName) {
        TabStoreRegion storeRegion = new TabStoreRegion();
        storeRegion.setRegionCode(regionCode);
        storeRegion.setRegionName(regionName);
        storeRegion.setCreateTime(new Date());
        storeRegion.setDeleteFlag(0);
        storeRegionMapper.insertSelective(storeRegion);
        return storeRegion.getRegionId();
    }

    @Override
    public int update(Integer regionId, String regionName) {
        TabStoreRegion storeRegion = new TabStoreRegion();
        storeRegion.setRegionId(regionId);
        storeRegion.setRegionName(regionName);
        int i = storeRegionMapper.updateByPrimaryKeySelective(storeRegion);
        return i;
    }

    @Override
    public Page<StoreRegionDTO> listStoreRegion(Integer enterpriseId, String search, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        // 查列表
        List<StoreRegionDTO> storeRegionList = storeRegionMapper.listStoreRegion(enterpriseId, search, null);
        return (com.github.pagehelper.Page) storeRegionList;
    }

    @Override
    public List<StoreRegionDTO> listStoreRegion(Integer enterpriseId, String search, Integer ownType) {
        return storeRegionMapper.listStoreRegion(enterpriseId, search, ownType);
    }

    @Override
    public List<Integer> listStoreRegion(Integer enterpriseId) {
        List<StoreRegionDTO> list = storeRegionMapper.listStoreRegion(enterpriseId, null, null);
        return Optional.ofNullable(list).orElse(Collections.emptyList()).stream().map(StoreRegionDTO::getRegionId).collect(Collectors.toList());
    }

    @Override
    public StoreRegionDTO getById(Integer enterpriseId, Integer regionId) {
        return storeRegionMapper.selectByPrimaryKey(enterpriseId, regionId);
    }

    @Override
    public List<StoreRegionDTO> listStoreRegionByIds(Integer enterpriseId, List<Integer> storeRegionIdList, Integer ownType) {
        return this.storeRegionMapper.listStoreRegionByIds(enterpriseId, storeRegionIdList, ownType);
    }

    @Override
    public List<StoreRegionDTO> listStoreRegionByIds(Integer enterpriseId, List<Integer> storeRegionIdList) {
        return this.storeRegionMapper.listStoreRegionByIds(enterpriseId, storeRegionIdList, null);
    }

    @Override
    public StoreRegionDTO getStoreRegionByRegionName(Integer enterpriseId, String regionName) {
        return storeRegionMapper.getStoreRegionByRegionName(enterpriseId, regionName);
    }

    @Override
    public String relationErp(Integer enterpriseId, Integer regionId) {
        StoreRegionDTO storeRegionDTO = this.getById(enterpriseId, regionId);
        if(storeRegionDTO.getOwnType() == StoreOwnTypeEnum.OTHER.getCode()){
            return "共享域无法关联";
        }
        int i = this.storeRegionMapper.relationErp(enterpriseId, regionId);
        if(i == 0){
            return "关联失败";
        }
        return null;
    }

    @Override
    public Integer delRegion(Integer enterpriseId, Integer regionId) {
        TabStoreRegion storeRegion = new TabStoreRegion();
        storeRegion.setRegionId(regionId);
        storeRegion.setDeleteFlag(1);
        int i = storeRegionMapper.updateByPrimaryKeySelective(storeRegion);
        tabStoreRegionRelMapper.deleteRegionRel(enterpriseId, regionId);
        return i;
    }
}
