package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.store.dto.ClerkTmpDTO;
import com.gic.store.entity.TabClerk;
import com.gic.store.service.ClerkImportApiService;
import com.gic.store.service.ClerkImportService;
import com.gic.store.service.ClerkService;
import com.gic.store.service.ClerkTaskApiService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;


/**
 * @author zhiwj
 * @date 2019/7/17
 */
@Service("clerkImportApiService")
@SuppressWarnings("unchecked")
public class ClerkImportApiServiceImpl implements ClerkImportApiService {

    @Autowired
    private ClerkImportService clerkImportService;

    @Autowired
    private ClerkTaskApiService clerkTaskService;

    @Autowired
    private ClerkService clerkService;

    @Override
    public ServiceResponse<Boolean> hasImportingClerk(Integer enterpriseId) {
        return ServiceResponse.success(clerkImportService.hasImportingClerk(enterpriseId));
    }

    @Override
    public ServiceResponse<Void> delClerkTmp(Integer enterpriseId) {
        clerkImportService.delClerkTmp(enterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> save(ClerkTmpDTO bean) {
        clerkImportService.save(bean);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> importDataToClerk(Integer enterpriseId, String userId, String uuId, boolean isReplace) {
        return ServiceResponse.success(clerkTaskService.importDataToClerk(enterpriseId, userId, uuId, isReplace));
    }

    @Override
    public ServiceResponse<Page> listClerkTmp(Integer enterpriseId, boolean isSuccess, Integer currentPage, Integer pageSize) {
        com.github.pagehelper.Page clerkTmpList = clerkImportService.listClerkTmp(enterpriseId, isSuccess, currentPage, pageSize);
        Page<ClerkTmpDTO> page = PageHelperUtils.changePageHelperToCurrentPage(clerkTmpList, ClerkTmpDTO.class);
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<Map<String, Object>> getClerkBatchProcess(Integer enterpriseId, String uuId, Integer currentPage, Integer pageSize) {
        Map<String, Object> data = new HashMap<>(8);
        Integer taskTotal = clerkImportService.countByTaskTotal(enterpriseId, uuId);
        Integer taskExecNum = clerkImportService.countByTaskExecNum(enterpriseId, uuId);
        Integer taskFailNum = clerkImportService.countByTaskFailNum(enterpriseId, uuId);
        data.put("taskTotal", taskTotal);
        data.put("taskExecNum", taskExecNum);
        data.put("taskFailNum", taskFailNum);
        if ((taskTotal - taskFailNum) > 0) {
            data.put("process", taskExecNum * 100 / (taskTotal - taskFailNum));
            // 获取所有错误数据
            Page resultPage = this.listClerkTmp(enterpriseId, false, currentPage, pageSize).getResult();
            data.put("errorData", resultPage);
        } else {
            data.put("process", 100);
        }
        return ServiceResponse.success(data);
    }

    @Override
    public ServiceResponse<List<ClerkTmpDTO>> listClerkTmp(Integer enterpriseId, Boolean isRight) {
        List<ClerkTmpDTO> result = EntityUtil.changeEntityListByOrika(ClerkTmpDTO.class, clerkImportService.listClerkTmp(enterpriseId, isRight));
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Page> pageError(Integer enterpriseId, Integer currentPage, Integer pageSize) {
        Page page = this.listClerkTmp(enterpriseId, Boolean.FALSE, currentPage, pageSize).getResult();
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<List<ClerkTmpDTO>> listAllClerkTmp(Integer enterpriseId) {
        return ServiceResponse.success(EntityUtil.changeEntityListByOrika(ClerkTmpDTO.class, clerkImportService.listAllClerkTmp(enterpriseId)));
    }

    @Override
    public ServiceResponse<List<List<String>>> listError(Integer enterpriseId) {
        List<List<String>> resultList = new ArrayList<>();
        List<ClerkTmpDTO> dataList = this.listClerkTmp(enterpriseId, Boolean.FALSE).getResult();
        for (int i = 0; i < dataList.size(); i++) {
            List<String> list = new LinkedList<>();
            ClerkTmpDTO clerkTmpDTO = dataList.get(i);
            list.add(i + 1 + "");
            list.add(clerkTmpDTO.getClerkName());
            list.add(clerkTmpDTO.getClerkCode());
            list.add(clerkTmpDTO.getStoreName());
            list.add(clerkTmpDTO.getStoreCode());
            list.add(clerkTmpDTO.getPositionName());
            list.add(clerkTmpDTO.getClerkGender());
            list.add(clerkTmpDTO.getNationcode());
            list.add(clerkTmpDTO.getPhoneNumber());
            list.add(clerkTmpDTO.getClerkType());
            list.add(clerkTmpDTO.getErrorMessage());
            resultList.add(list);
        }
        return ServiceResponse.success(resultList);
    }

    @Override
    public ServiceResponse<Boolean> validateClerkCode(Integer enterpriseId, Integer storeInfoId, String clerkCode) {
        TabClerk clerk = clerkService.getClerkByCode(enterpriseId, clerkCode, storeInfoId);
        return ServiceResponse.success(clerk != null);
    }
}
