package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.store.constant.*;
import com.gic.store.dto.*;
import com.gic.store.entity.TabClerk;
import com.gic.store.entity.TabClerkPosition;
import com.gic.store.entity.TabClerkTmp;
import com.gic.store.service.*;
import com.task.allocation.api.AbstractTaskAllocationOperation;
import com.task.allocation.exception.TaskAllocationException;
import com.task.allocation.qo.InitTaskQO;
import com.task.allocation.util.TaskAllocationSdkClient;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author zhiwj
 * @date 2019/7/17
 */
@Service("clerkTaskApiService")
public class ClerkTaskServiceImpl extends AbstractTaskAllocationOperation implements ClerkTaskApiService {

    private static final Logger logger = LogManager.getLogger(ClerkTaskServiceImpl.class);

    @Autowired
    private ClerkImportService clerkImportService;

    @Autowired
    private ClerkApiService clerkApiService;
    @Autowired
    private ClerkService clerkService;

    @Autowired
    private StoreService storeService;
    @Autowired
    private ClerkPositionService clerkPositionService;
    @Autowired
    private StoreRegionService storeRegionService;

    @Override
    public String importDataToClerk(Integer enterpriseId, String userId, String uuId, boolean isReplace) {
        logger.info("enterpriseId:{}, userId:{}, uuId:{}", enterpriseId, userId, uuId);
        String message = "";
        InitTaskQO initTaskQo = new InitTaskQO();
        initTaskQo.setEnterpriseId(enterpriseId.toString());// 企业id
        initTaskQo.setOperationUserId(userId);// 操作人id
        // key
        initTaskQo.setTaskMqKey(Constants.CLERK_BATCH_IMPORT_MQ_KEY);// 注册的队列名称
        initTaskQo.setTaskType(Constants.CLERK_BATCH_IMPORT_TASK_TYPE);// 任务类型
        initTaskQo.setTaskSignKey(uuId);// 任务类型 唯一标识符

        JSONObject params = new JSONObject();
        //如果门店code重复，是否替换更新
        params.put("isReplace", isReplace);
        params.put("enterpriseId", enterpriseId);
        initTaskQo.setParams(params.toString());

        try {
            TaskAllocationSdkClient instance = TaskAllocationSdkClient.getInstance();// 初始化sdk
            instance.initTask(initTaskQo);// 初始化接口
            logger.info("导购批量导入队列初始化成功");
        } catch (Exception e) {
            logger.warn("导购批量导入队列初始化异常", e);
            message = "导购批量导入队列初始化异常";
        }
        return message;
    }

    @Override
    public List<Object> getListTasks(String params) {
        JSONObject json = JSON.parseObject(params);
        String enterpriseId = json.getString("enterpriseId");
        boolean isReplace = json.getBoolean("isReplace");
        List<TabClerkTmp> list = clerkImportService.listUnImportClerk(enterpriseId);
        logger.info("导购批量导入放入队列数据数量：{}", list.size());
        List<ClerkTmpDTO> taskList = EntityUtil.changeEntityListByJSON(ClerkTmpDTO.class, list);
        if (CollectionUtils.isNotEmpty(taskList)) {
            for (ClerkTmpDTO dto : taskList) {
                dto.setReplace(isReplace);
            }
        }
        List listTask = taskList;
        return listTask;
    }

    @Override
    public void dealSingle(Object obj) throws TaskAllocationException {
        logger.info("开始执行队列");
        ClerkTmpDTO t = (ClerkTmpDTO) obj;
        try {
            this.importDataToStore(t);
        } catch (Exception e) {
            logger.warn("写入门成员表异常", e);
            t.setErrorMessage("导入异常");
            clerkImportService.updateData(t);
        }
    }

    private void importDataToStore(ClerkTmpDTO t) {

        StoreRegionDTO region = storeRegionService.getStoreRegionByRegionName(t.getEnterpriseId(), t.getRegionName());
        if (region == null) {
            t.setErrorMessage("对应的门店域不存在");
            clerkImportService.updateData(t);
            return;
        }

        StoreDTO store = storeService.getByStoreNameAndStoreCode(t.getEnterpriseId(), region.getRegionId(), t.getStoreName(), t.getStoreCode());
        if (store == null) {
            t.setErrorMessage("对应的门店不存在");
            clerkImportService.updateData(t);
            return;
        }
        if (StoreOwnTypeEnum.OWNER.getCode() != store.getOwnType()) {
            t.setErrorMessage("不能在被授权门店下导入导购");
            clerkImportService.updateData(t);
            return;
        }
        TabClerkPosition position = clerkPositionService.getByName(t.getEnterpriseId(), t.getPositionName());
        if (position == null) {
            t.setErrorMessage("角色不存在");
            clerkImportService.updateData(t);
            return;
        }
        ClerkDTO clerkDTO = new ClerkDTO();
        // 判断code是否重复
        TabClerk clerk = clerkService.getClerkByCode(t.getEnterpriseId(), t.getClerkCode(), store.getStoreInfoId());
        if (clerk != null) {
            if (t.getReplace()) {
                //替换更新code重复的记录
                clerkDTO.setClerkId(clerk.getClerkId());
            } else {
                t.setErrorMessage("门店code已经存在");
                clerkImportService.updateData(t);
                return;
            }
        }

        clerkDTO.setEnterpriseId(t.getEnterpriseId());
        clerkDTO.setStoreInfoId(store.getStoreInfoId());
        clerkDTO.setClerkName(t.getClerkName().trim());
        clerkDTO.setClerkCode(t.getClerkCode().trim());
        clerkDTO.setClerkGender(GenderEnum.parseMessage(t.getClerkGender().trim()).getCode());
        clerkDTO.setNationcode(t.getNationcode().trim());
        clerkDTO.setPhoneNumber(t.getPhoneNumber().trim());
        clerkDTO.setPositionId(position.getClerkPositionId());
        clerkDTO.setPositionName(t.getPositionName().trim());

        clerkDTO.setReason(ClerkLogReasonEnum.WEB.getCode());
        clerkDTO.setRemark("批量导入");
        clerkDTO.setOperatorId(t.getOperatorId());
        clerkDTO.setOperatorName(t.getOperatorName());

        ServiceResponse serviceResponse = clerkApiService.saveOrUpdate(clerkDTO);
        String msg;
        if (serviceResponse.isSuccess()) {
            msg = StoreImportEnum.SUCCESS.getCode();
        } else {
            msg = serviceResponse.getMessage();
        }
        t.setErrorMessage(msg);
        clerkImportService.updateData(t);
    }
}
