package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.dto.StoreResourceDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.dto.UserResourceDTO;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.service.UserResourceApiService;
import com.gic.commons.util.*;
import com.gic.enterprise.dto.CustomStoreDTO;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseLicenseDTO;
import com.gic.enterprise.dto.wm.WmStoreSyncLogDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.CustomStoreApiService;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.WmStoreSyncLogApiService;
import com.gic.enterprise.utils.LicenseUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.member.api.service.StoreUpdateApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import com.gic.search.business.api.constant.enums.OperateEnum;
import com.gic.search.business.api.dto.DynamicDocDTO;
import com.gic.search.business.api.dto.DynamicSearchDTO;
import com.gic.search.business.api.dto.ESResponseQueryBatchDTO;
import com.gic.search.business.api.service.EsBusinessOperaApiService;
import com.gic.search.business.api.utils.QueryConditionAssemblyUtil;
import com.gic.spark.api.service.SparkJobApiService;
import com.gic.store.constant.*;
import com.gic.store.dto.*;
import com.gic.store.entity.*;
import com.gic.store.service.*;
import com.gic.store.strategy.BulkUpdateStoreStrtegy;
import com.gic.store.utils.StoreRedisKeyUtils;
import com.gic.store.utils.field.*;
import com.gic.store.utils.valid.ValidUtil;
import com.gic.weimob.api.dto.WeimobPhysicalStoreDTO;
import com.gic.weimob.api.dto.WeimobStoreAccountDTO;
import com.gic.weimob.api.service.WeimobStoreSiteApiService;
import com.google.common.base.Joiner;
import com.google.common.collect.MapDifference;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.common.geo.GeoUtils;
import org.redisson.api.RKeys;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @date 2019/6/27
 */
@Service("storeApiService")
@SuppressWarnings("unchecked")
public class StoreApiServiceImpl implements StoreApiService {
    private static final Logger logger = LogManager.getLogger(StoreApiServiceImpl.class);
    private Long storeExpiredTime = 1L;
    private static Pattern NUMBER_PATTERN = Pattern.compile("\\d+\\.\\d+$|-\\d+\\.\\d+$");
    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreBrandService storeBrandService;
    @Autowired
    private StorePhotoService storePhotoService;
    @Autowired
    private StoreBusinessTimeService storeBusinessTimeService;
    @Autowired
    private StoreExtendService storeExtendService;
    @Autowired
    private StoreStrategyApiService storeStrategyApiService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreGroupApiService storeGroupApiService;
    @Autowired
    private StoreStrategyService storeStrategyService;
    @Autowired
    private StoreFieldSelectService storeFieldSelectService;
    @Autowired
    private StoreRegionService storeRegionService;
    @Autowired
    private EsBusinessOperaApiService esBusinessOperaApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private StoreLogService storeLogService;
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private CustomStoreApiService customStoreApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreWidgetService storeWidgetService;
    @Autowired
    private ProvincesApiService provincesApiService;
    @Autowired
    private StoreFieldService storeFieldService;
    @Autowired
    private StoreIndexLogService storeIndexLogService;
    @Autowired
    private ClerkApiService clerkApiService;
    @Autowired
    private StoreTagService storeTagService;
    @Autowired
    private WeimobStoreSiteApiService weimobStoreSiteApiService;
    @Autowired
    private WmStoreSyncLogApiService wmStoreSyncLogApiService;
    @Autowired
    private SparkJobApiService sparkJobApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private StoreUpdateApiService storeUpdateApiService;

    private Map<String, BulkUpdateStoreStrtegy> storeStrtegyMap = new ConcurrentHashMap<>();

    private static Map<String, String> BUSINESS_TIME_MAP = new HashMap<>(16);
    static {
        BUSINESS_TIME_MAP.put("1", "周一");
        BUSINESS_TIME_MAP.put("2", "周二");
        BUSINESS_TIME_MAP.put("3", "周三");
        BUSINESS_TIME_MAP.put("4", "周四");
        BUSINESS_TIME_MAP.put("5", "周五");
        BUSINESS_TIME_MAP.put("6", "周六");
        BUSINESS_TIME_MAP.put("7", "周日");
    }

    @Autowired
    public StoreApiServiceImpl(Map<String, BulkUpdateStoreStrtegy> strategyMap) {
        this.storeStrtegyMap.clear();
        this.storeStrtegyMap.putAll(strategyMap);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<StoreDTO> saveOrUpdate(StoreDTO storeDTO) {
        logger.info("storeDTO:{}", JSON.toJSONString(storeDTO));
        ServiceResponse<StoreDTO> x = valid(storeDTO);
        if (x != null){
            return x;
        }
        // 校验是否完全填写
        Integer completeStatus = storeService.checkCompleteStatus(storeDTO);
        storeDTO.setCompleteStatus(completeStatus);
        // 分组策略
        String msg = getGroup(storeDTO);
        if (StringUtils.isNotBlank(msg)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), msg);
        }
        // 校验填写的品牌是否存在
        String brandMsg = validBrandIsExist(storeDTO.getBrandIds());
        if (StringUtils.isNotBlank(brandMsg)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), brandMsg);
        }
        // 启用策略
        getStatus(storeDTO);
        if (storeDTO.getStoreId() == null) {
            ServiceResponse<List<EnterpriseLicenseDTO>> listEnterpriseLicense = this.enterpriseApiService.listEnterpriseLicense(storeDTO.getEnterpriseId());
            if(listEnterpriseLicense.isSuccess() && CollectionUtils.isNotEmpty(listEnterpriseLicense.getResult())){
                boolean hasStoreLicense = listEnterpriseLicense.getResult()
                        .stream()
                        .anyMatch(e -> LicenseUtils.STORE_LICENSE_CODE.equals(e.getLicenseCode()));
                if(hasStoreLicense){
                    Integer storeLicenseNum = listEnterpriseLicense.getResult().stream()
                            .filter(e -> LicenseUtils.STORE_LICENSE_CODE.equals(e.getLicenseCode()))
                            .collect(Collectors.toList()).get(0).getUpperLimit();
                    storeDTO.setLicense(storeLicenseNum);
                }
            }
            Integer id = storeService.save(storeDTO);
            storeDTO.setStoreId(id);
            // 记录门店日志
            saveAddLog(storeDTO);
        } else {
            StoreBrandChangeDTO storeBrandChangeDTO = checkBrandHasChange(storeDTO.getBrandIds(), storeDTO.getStoreId(), storeDTO.getEnterpriseId());
            String lock = "lock:"+ storeDTO.getEnterpriseId() + ":" + storeDTO.getStoreId();
            RedisUtil.lock(lock, 500l, TimeUnit.MILLISECONDS, 500l);
            try{
                ServiceResponse<StoreDTO> existStore = this.getStoreById(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
                storeDTO.setStoreInfoId(existStore.getResult().getStoreInfoId());
                storeDTO.setStatus(storeDTO.getStatus() != null ? storeDTO.getStatus() : existStore.getResult().getStatus());
                Integer update = storeService.update(storeDTO);
                if (update == 0) {
                    return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
                }
                if(storeBrandChangeDTO.isChange()){
                    //调用归属重算
                    storeBrandChangeDTO.setStoreInfoIdList(Collections.singletonList(storeDTO.getStoreInfoId()));
                    this.storeUpdateApiService.storeBrandChange(storeDTO.getEnterpriseId(), storeBrandChangeDTO, storeDTO.getOperatorId());
                    logger.info("调用会员接口进行归属重算， {}", JSON.toJSONString(storeBrandChangeDTO));
                }
                // 记录门店日志
                try {
                    saveUpdateLog(existStore.getResult(), storeDTO);
                } catch (Exception e) {
                    logger.warn("门店编辑日志记录错误", e);
                }
                //修改门店同步到微盟
                updateStoreToWm(storeDTO);
            }catch (Exception e){
                logger.warn("门店更新异常", e);
            }finally {
                RedisUtil.unlock(lock);
            }
        }

        // 保存店长 如果店长保存失败 回滚数据
        ServiceResponse<ClerkDTO> headClerk = this.clerkApiService.getHeadClerk(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
        if(headClerk.isSuccess() && headClerk.getResult() == null){
            ServiceResponse<Void> serviceResponse = saveClerkLeader(storeDTO);
            if (!serviceResponse.isSuccess()) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
            }
        }

        // 自定义属性
        String error = storeExtendService.saveOrUpdate(storeDTO.getEnterpriseId(), storeDTO.getStoreInfoId(), storeDTO.getStoreExtendList());
        if (StringUtils.isNotBlank(error)) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), error);
        }
        // 门店图片
        List<Integer> delPhotos = null;
        if (StringUtils.isNotEmpty(storeDTO.getDelPhotos())) {
            List<String> photoIds = Arrays.asList(storeDTO.getDelPhotos().split(","));
            delPhotos = photoIds.stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        storePhotoService.editStoreImage(storeDTO.getEnterpriseId(), storeDTO.getStoreInfoId(), storeDTO.getPhotoList(), delPhotos);
        // 营业时间
        List<Integer> delBusinessTimes = null;
        if (StringUtils.isNotEmpty(storeDTO.getDelBusinessTimes())) {
            List<String> delBusinessTimeIds = Arrays.asList(storeDTO.getDelBusinessTimes().split(","));
            delBusinessTimes = delBusinessTimeIds.stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        storeBusinessTimeService.editStoreBusinessTime(storeDTO.getEnterpriseId(), storeDTO.getStoreInfoId(), storeDTO.getBusinessTimeList(), delBusinessTimes);
        // 更新缓存
        refreshCache(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
        return ServiceResponse.success(storeDTO);
    }

    private ServiceResponse<Void> saveClerkLeader(StoreDTO storeDTO) {
        ClerkDTO clerkLeader = storeDTO.getClerkLeader();
        if (clerkLeader != null) {
            clerkLeader.setStoreInfoId(storeDTO.getStoreInfoId());
            return clerkApiService.saveOrUpdate(clerkLeader);
        }
        return ServiceResponse.success();
    }

    private ServiceResponse<StoreDTO> valid(StoreDTO storeDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(storeDTO);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        boolean codeValid = storeService.validateStoreCodeIsExist(storeDTO.getEnterpriseId(), storeDTO.getRegionId(), storeDTO.getStoreCode(), storeDTO.getStoreId());
        if (codeValid) {
            return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "门店代码已经存在");
        }
        boolean nameValid = storeService.validateStoreNameIsExist(storeDTO.getEnterpriseId(), storeDTO.getRegionId(), storeDTO.getStoreName(), storeDTO.getStoreId());
        if (nameValid) {
            return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "门店名称已经存在");
        }
        StoreRegionDTO region = storeRegionService.getById(storeDTO.getEnterpriseId(), storeDTO.getRegionId());
        if (region == null) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店域不存在");
        }
        if (storeDTO.getStoreGroupId() != null) {
            Integer storeGroupId = storeDTO.getStoreGroupId();
            Boolean result = storeGroupApiService.isLeafNode(storeGroupId).getResult();
            if (!result) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店分组必须为叶子节点");
            }
        }
        if(org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getLatitude()) || org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getLongitude())){
            Matcher matcher = NUMBER_PATTERN.matcher(storeDTO.getLatitude());
            if(!matcher.matches()){
                storeDTO.setLatitude(null);
            }
            matcher = NUMBER_PATTERN.matcher(storeDTO.getLongitude());
            if(!matcher.matches()){
                storeDTO.setLongitude(null);
            }
        }
        return null;
    }

    @Override
    @Transactional
    public ServiceResponse<Integer> saveStoreForOrder(Integer enterpriseId, String storeCode, String storeName, Integer regionId, String orderNum) {
        return saveStoreForOrder(enterpriseId, storeCode, storeName, regionId, orderNum, null);
    }

    @Override
    @Transactional
    public ServiceResponse<Integer> saveStoreForOrder(Integer enterpriseId, String storeCode, String storeName, Integer regionId, String orderNum, Integer storeBrandId) {
        ServiceResponse<List<EnterpriseLicenseDTO>> listEnterpriseLicense = this.enterpriseApiService.listEnterpriseLicense(enterpriseId);
        Integer storeLicenseNum = 0;
        if(listEnterpriseLicense.isSuccess() && CollectionUtils.isNotEmpty(listEnterpriseLicense.getResult())){
            boolean hasStoreLicense = listEnterpriseLicense.getResult()
                    .stream()
                    .anyMatch(e -> LicenseUtils.STORE_LICENSE_CODE.equals(e.getLicenseCode()));
            if(hasStoreLicense){
                storeLicenseNum = listEnterpriseLicense.getResult().stream()
                        .filter(e -> LicenseUtils.STORE_LICENSE_CODE.equals(e.getLicenseCode()))
                        .collect(Collectors.toList()).get(0).getUpperLimit();
            }
        }
        Integer count = this.storeService.countByStoreName(enterpriseId, null, null);
        if(count >= storeLicenseNum){
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "门店license超出限制");
        }
        StoreDTO storeDTO = new StoreDTO();
        storeDTO.setEnterpriseId(enterpriseId);
        storeDTO.setStoreCode(storeCode);
        storeDTO.setStoreName(storeName);
        storeDTO.setRegionId(regionId);
        storeDTO.setStoreBrands(storeBrandId.toString());
        storeDTO.setCreateType(CreateTypeEnum.ORDER.getCode());
        ServiceResponse<StoreDTO> response = this.saveOrUpdate(storeDTO);
        if(response.isSuccess()){
            return ServiceResponse.success(response.getResult().getStoreInfoId());
        }
        return EnterpriseServiceResponse.failure(response.getCode(), response.getMessage());
    }

    @Override
    @Transactional
    public ServiceResponse<Integer> saveStoreForPosMember(Integer enterpriseId, String storeCode, String storeName, Integer regionId, String cardNo) {
        return saveStoreForPosMember(enterpriseId, storeCode, storeName, regionId, cardNo, null);
    }

    @Override
    @Transactional
    public ServiceResponse<Integer> saveStoreForPosMember(Integer enterpriseId, String storeCode, String storeName, Integer regionId, String cardNo, Integer storeBrandId) {
        ServiceResponse<List<EnterpriseLicenseDTO>> listEnterpriseLicense = this.enterpriseApiService.listEnterpriseLicense(enterpriseId);
        Integer storeLicenseNum = 0;
        if(listEnterpriseLicense.isSuccess() && CollectionUtils.isNotEmpty(listEnterpriseLicense.getResult())){
            boolean hasStoreLicense = listEnterpriseLicense.getResult()
                    .stream()
                    .anyMatch(e -> LicenseUtils.STORE_LICENSE_CODE.equals(e.getLicenseCode()));
            if(hasStoreLicense){
                storeLicenseNum = listEnterpriseLicense.getResult().stream()
                        .filter(e -> LicenseUtils.STORE_LICENSE_CODE.equals(e.getLicenseCode()))
                        .collect(Collectors.toList()).get(0).getUpperLimit();
            }
        }
        Integer count = this.storeService.countByStoreName(enterpriseId, null, null);
        if(count >= storeLicenseNum){
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "门店license超出限制");
        }
        StoreDTO storeDTO = new StoreDTO();
        storeDTO.setEnterpriseId(enterpriseId);
        storeDTO.setStoreCode(storeCode);
        storeDTO.setStoreName(storeName);
        storeDTO.setRegionId(regionId);
        storeDTO.setBrandIds(storeBrandId.toString());
        storeDTO.setCreateType(CreateTypeEnum.MEMBER.getCode());
        ServiceResponse<StoreDTO> response = this.saveOrUpdate(storeDTO);
        if(response.isSuccess()){
            return ServiceResponse.success(response.getResult().getStoreInfoId());
        }
        return EnterpriseServiceResponse.failure(response.getCode(), response.getMessage());
    }

    @Override
    public void refreshCache(Integer enterpriseId, Integer storeId) {
        String key = Constants.STORE_KEY + enterpriseId + ":" + storeId;
        RedisUtil.delCache(key);
    }

    @Override
    @Transactional
    public ServiceResponse<Void> tranOverFlow(Integer enterpriseId, String storeIds) {
        ServiceResponse<List<EnterpriseLicenseDTO>> listEnterpriseLicense = this.enterpriseApiService.listEnterpriseLicense(enterpriseId);
        if (listEnterpriseLicense.isSuccess() && CollectionUtils.isNotEmpty(listEnterpriseLicense.getResult())) {
            boolean hasStoreLicense = listEnterpriseLicense.getResult()
                    .stream()
                    .anyMatch(e -> LicenseUtils.STORE_LICENSE_CODE.equals(e.getLicenseCode()));
            if (hasStoreLicense) {
                Integer currCount = this.storeService.countByOverflowStatus(enterpriseId, 0);
                Integer upperLimit = listEnterpriseLicense.getResult().stream()
                        .filter(e -> LicenseUtils.STORE_LICENSE_CODE.equals(e.getLicenseCode()))
                        .collect(Collectors.toList()).get(0).getUpperLimit();
                // 剩余数量
                int remainingCount = upperLimit - currCount;
                if (storeIds.split(GlobalInfo.FLAG_COMMA).length > remainingCount) {
                    return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), String.format("本商户剩余门店数量为%s家， 请重新勾选正确的数量!", remainingCount));
                }
                List<Integer> storeIdList;
                if (Constants.ALL.equals(storeIds)) {
                    storeIdList = this.storeService.listAllStoreId(enterpriseId);
                } else {
                    storeIdList = Stream.of(storeIds.split(",")).map(Integer::valueOf).collect(Collectors.toList());
                }
                for (Integer storeId : storeIdList) {
                    StoreDTO store = this.getStoreById(enterpriseId, storeId).getResult();
                    if (store != null) {
                        store.setOverflowStatus(0);
                        saveOrUpdate(store);
                    }
                }
                return ServiceResponse.success();
            }
        }
        return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "商户没有门店license");
    }

    @Override
    public ServiceResponse<List<StoreDTO>> listStoreFromDb(Integer enterpriseId, StoreSearchDbDTO storeSearchDbDTO) {
        if(enterpriseId != null){
            storeSearchDbDTO.setEnterpriseId(enterpriseId);
            com.github.pagehelper.Page<StoreDTO> storeDTOS = this.storeService.listStore(storeSearchDbDTO, 1, Integer.MAX_VALUE);
            return ServiceResponse.success(storeDTOS.getResult());
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Boolean> hasUnCompleteStatusStore(Integer enterpriseId) {
        return ServiceResponse.success(this.storeService.hasUnCompleteStatusStore(enterpriseId));
    }

    @Override
    public ServiceResponse<Integer> countByOverflowStatus(Integer enterpriseId) {
        this.storeService.countByOverflowStatus(enterpriseId, 0);
        return EnterpriseServiceResponse.success(this.storeService.countByOverflowStatus(enterpriseId, 0));
    }

    /**
     * 修改日志
     */
    private void saveUpdateLog(StoreDTO oldStore, StoreDTO newStore) {
        StoreLogDTO storeLogDTO = new StoreLogDTO();
        storeLogDTO.setEnterpriseId(newStore.getEnterpriseId());
        storeLogDTO.setStoreId(newStore.getStoreId());
        storeLogDTO.setReason(newStore.getReason());
        storeLogDTO.setLogType(StoreLogTypeEnum.INFO_UPDATE.getCode());
        storeLogDTO.setRemark(newStore.getRemark());
        storeLogDTO.setOperatorId(newStore.getOperatorId());
        storeLogDTO.setOperatorName(newStore.getOperatorName());
        StringBuilder sb = getContent(oldStore, newStore);
        storeLogDTO.setContent(sb.toString());
        if (StringUtils.isNotBlank(storeLogDTO.getContent().trim())) {
            this.storeLogService.saveStoreLog(storeLogDTO);
        }
        if (newStore.getStoreGroupId() != null && !newStore.getStoreGroupId().equals(oldStore.getStoreGroupId())) {
            storeLogDTO.setLogType(StoreLogTypeEnum.GROUP_UPDATE.getCode());
            TabStoreGroup storeGroup = this.storeGroupService.getStoreGroupById(newStore.getStoreGroupId());
            storeLogDTO.setContent("门店分组变更为【" + storeGroup.getStoreGroupName() + "】");
            this.storeLogService.saveStoreLog(storeLogDTO);
        }
        if (newStore.getStatus() != null && !newStore.getStatus().equals(oldStore.getStatus())) {
            storeLogDTO.setLogType(StoreLogTypeEnum.STATUS_UPDATE.getCode());
            storeLogDTO.setContent("门店启用状态变更为【" + StoreStatusEnum.parseCode(newStore.getStatus()) + "】");
            this.storeLogService.saveStoreLog(storeLogDTO);
        }
    }

    private StringBuilder getContent(StoreDTO oldStore, StoreDTO newStore) {
        StringBuilder sb = new StringBuilder();
        if (StringUtils.isNotBlank(newStore.getStoreName()) && !StringUtils.equals(newStore.getStoreName(), oldStore.getStoreName())) {
            sb.append("门店名称").append("【").append(oldStore.getStoreName()).append("】")
                    .append("变更为【").append(newStore.getStoreName()).append("】");
        }
        if (CollectionUtils.isNotEmpty(oldStore.getPhotoList()) && CollectionUtils.isNotEmpty(newStore.getPhotoList())) {
            if (!CollectionUtils.isEqualCollection(oldStore.getPhotoList(), newStore.getPhotoList())) {
                sb.append("门店照片变更");
            }
        }
        if (newStore.getStoreType() != null && !newStore.getStoreType().equals(oldStore.getStoreType())) {
            sb.append("门店类型");
            if (oldStore.getStoreType() != null) {
                ServiceResponse<StoreDictDTO> oldStoreType = this.storeDictApiService.getStoreDict("storeType", oldStore.getStoreType().toString());
                sb.append("【").append(oldStoreType.getResult().getKey()).append("】");
            }
            ServiceResponse<StoreDictDTO> newStoreType = this.storeDictApiService.getStoreDict("storeType", newStore.getStoreType().toString());
            sb.append("变更为【").append(newStoreType.getResult().getKey()).append("】");
        }
        if (newStore.getRealStatus() != null && !newStore.getRealStatus().equals(oldStore.getRealStatus())) {
            sb.append("门店状态");
            if (oldStore.getRealStatus() != null) {
                ServiceResponse<StoreDictDTO> oldStoreStatus = this.storeDictApiService.getStoreDict("storeStatus", oldStore.getRealStatus().toString());
                sb.append("【").append(oldStoreStatus.getResult().getKey()).append("】");
            }
            ServiceResponse<StoreDictDTO> newStoreStatus = this.storeDictApiService.getStoreDict("storeStatus", newStore.getRealStatus().toString());
            sb.append("变更为【").append(newStoreStatus.getResult().getKey()).append("】");
        }
        if (StringUtils.isNotBlank(newStore.getConactsPhone()) && !StringUtils.equals(newStore.getConactsPhone(), oldStore.getConactsPhone())) {
            sb.append("门店电话").append("【").append(oldStore.getConactsPhone()).append("】")
                    .append("变更为【").append(newStore.getConactsPhone()).append("】");
        }
        if (CollectionUtils.isNotEmpty(newStore.getBusinessTimeList())) {
            if (CollectionUtils.isEmpty(oldStore.getBusinessTimeList()) || !CollectionUtils.isEqualCollection(oldStore.getBusinessTimeList(), newStore.getBusinessTimeList())) {
                if (StringUtils.isNotBlank(newStore.getBusinessTimeList().get(0).getWeekday())) {
                    this.storeBusinessTimeService.convertBusinessTime(newStore.getBusinessTimeList());
                    String newBusinessTimeShow = newStore.getBusinessTimeList().stream()
                            .map(e -> e.getWeekdayShow() + ":" + DateUtil.dateToStr(e.getOpenTime(), DateUtil.FORMAT_MINUTE_TIME) + "~" + DateUtil.dateToStr(e.getCloseTime(), DateUtil.FORMAT_MINUTE_TIME))
                            .collect(Collectors.joining(","));
                    sb.append("营业时间").append("变更为【").append(newBusinessTimeShow).append("】");
                }
            }
        }
        if (StringUtils.isNotBlank(newStore.getBrandIds()) && !StringUtils.equals(newStore.getBrandIds(), oldStore.getBrandIds())) {
            List<TabStoreBrand> tabStoreBrands = this.storeBrandService.listStoreBrandByIds(newStore.getBrandIds());
            String storeBrandShow = tabStoreBrands.stream().map(TabStoreBrand::getStoreBrandName).collect(Collectors.joining(","));;
            sb.append("门店品牌").append("变更为【").append(storeBrandShow).append("】");
        }
        if (StringUtils.isNotBlank(newStore.getAddress()) && !StringUtils.equals(newStore.getAddress(), oldStore.getAddress())) {
            sb.append("门店地址").append("【").append(oldStore.getAddress()).append("】")
                    .append("变更为【").append(newStore.getAddress()).append("】");
        }
        if (StringUtils.isNotBlank(newStore.getLongitude()) && !StringUtils.equals(newStore.getLongitude(), oldStore.getLongitude())) {
            if (StringUtils.isNotBlank(newStore.getLatitude()) && !StringUtils.equals(newStore.getLatitude(), oldStore.getLatitude())) {
                sb.append("门店经纬度").append("【").append(oldStore.getLongitude()).append(",").append(oldStore.getLatitude()).append("】")
                        .append("变更为【").append(newStore.getLongitude()).append(",").append(newStore.getLatitude()).append("】");
            }
        }
        return sb;
    }

    /**
     * 新增日志
     */
    private void saveAddLog(StoreDTO storeDTO) {
        StoreLogDTO storeLogDTO = new StoreLogDTO();
        storeLogDTO.setEnterpriseId(storeDTO.getEnterpriseId());
        storeLogDTO.setStoreId(storeDTO.getStoreId());
        storeLogDTO.setContent("门店新增");
        storeLogDTO.setReason(storeDTO.getReason());
        storeLogDTO.setLogType(StoreLogTypeEnum.SAVE.getCode());
        storeLogDTO.setRemark(storeDTO.getRemark());
        storeLogDTO.setOperatorId(storeDTO.getOperatorId());
        storeLogDTO.setOperatorName(storeDTO.getOperatorName());

        this.storeLogService.saveStoreLog(storeLogDTO);
    }

    private String validBrandIsExist(String brandIds) {
        if (StringUtils.isNotBlank(brandIds)) {
            List<TabStoreBrand> storeBrandList = storeBrandService.listStoreBrandByIds(brandIds);
            if (CollectionUtils.isEmpty(storeBrandList) || brandIds.split(GlobalInfo.FLAG_COMMA).length > storeBrandList.size()) {
                return "品牌不存在";
            }
        }
        return null;
    }

    private StoreBrandChangeDTO checkBrandHasChange(String brandIds, Integer storeId, Integer enterpriseId){
        StoreBrandChangeDTO storeBrandChangeDTO = new StoreBrandChangeDTO();
        StoreDTO result = this.getStoreById(enterpriseId, storeId).getResult();
        if(result!= null){
            if(brandIds.equals(result.getBrandIds())){
                return storeBrandChangeDTO;
            }
        }
        storeBrandChangeDTO.setChange(true);
        Map<String, String> newBrandIdMap = Arrays.stream(brandIds.split(",")).collect(Collectors.toMap(id -> id, id -> id));
        Map<String, String> oldBrandIdMap = Arrays.stream(result.getBrandIds().split(",")).collect(Collectors.toMap(id -> id, id -> id));
        MapDifference<String, String> difference = Maps.difference(newBrandIdMap, oldBrandIdMap);
        Map<String, String> add = difference.entriesOnlyOnLeft();
        Map<String, String> del = difference.entriesOnlyOnRight();
        if(MapUtils.isNotEmpty(add)){
            storeBrandChangeDTO.setAdd(add.keySet().stream().map(s->Integer.valueOf(s)).collect(Collectors.toList()));
        }
        if(MapUtils.isNotEmpty(del)){
            storeBrandChangeDTO.setDel(del.keySet().stream().map(s->Integer.valueOf(s)).collect(Collectors.toList()));
        }
        return storeBrandChangeDTO;
    }

    private void refreshIndex(Integer storeId, Integer enterpriseId) {
        this.addStoreToIndex(enterpriseId, storeId);
    }

    /**
     * 根据策略判断启用状态
     * @param storeDTO
     */
    private void getStatus(StoreDTO storeDTO) {
        // 所有必填字段都填之后才能走策略 不然就是未启用  必填字段一共有门店名称 门店代码 门店域  店招品牌，
        // 门店名称 门店代码 门店域没有的话不能创建  能走到这里只有店招品牌有可能没有
        if(storeDTO.getStatus() == null){
            if (StringUtils.isNotBlank(storeDTO.getBrandIds())) {
                String hitStrategy = storeStrategyService.isHitStrategy(storeDTO, StoreGroupConstant.STORE_STRATEGY_TYPE);
                storeDTO.setStatus(StringUtils.isNotBlank(hitStrategy) ? Integer.valueOf(hitStrategy) : StoreStatusEnum.INONLINE.getCode());
            } else {
                storeDTO.setStatus(StoreStatusEnum.INONLINE.getCode());
            }
        }
    }

    /**
     * 根据策略判断分组
     * @param storeDTO
     * @return
     */
    private String getGroup(StoreDTO storeDTO) {
        if (storeDTO.getIsEditStoreGroup() == 1 && Integer.valueOf(1).equals(storeStrategyApiService.getStoreGroupStrategyWeight(storeDTO.getEnterpriseId()).getResult())) {
            // 不启用分组策略
            if (storeDTO.getStoreGroupId() == null) {
                // 取 未分类 分组
                TabStoreGroup group = storeGroupService.selectUnGroupedStore(storeDTO.getEnterpriseId());
                storeDTO.setStoreGroupId(group.getStoreGroupId());
            } else {
                // 判断分组是否存在
                TabStoreGroup group = storeGroupService.getStoreGroupById(storeDTO.getStoreGroupId());
                if (group == null) {
                    logger.info("enterpriseId:{}, groupName:{}, 不存在该分组", storeDTO.getEnterpriseId(), storeDTO.getStoreGroupName());
                    return "门店分组不存在";
                }
                if (storeGroupService.isLeafNode(group.getStoreGroupId())) {
                    // 如果是叶子节点
                    storeDTO.setStoreGroupId(group.getStoreGroupId());
                } else {
                    logger.info("enterpriseId:{}, groupName:{}, groupId:{}, 分组不是叶子节点", storeDTO.getEnterpriseId(), storeDTO.getStoreGroupName(), group.getStoreGroupId());
                    return "分组不是叶子节点";
                }
            }
        } else {
            // 启用分组策略
            String groupId = storeStrategyService.isHitStrategy(storeDTO, StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE);
            if (StringUtils.isNotBlank(groupId)) {
                // 命中
                storeDTO.setStoreGroupId(Integer.valueOf(groupId));
            } else {
                // 未命中
                if (storeDTO.getStoreGroupId() == null) {
                    TabStoreGroup group = storeGroupService.selectUnGroupedStore(storeDTO.getEnterpriseId());
                    storeDTO.setStoreGroupId(group.getStoreGroupId());
                }
            }
        }
        return null;
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> listStore(StoreSearchDTO storeSearchDTO, Integer pageNum, Integer pageSize, String returnFileds) {
        JSONObject enterpriseJson = this.buildSearchJson(storeSearchDTO);
        if(enterpriseJson == null){ //null表示没有权限
            return ServiceResponse.success(new Page<>(pageNum, pageSize));
        }
        JSONObject sortJson = null;
        if(StringUtils.isNotBlank(storeSearchDTO.getLatitude()) && StringUtils.isNotBlank(storeSearchDTO.getLongitude())){
            sortJson = QueryConditionAssemblyUtil.createSortNodeByLocation(StoreESFieldsEnum.LOCATION.getField(), QueryConditionAssemblyUtil.SORT_RULE_ASC,
                    Double.valueOf(storeSearchDTO.getLatitude()), Double.valueOf(storeSearchDTO.getLongitude()));
        }else {
            sortJson = QueryConditionAssemblyUtil.createSortNode(StoreESFieldsEnum.STATUS.getField(), QueryConditionAssemblyUtil.SORT_RULE_ASC);
        }
        ServiceResponse<ESResponseQueryBatchDTO> response = this.queryStoreFromEs(enterpriseJson, pageNum, pageSize, sortJson, returnFileds);
        if (response.isSuccess() && response.getResult() != null) {
            Page<StoreDTO> page = new Page(pageNum, pageSize);
            page.setTotalCount(Integer.valueOf(response.getResult().getTotalNumber() + ""));
            List<JSONObject> list = response.getResult().getRes();
            List<StoreDTO> result = new ArrayList<>();
            for(int i=0; i<list.size(); i++){
                StoreDTO storeDTO = JSON.toJavaObject(list.get(i), StoreDTO.class);
                storeDTO.setAreaId(storeDTO.getCountyId());
                if(StringUtils.isNotBlank(storeSearchDTO.getLatitude()) && StringUtils.isNotBlank(storeSearchDTO.getLongitude()) &&
                        CollectionUtils.isNotEmpty(storeDTO.getLocation())){
                    double v = GeoUtils.arcDistance(Double.valueOf(storeSearchDTO.getLatitude()), Double.valueOf(storeSearchDTO.getLongitude()),
                            storeDTO.getLocation().get(1), storeDTO.getLocation().get(0));
                    storeDTO.setDistance(v);
                }
                result.add(storeDTO);
            }
            page.setResult(result);
            return ServiceResponse.success(page);
        }
        return ServiceResponse.success(new Page<>(pageNum, pageSize));
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> listStore(StoreSearchDTO storeSearchDTO, Integer pageNum, Integer pageSize){
        StoreESFieldsEnum[] values = StoreESFieldsEnum.values();
        StringBuilder returnFileds = new StringBuilder();
        for(int i=0; i< values.length; i++){
            if(i == values.length-1){
                returnFileds.append(values[i].getField());
            }else {
                returnFileds.append(values[i].getField()).append(",");
            }
        }
        return this.listStore(storeSearchDTO, pageNum, pageSize, returnFileds.toString());
    }

    @Override
    public ServiceResponse<List<StoreDTO>> listStore(Integer userId) {
        logger.info("查询管理员的权限门店数据参数:{}", userId);
        ServiceResponse<UserDTO> userResponse = userApiService.getUserInfoById(userId);
        if (userResponse.isSuccess()) {
            UserDTO user = userResponse.getResult();
            //组装门店查询条件
            StoreSearchDTO dto = new StoreSearchDTO();
            dto.setEnterpriseId(user.getEnterpriseId());

            if (user.getSuperAdmin() != 1) {
                //子管理员查询资源组配置门店资源信息
                ServiceResponse<ResourceGroupDTO> resourceResponse = userResourceApiService.getResourceGroupByUserId(userId);
                if (resourceResponse.isSuccess()) {
                    ResourceGroupDTO resourceGroupDTO = resourceResponse.getResult();
                    dto.setStoreResource(resourceGroupDTO.getStoreResource());
                }
            }
            //查询门店
            ServiceResponse<Page<StoreDTO>> storeResponse = listStore(dto, 1, 10000);
            if (storeResponse.isSuccess()) {
                Page<StoreDTO> page = storeResponse.getResult();
                if (page != null) {
                    return ServiceResponse.success(page.getResult());
                }
            }
            return ServiceResponse.failure(storeResponse.getCode(), storeResponse.getMessage());
        }
        return ServiceResponse.failure(userResponse.getCode(), userResponse.getMessage());
    }

    @Override
    public ServiceResponse<Map<Integer, List<String>>> listStorePhotoByStoreIds(Integer enterpriseId, List<Integer> storeIdList) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStoreIds(Joiner.on(" ").join(storeIdList));
        ServiceResponse<Page<StoreDTO>> serviceResponse = this.listStore(storeSearchDTO, 1, 20);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }

        Map<Integer, List<String>> resultMap = new HashMap<>(16);
        List<StoreDTO> storeDTOList = serviceResponse.getResult().getResult();
        for (StoreDTO storeDTO : storeDTOList) {
            List<StorePhotoDTO> photoList = storeDTO.getPhotoList();
            List<String> list = Optional.ofNullable(photoList).orElse(Collections.emptyList()).stream().map(StorePhotoDTO::getImageUrl).collect(Collectors.toList());
            // 给一个门店图片默认值测试
            if (CollectionUtils.isNotEmpty(list)) {
                list = Arrays.asList(Constants.DEFAULT_PHOTO);
            }
            resultMap.put(storeDTO.getStoreId(), list);
        }

        return EnterpriseServiceResponse.success(resultMap);
    }

    @Override
    public ServiceResponse<Map<Integer, List<String>>> listStorePhotoByStoreInfoIds(Integer enterpriseId, List<Integer> storeInfoIdList) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStoreInfoIds(Joiner.on(" ").join(storeInfoIdList));
        ServiceResponse<Page<StoreDTO>> serviceResponse = this.listStore(storeSearchDTO, 1, 20);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }

        Map<Integer, List<String>> resultMap = new HashMap<>(16);
        List<StoreDTO> storeDTOList = serviceResponse.getResult().getResult();
        for (StoreDTO storeDTO : storeDTOList) {
            List<StorePhotoDTO> photoList = storeDTO.getPhotoList();
            List<String> list = Optional.ofNullable(photoList).orElse(Collections.emptyList()).stream().map(StorePhotoDTO::getImageUrl).collect(Collectors.toList());
            // 给一个门店图片默认值测试
            if (CollectionUtils.isNotEmpty(list)) {
                list = Arrays.asList(Constants.DEFAULT_PHOTO);
            }
            resultMap.put(storeDTO.getStoreInfoId(), list);
        }
        return EnterpriseServiceResponse.success(resultMap);
    }

    @Override
    public ServiceResponse queryStoreFromEs(JSONObject jsonObject, Integer pageNum, Integer pageSize, JSONObject sortJson, String returnFileds) {
        if (StringUtils.isBlank(returnFileds)) {
            returnFileds = StoreESFieldsEnum.STOREID.getField();
        }
        DynamicSearchDTO dynamicSearchDTO = new DynamicSearchDTO();
        dynamicSearchDTO.setSearchJson(jsonObject);
        dynamicSearchDTO.setEnterpriseId(com.gic.enterprise.constants.Constants.INDEX_ENTERPRISEID);
        dynamicSearchDTO.setColumnCategoryCode(com.gic.enterprise.constants.Constants.ColumnCategoryCode);
        dynamicSearchDTO.setBegin((pageNum - 1) * pageSize);
        dynamicSearchDTO.setRecordNumber(pageSize > 10000 ? 10000 : pageSize);
        dynamicSearchDTO.setSortJson(sortJson);
        dynamicSearchDTO.setReturnFileds(returnFileds);
        logger.info("es查询条件：{}", JSON.toJSONString(dynamicSearchDTO,true));
        ServiceResponse<ESResponseQueryBatchDTO> response = this.esBusinessOperaApiService.queryDataBatch(dynamicSearchDTO, false, null);
        return response;
    }

    @Override
    public ServiceResponse<Long> queryStoreCountFromEs(StoreSearchDTO storeSearchDTO){
        JSONObject json = this.buildSearchJson(storeSearchDTO);
        logger.info("es查询门店数量json：{}", JSON.toJSONString(json));
        DynamicSearchDTO dynamicSearchDTO = new DynamicSearchDTO();
        dynamicSearchDTO.setSearchJson(json);
        dynamicSearchDTO.setEnterpriseId(com.gic.enterprise.constants.Constants.INDEX_ENTERPRISEID);
        dynamicSearchDTO.setColumnCategoryCode(com.gic.enterprise.constants.Constants.ColumnCategoryCode);
        ServiceResponse<Long> response = this.esBusinessOperaApiService.queryDataCount(dynamicSearchDTO);
        if(response.isSuccess()){
            return response;
        }
        return ServiceResponse.success(0L);
    }

    @Override
    public ServiceResponse<Void> deleteStoreFromEs(Integer enterpriseId, Integer storeId) {
        logger.info("es删除门店：{}，{}", enterpriseId, storeId);
        JSONObject json = new JSONObject();
        json.put(StoreESFieldsEnum.ID.getField(), enterpriseId+"_"+storeId);
        DynamicSearchDTO dynamicSearchDTO = new DynamicSearchDTO();
        dynamicSearchDTO.setSearchJson(json);
        dynamicSearchDTO.setEnterpriseId(com.gic.enterprise.constants.Constants.INDEX_ENTERPRISEID);
        dynamicSearchDTO.setColumnCategoryCode(com.gic.enterprise.constants.Constants.ColumnCategoryCode);
        return this.esBusinessOperaApiService.deleteDataSingle(dynamicSearchDTO);
    }

    @Override
    public ServiceResponse<JSONObject> parseStoreSelectJson(String searchJson, Integer enterpriseId) {
        JSONArray array = JSON.parseArray(searchJson);
        JSONObject result = null;
        for(int i=0; i< array.size(); i++){
            JSONObject jsonObject = array.getJSONObject(i);
            String type = jsonObject.getString("type");
            JSONArray list = jsonObject.getJSONArray("list");
            List<JSONObject> jsonObjectList = new ArrayList<>();
            for(int j =0 ;j< list.size(); j++){
                JSONObject data = list.getJSONObject(j);
                String key = data.getJSONObject("data").getString("key");
                logger.info("11111:{}", key);
                //前端调整，把数据改为逗号隔开了
                data.getJSONObject("data").put("value", data.getJSONObject("data").getString("value").replaceAll(",", " "));

                if(StoreESFieldsEnum.STORETAG.getField().equals(key)){
                    JSONObject json = new JSONObject();
                    json.put("type", OperateEnum.OPERATE_OR.getValue());
                    List<JSONObject> listTag = new ArrayList<>();
                    String[] valueArr = data.getJSONObject("data").getString("value").split(" ");
                    for(String s : valueArr){
                        JSONObject jsonTmp = new JSONObject();
                        jsonTmp.put("type", OperateEnum.OPERATE_AND.getValue());
                        List<JSONObject> listTmp = new ArrayList<>();
                        TabStoreTag storeTag = this.storeTagService.getById(Integer.valueOf(s));
                        if(storeTag == null) {
                            continue;
                        }
                        StoreWidgetDTO storeWidget = this.storeWidgetService.getStoreWidget(storeTag.getStoreWidgetId());
                        if(storeWidget == null) {
                            continue;
                        }
                        if(StringUtils.isNotBlank(storeWidget.getAuthSearchParam()) && !"{}".equals(storeWidget.getAuthSearchParam())){
                            ServiceResponse<JSONObject> response = this.parseStoreSelectJson(storeWidget.getAuthSearchParam(), enterpriseId);
                            if(response.isSuccess() && response.getResult() != null){
                                listTmp.add(response.getResult());
                            }
                        }
                        if(StringUtils.isNotBlank(storeWidget.getSearchParam()) && !"{}".equals(storeWidget.getSearchParam())){
                            ServiceResponse<JSONObject> response = this.parseStoreSelectJson(storeWidget.getSearchParam(), enterpriseId);
                            if(response.isSuccess() && response.getResult() != null){
                                listTmp.add(response.getResult());
                            }
                        }
                        jsonTmp.put("list", listTmp);
                        listTag.add(jsonTmp);
                    }
                    logger.info("storeTag:{}", JSON.toJSONString(listTag));
                    json.put("list", listTag);
                    list.getJSONObject(j).clear();
                    list.getJSONObject(j).put("type", OperateEnum.OPERATE_OR.getValue());
                    list.getJSONObject(j).put("list", listTag);
                }else {
                    data.put("type", "data");
                    String compute = data.getJSONObject("data").getString("compute");

                    dealCustomFieldData(data.getJSONObject("data"), enterpriseId);

                    if(OperateEnum.OPERATE_BETWEEN.getValue().equals(compute)){
                        data.getJSONObject("data").put("value", data.getJSONObject("data").getString("value").replaceAll(" ", ","));
                    }
                }
                jsonObjectList.add(data);
            }
            JSONObject json = null;
            if(i > 0 && OperateEnum.OPERATE_AND_NOT.getValue().equals(type)){
                json = QueryConditionAssemblyUtil.createListQueryNode(jsonObjectList, OperateEnum.OPERATE_AND_NOT);
            }else {
                json = QueryConditionAssemblyUtil.createListQueryNode(jsonObjectList, OperateEnum.OPERATE_AND);
            }
            if(result == null){
                result = json;
            }else {
                if(OperateEnum.OPERATE_OR.getValue().equals(type)){
                    result = QueryConditionAssemblyUtil.addQueryNodeForSiblin(result, json, OperateEnum.OPERATE_OR);
                }else if(OperateEnum.OPERATE_AND_NOT.getValue().equals(type)){
                    result = QueryConditionAssemblyUtil.addQueryNodeForSiblin(result, json, OperateEnum.OPERATE_AND);
                }
            }
        }
        return ServiceResponse.success(result);
    }

    private void dealCustomFieldData(JSONObject jsonObject, Integer enterpriseId) {
        String key = jsonObject.getString("key");
        if (getStoreFieldKey().contains(key)) {
            //自定义字段
            String keyC = StoreRedisKeyUtils.getStoreFieldKeyC(enterpriseId, "");
            Object storeFieldId = RedisUtil.getCache(keyC + key);
            if (storeFieldId != null) {
                TabStoreField tabStoreField = storeFieldService.getStoreFieldById(Integer.parseInt(storeFieldId.toString()));
                if (tabStoreField != null) {
                    if (StoreFieldTypeEnum.DATE.getCode() == tabStoreField.getStoreFieldType()) {
                        //时间类型的数据需要转化
                        JSONObject json = JSON.parseObject(tabStoreField.getStoreFieldDetail());
                        int dateType = json.getIntValue("dateType");

                        StoreFieldDateEnum dateEnum = StoreFieldDateEnum.getByCode(dateType);
                        logger.info("门店选择器-自定义属性-时间类型数值:{}", jsonObject.getString("value"));
                        String[] valueArr = jsonObject.getString("value").split(" ");
                        SimpleDateFormat sdf = new SimpleDateFormat(dateEnum.getFormat());

                        Calendar calendar = Calendar.getInstance();
                        calendar.setTimeInMillis(Long.valueOf(valueArr[0]));

                        Calendar calendarSecond = Calendar.getInstance();
                        calendarSecond.setTimeInMillis(Long.valueOf(valueArr[1]));

                        if (dateEnum == StoreFieldDateEnum.YM || dateEnum == StoreFieldDateEnum.YMDHMS || dateEnum == StoreFieldDateEnum.YMD) {
                            jsonObject.put("value", sdf.format(calendar.getTime()) + "," + sdf.format(calendarSecond.getTime()));
                        } else {
                            //为了下面统一replaceAll
                            jsonObject.put("value", valueArr[0] + "," + valueArr[1]);
                        }
                        jsonObject.put("value", jsonObject.getString("value").replaceAll("-", "").
                                replaceAll(":", "").replaceAll(" ", ""));

                    }
                }
            }
        }
    }

    /**
     * 自定义字段的key
     * @return
     */
    private static Set<String> getStoreFieldKey() {
        Set<String> storeField = new HashSet<>();
        storeField.add(StoreESFieldsEnum.C1.getField());
        storeField.add(StoreESFieldsEnum.C2.getField());
        storeField.add(StoreESFieldsEnum.C3.getField());
        storeField.add(StoreESFieldsEnum.C4.getField());
        storeField.add(StoreESFieldsEnum.C5.getField());
        storeField.add(StoreESFieldsEnum.C6.getField());
        storeField.add(StoreESFieldsEnum.C7.getField());
        storeField.add(StoreESFieldsEnum.C8.getField());
        storeField.add(StoreESFieldsEnum.C9.getField());
        storeField.add(StoreESFieldsEnum.C10.getField());
        return storeField;
    }

    @Override
    public ServiceResponse<List<StoreDTO>> listStoreByStoreInfoId(Integer storeInfoId) {
        StoreSearchDbDTO searchDbDTO = new StoreSearchDbDTO();
        searchDbDTO.setStoreInfoId(storeInfoId);
        com.github.pagehelper.Page<StoreDTO> storeDTOS = this.storeService.listStore(searchDbDTO, 1, 10000);
        return ServiceResponse.success(storeDTOS.getResult());
    }

    @Override
    public ServiceResponse<List<StoreDTO>> listStore(String search, Integer enterpriseId) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setSearch(search);
        ServiceResponse<Page<StoreDTO>> response = this.listStore(storeSearchDTO, 1, 10000);
        if(response.isSuccess() && response.getResult() != null){
            return ServiceResponse.success(response.getResult().getResult());
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> pageNearbyStore(WxSearchStoreSearchDTO nearByStoreSearchDTO) {
        ServiceResponse<CustomStoreDTO> customStore = this.customStoreApiService.getCustomStore(nearByStoreSearchDTO.getEnterpriseId(), nearByStoreSearchDTO.getAppid());
        if(customStore.isSuccess() && customStore.getResult() != null){
            Integer storeWidgetId = customStore.getResult().getStoreWidgetId();
            StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
            storeSearchDTO.setEnterpriseId(nearByStoreSearchDTO.getEnterpriseId());
            storeSearchDTO.setStoreBrandId(nearByStoreSearchDTO.getBrandId());
            storeSearchDTO.setLongitude(nearByStoreSearchDTO.getLongitude());
            storeSearchDTO.setLatitude(nearByStoreSearchDTO.getLatitude());
            storeSearchDTO.setCityId(nearByStoreSearchDTO.getCityId());
            if(storeWidgetId != null){
                storeSearchDTO.setStoreResource(Long.valueOf(storeWidgetId+""));
            }
            ServiceResponse<Page<StoreDTO>> storePage = this.listStore(storeSearchDTO, nearByStoreSearchDTO.getPageNum(), nearByStoreSearchDTO.getPageSize());
            List<StoreDTO> list = storePage.getResult().getResult();
            StorePhotoDTO storePhotoDTO = new StorePhotoDTO();
            storePhotoDTO.setImageUrl(Constants.DEFAULT_PHOTO);
            List<StorePhotoDTO> defaultList = Collections.singletonList(storePhotoDTO);
            if (CollectionUtils.isNotEmpty(list)) {
                for (StoreDTO storeDTO : list) {
                    if (CollectionUtils.isNotEmpty(storeDTO.getLocation())) {
                        storeDTO.setLongitude(storeDTO.getLocation().get(0).toString());
                        storeDTO.setLatitude(storeDTO.getLocation().get(1).toString());
                    }
                    if (CollectionUtils.isEmpty(storeDTO.getPhotoList())) {
                        storeDTO.setPhotoList(defaultList);
                    }
                }
            }
            logger.info("pageNearbyStore:{}", JSON.toJSONString(storePage, true));
            return ServiceResponse.success(storePage.getResult());
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> pageCoupApplyStore(WxSearchStoreSearchDTO coupApplyStoreSearchDTO) {
        Integer storeWidgetId = coupApplyStoreSearchDTO.getStoreWidgetId();
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(coupApplyStoreSearchDTO.getEnterpriseId());
        storeSearchDTO.setStoreBrandId(coupApplyStoreSearchDTO.getBrandId());
        storeSearchDTO.setLongitude(coupApplyStoreSearchDTO.getLongitude());
        storeSearchDTO.setLatitude(coupApplyStoreSearchDTO.getLatitude());
        storeSearchDTO.setCityId(coupApplyStoreSearchDTO.getCityId());
        storeSearchDTO.setStoreResourceList(coupApplyStoreSearchDTO.getStoreWidgetIdList());
        if(coupApplyStoreSearchDTO.getStoreWidgetId() != null){
            storeSearchDTO.setStoreResource(Long.valueOf(storeWidgetId+""));
        }
        ServiceResponse<Page<StoreDTO>> storePage = this.listStore(storeSearchDTO, coupApplyStoreSearchDTO.getPageNum(), coupApplyStoreSearchDTO.getPageSize());
        logger.info("pageCoupApplyStore:{}", JSON.toJSONString(storePage, true));
        return ServiceResponse.success(storePage.getResult());
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> pageOpenCardStore(WxSearchStoreSearchDTO wxSearchStoreSearchDTO) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(wxSearchStoreSearchDTO.getEnterpriseId());
        storeSearchDTO.setStoreBrandId(wxSearchStoreSearchDTO.getBrandId());
        storeSearchDTO.setLongitude(wxSearchStoreSearchDTO.getLongitude());
        storeSearchDTO.setLatitude(wxSearchStoreSearchDTO.getLatitude());
        storeSearchDTO.setCityId(wxSearchStoreSearchDTO.getCityId());
        storeSearchDTO.setStoreResourceList(wxSearchStoreSearchDTO.getStoreWidgetIdList());
        storeSearchDTO.setStatus(com.gic.enterprise.constants.Constants.NORMAL_STATUS);
        ServiceResponse<Page<StoreDTO>> storePage = this.listStore(storeSearchDTO, wxSearchStoreSearchDTO.getPageNum(), wxSearchStoreSearchDTO.getPageSize());
        logger.info("pageCoupApplyStore:{}", JSON.toJSONString(storePage, true));
        return ServiceResponse.success(storePage.getResult());
    }

    @Override
    public ServiceResponse<Void> storeQrcodeCallBack(String param) {
        return null;
    }

    @Override
    @Transactional
    public ServiceResponse<String> importStoreForOperation(ProcessBatchStoreDTO processBatchStoreDTO) {
        logger.info("批量处理门店:{}", JSON.toJSONString(processBatchStoreDTO));
        StoreDTO storeDTO = new StoreDTO();
        ServiceResponse enterpriseResult = enterpriseApiService.getEnterpriseById(processBatchStoreDTO.getEnterpriseId());
        if (!enterpriseResult.isSuccess()) {
            return responseErrorElement(ImportStoreBatchResultEnum.E1);
        }
        storeDTO.setEnterpriseId(processBatchStoreDTO.getEnterpriseId());
        if (processBatchStoreDTO.getRegionId() == null) {
            return responseErrorElement(ImportStoreBatchResultEnum.A1);
        }
        StoreRegionDTO storeRegion = storeRegionService.getById(processBatchStoreDTO.getEnterpriseId(), processBatchStoreDTO.getRegionId());
        if (storeRegion == null) {
            return responseErrorElement(ImportStoreBatchResultEnum.A2);
        }
        storeDTO.setRegionId(storeRegion.getRegionId());
        if (StringUtils.isBlank(processBatchStoreDTO.getStoreName())) {
            return responseErrorElement(ImportStoreBatchResultEnum.A5);
        }
        storeDTO.setStoreName(processBatchStoreDTO.getStoreName());
        if (StringUtils.isBlank(processBatchStoreDTO.getStoreCode())) {
            return responseErrorElement(ImportStoreBatchResultEnum.A4);
        }
        storeDTO.setStoreCode(processBatchStoreDTO.getStoreCode());
        TabStore oldStore = this.storeService.getByStoreCodeAndRegionId(processBatchStoreDTO.getEnterpriseId(), processBatchStoreDTO.getRegionId(), processBatchStoreDTO.getStoreCode());
        if (oldStore != null) {
            storeDTO.setStoreId(oldStore.getStoreId());
        }
        // 店招品牌
        if (processBatchStoreDTO.getBrandId() != null) {
            TabStoreBrand storeBrand = storeBrandService.getById(processBatchStoreDTO.getBrandId());
            if (storeBrand == null) {
                return responseErrorElement(ImportStoreBatchResultEnum.B1);
            }
            storeDTO.setBrandIds(processBatchStoreDTO.getBrandId().toString());
        }else if (StringUtils.isNotBlank(processBatchStoreDTO.getBrandName())) {
            TabStoreBrand storeBrand = storeBrandService.getByStoreBrandName(processBatchStoreDTO.getEnterpriseId(), processBatchStoreDTO.getBrandName());
            if (storeBrand == null) {
                return responseErrorElement(ImportStoreBatchResultEnum.B1);
            }
            storeDTO.setBrandIds(storeBrand.getStoreBrandId().toString());
        }
        // 门店类型
        ServiceResponse<String> x = validStoreType(processBatchStoreDTO, storeDTO);
        if (x != null) {
            return x;
        }
        // 验证省市区数据的正确性
        validArea(processBatchStoreDTO, storeDTO);
        storeDTO.setStoreGroupName(processBatchStoreDTO.getStoreGroupName());
        TabStoreGroup group = storeGroupService.getStoreGroupByName(processBatchStoreDTO.getStoreGroupName(), processBatchStoreDTO.getEnterpriseId());
        if (group != null) {
            storeDTO.setStoreGroupId(group.getStoreGroupId());
        }

        List<StoreFieldDTO> fieldList = storeFieldService.listStoreFieldByRegionId(processBatchStoreDTO.getRegionId());
        String customField = processBatchStoreDTO.getCustomField();
        if (StringUtils.isBlank(customField)) {
            customField = "";
        }
        ServiceResponse<String> fieldResponse = validCustomField(storeDTO, fieldList, customField);
        if (fieldResponse != null) {
            return fieldResponse;
        }
        storeDTO.setConactsPhone(processBatchStoreDTO.getConactsPhone());
        storeDTO.setAddress(processBatchStoreDTO.getAddress());
        storeDTO.setLongitude(processBatchStoreDTO.getLongitude());
        storeDTO.setLatitude(processBatchStoreDTO.getLatitude());
        storeDTO.setReason(StoreLogReasonEnum.ETL.getCode());
        storeDTO.setCreateType(CreateTypeEnum.ERP.getCode());
        ServiceResponse<StoreDTO> serviceResponse = this.saveOrUpdate(storeDTO);
        if (!serviceResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        return EnterpriseServiceResponse.success();
    }

    private ServiceResponse<String> validStoreType(ProcessBatchStoreDTO processBatchStoreDTO, StoreDTO storeDTO) {
        if (StringUtils.isNotBlank(processBatchStoreDTO.getStoreType())) {
            ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreType(processBatchStoreDTO.getEnterpriseId());
            if (serviceResponse.isSuccess()) {
                List<StoreDictDTO> list = serviceResponse.getResult();
                final String storeTypeTmp = processBatchStoreDTO.getStoreType().trim();
                for (StoreDictDTO storeDictDTO : list) {
                    if (storeDictDTO.getKey().equals(storeTypeTmp)) {
                        storeDTO.setStoreType(Integer.valueOf(storeDictDTO.getValue()));
                        break;
                    }
                }
            } else {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "调用门店类型接口失败");
            }
        }
        return null;
    }

    private ServiceResponse<String> validCustomField(StoreDTO storeDTO, List<StoreFieldDTO> fieldList, String customField) {
        List<JSONObject> customFieldList = JSON.parseArray(customField, JSONObject.class);
        if (CollectionUtils.isNotEmpty(fieldList)) {
            List<StoreExtendDTO> extendDTOList = new ArrayList<>();
            for (int i = 0; i < fieldList.size(); i++) {
                StoreExtendDTO storeExtendDTO = new StoreExtendDTO();
                storeExtendDTO.setStoreFieldId(fieldList.get(i).getStoreFieldId());
                for (JSONObject storeFieldObject : customFieldList) {
                    if (fieldList.get(i).getStoreFieldName().equals(storeFieldObject.getString("fieldName"))) {
                        storeExtendDTO.setValue(storeFieldObject.getString("fieldValue"));
                        break;
                    }
                }
                storeExtendDTO.setSort(fieldList.get(i).getSort());
                // 验证
                ServiceResponse fieldResponse = validateFieldExceptRepeat(storeExtendDTO.getValue(), fieldList.get(i));
                if (!fieldResponse.isSuccess()) {
                    return EnterpriseServiceResponse.failure(ImportStoreBatchResultEnum.B5.getCode(), fieldResponse.getMessage());
                }
                extendDTOList.add(storeExtendDTO);
            }
            storeDTO.setStoreExtendList(extendDTOList);
        } else {
            storeDTO.setStoreExtendList(Collections.emptyList());
        }
        return null;
    }

    private void validArea(ProcessBatchStoreDTO processBatchStoreDTO, StoreDTO storeDTO) {
        if (!(StringUtils.isBlank(processBatchStoreDTO.getProvince()) && StringUtils.isBlank(processBatchStoreDTO.getCity()) && StringUtils.isBlank(processBatchStoreDTO.getCounty()))) {
            ServiceResponse<List<ProvinceDTO>> provinceResponse = provincesApiService.selectAllProvince();
            ServiceResponse<List<CityDTO>> cityResponse = provincesApiService.selectAllCity();
            ServiceResponse<List<CountyDTO>> countyResponse = provincesApiService.selectAllCounty();
            List<ProvinceDTO> provinceDTOList = provinceResponse.getResult();
            String finalProvince = processBatchStoreDTO.getProvince();
            ProvinceDTO provinceDTO = provinceDTOList.stream().filter(e -> StringUtils.equals(e.getProvinceName(), finalProvince)).findFirst().orElse(null);
            List<CityDTO> cityDTOList = cityResponse.getResult();
            String finalCity = processBatchStoreDTO.getCity();
            CityDTO cityDTO = cityDTOList.stream().filter(e -> StringUtils.equals(e.getCityName(), finalCity)).findFirst().orElse(null);
            List<CountyDTO> countyDTOList = countyResponse.getResult();
            String finalCounty = processBatchStoreDTO.getCounty();
            CountyDTO countyDTO = countyDTOList.stream().filter(e -> StringUtils.equals(e.getCountyName(), finalCounty)).findFirst().orElse(null);
            if (provinceDTO != null && cityDTO != null && countyDTO != null) {
                storeDTO.setProvinceId(countyDTO.getProvinceId());
                storeDTO.setCityId(countyDTO.getCityId());
                storeDTO.setAreaId(countyDTO.getCountyId());
                storeDTO.setProvinces(provinceDTO.getProvinceName() + "/" + countyDTO.getCountyName() + "/" + cityDTO.getCityName());
            }
        }
    }

    @Override
    public ServiceResponse<Void> updateScore(Integer enterpriseId, Integer storeId, double storeScore, Integer evaluateCount) {
        StoreDTO storeDTO = new StoreDTO();
        storeDTO.setEnterpriseId(enterpriseId);
        storeDTO.setStoreId(storeId);
        storeDTO.setScore(storeScore);
        storeDTO.setEvaluateCount(evaluateCount);
        this.storeService.updateStore(storeDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> getStoreIdByStoreNameAndStoreCodeAndRegionId(String storeCode, Integer regionId, Integer enterpriseId) {
        TabStore store = this.storeService.getByStoreCodeAndRegionId(enterpriseId, regionId, storeCode);
        if (store != null) {
            return EnterpriseServiceResponse.success(store.getStoreId());
        }
        return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
    }

    @Override
    public ServiceResponse<Boolean> checkExistStore(Integer enterpriseId) {
        Boolean exist = this.storeService.checkExistStore(enterpriseId);
        return EnterpriseServiceResponse.success(exist);
    }

    @Override
    @Transactional
    public ServiceResponse<Void> saveStoreAndClerkForBulkOrder(String params) {
        logger.info("saveStoreAndClerkForBulkOrder:{}", params);
        Map<String, Object> message = new HashMap<>();
        message.put("type", 5);
        try{
            StoreBulkOrderDTO storeBulkOrderDTO = JSON.parseObject(params, StoreBulkOrderDTO.class);
            message.put("jobId", storeBulkOrderDTO.getBatchId());
            if(CollectionUtils.isNotEmpty(storeBulkOrderDTO.getStore())){
                for(StoreOrderDTO storeOrderDTO : storeBulkOrderDTO.getStore()){
                    StoreDTO storeDTO = new StoreDTO();
                    storeDTO.setEnterpriseId(storeOrderDTO.getEnterpriseId());
                    storeDTO.setStoreCode(storeOrderDTO.getStoreCode());
                    storeDTO.setStoreName(storeOrderDTO.getStoreName());
                    storeDTO.setRegionId(storeOrderDTO.getRegionId());
                    storeDTO.setReason(storeOrderDTO.getReason());
                    storeDTO.setRemark(storeOrderDTO.getRemark());
                    storeDTO.setCreateType(storeOrderDTO.getReason());
                    ServiceResponse<StoreDTO> response = this.saveOrUpdate(storeDTO);
                    logger.info("批量处理门店结果：{}", JSON.toJSONString(response));
                }
            }
            if(CollectionUtils.isNotEmpty(storeBulkOrderDTO.getClerk())){
                for(StoreOrderDTO storeOrderDTO : storeBulkOrderDTO.getClerk()){
                    ServiceResponse<StoreDTO> storeResponse = this.getStoreByStoreCode(storeOrderDTO.getEnterpriseId(),
                            storeOrderDTO.getRegionId(), storeOrderDTO.getStoreCode());
                    if(storeResponse.isSuccess() && storeResponse.getResult() != null){
                        StoreDTO store = storeResponse.getResult();
                        ClerkDTO clerkDTO = new ClerkDTO();
                        clerkDTO.setClerkName(storeOrderDTO.getClerkName());
                        clerkDTO.setClerkCode(storeOrderDTO.getClerkCode());
                        clerkDTO.setEnterpriseId(storeOrderDTO.getEnterpriseId());
                        clerkDTO.setStoreInfoId(store.getStoreInfoId());
                        clerkDTO.setReason(storeOrderDTO.getReason());
                        if (ClerkLogReasonEnum.BULKORDER.getCode() == storeOrderDTO.getReason()) {
                            clerkDTO.setRemark("订单号:" + storeOrderDTO.getRemark());
                        } else {
                            clerkDTO.setRemark("卡号:" + storeOrderDTO.getRemark());
                        }
                        clerkDTO.setStoreName(store.getStoreName());
                        ServiceResponse serviceResponse = this.clerkApiService.saveOrUpdate(clerkDTO);
                        logger.info("批量处理导购结果：{}", JSON.toJSONString(serviceResponse));
                    }
                }
            }
            message.put("status", 1);
        }catch (Exception e){
            message.put("status", 3);
            message.put("message", e.getMessage());
            logger.warn("exception:", e);
        }
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            client.sendMessage("batchJobNotify", JSON.toJSONString(message));
        } catch (Exception e) {
            logger.warn("发送消息路由失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<StoreDTO> getStoreById(Integer enterpriseId, Integer storeId) {
        String key = Constants.STORE_KEY + enterpriseId + ":" + storeId;

        StoreDTO storeDTO = (StoreDTO) RedisUtil.getCache(key);
        if (storeDTO == null) {
            storeDTO = this.getStoreDTO(enterpriseId, storeId);
            if (storeDTO == null) {
                logger.info("enterpriseId:{}, storeId:{} , 门店不存在", enterpriseId, storeId);
                return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
            }
            // 修改门店缓存时间
            RedisUtil.setCache(key, storeDTO, storeExpiredTime, TimeUnit.HOURS);
        }
        StorePhotoDTO storePhotoDTO = new StorePhotoDTO();
        storePhotoDTO.setImageUrl(Constants.DEFAULT_PHOTO);
        storeDTO.setDefaultPhoto(storePhotoDTO);
        return ServiceResponse.success(storeDTO);
    }

    @Override
    public ServiceResponse<StoreDTO> getStoreByStoreInfoId(Integer enterpriseId, Integer storeInfoId) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStoreInfoIds(storeInfoId+"");
        ServiceResponse<Page<StoreDTO>> response = this.listStore(storeSearchDTO, 1, 1);
        if(response.isSuccess() && response.getResult() != null){
            List<StoreDTO> result = response.getResult().getResult();
            if(CollectionUtils.isNotEmpty(result)){
                StoreDTO storeDTO = result.get(0);
                if (CollectionUtils.isNotEmpty(storeDTO.getLocation())) {
                    storeDTO.setLongitude(storeDTO.getLocation().get(0).toString());
                    storeDTO.setLatitude(storeDTO.getLocation().get(1).toString());
                    if (CollectionUtils.isNotEmpty(storeDTO.getStoreBrandIdList())) {
                        Integer brandIds = storeDTO.getStoreBrandIdList().get(0);
                        storeDTO.setBrandIds(brandIds.toString());
                    }
                }
                return ServiceResponse.success(storeDTO);
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<StoreDTO> getStoreByStoreInfoIdFromDb(Integer enterpriseId, Integer storeInfoId) {
        StoreSearchDbDTO storeSearchDbDTO = new StoreSearchDbDTO();
        storeSearchDbDTO.setEnterpriseId(enterpriseId);
        storeSearchDbDTO.setStoreId(storeInfoId);
        com.github.pagehelper.Page<StoreDTO> storeDTOS = this.storeService.listStore(storeSearchDbDTO, 1, 1);
        if(storeDTOS != null && CollectionUtils.isNotEmpty(storeDTOS.getResult())){
            return ServiceResponse.success(storeDTOS.getResult().get(0));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<StoreDTO> getStoreByStoreCode(Integer enterpriseId, Integer regionId, String storeCode) {
        StoreSearchDbDTO storeSearchDbDTO = new StoreSearchDbDTO();
        storeSearchDbDTO.setEnterpriseId(enterpriseId);
        storeSearchDbDTO.setStoreCode(storeCode);
        storeSearchDbDTO.setRegionId(regionId);
        com.github.pagehelper.Page<StoreDTO> page = this.storeService.listStore(storeSearchDbDTO, 1, 1);
        if(page != null && CollectionUtils.isNotEmpty(page.getResult())){
            return ServiceResponse.success(page.getResult().get(0));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<StoreDTO> getStoreContainBusinessTimeByStoreCode(Integer enterpriseId, Integer regionId, String storeCode) {
        ServiceResponse<StoreDTO> result = getStoreByStoreCode(enterpriseId, regionId, storeCode);
        if (result.isSuccess()) {
            StoreDTO storeDTO = result.getResult();
            if (storeDTO == null) {
                return ServiceResponse.success();
            }
            List<TabStoreBusinessTime> list = storeBusinessTimeService.listBusinessTime(storeDTO.getStoreInfoId());
            List<StoreBusinessTimeDTO> businessTimeDTOList = EntityUtil.changeEntityListByOrika(StoreBusinessTimeDTO.class, list);
            storeDTO.setBusinessTimeList(businessTimeDTOList);
            storeBusinessTimeService.convertBusinessTime(businessTimeDTOList);
            return ServiceResponse.success(storeDTO);
        }
        return ServiceResponse.success();
    }

    private StoreDTO getStoreDTO(Integer enterpriseId, Integer storeId) {
        StoreDTO store = storeService.getById(storeId);
        if (store == null) {
            return null;
        }
        StoreRegionDTO tabStoreRegion = this.storeRegionService.getById(enterpriseId, store.getRegionId());
        if (tabStoreRegion == null) {
            return null;
        }
        StoreDTO storeDTO = EntityUtil.changeEntityByOrika(StoreDTO.class, store);
        storeDTO.setRegionName(tabStoreRegion.getRegionName());
        TabStoreGroup storeGroup = storeGroupService.getStoreGroupById(storeDTO.getStoreGroupId());
        if (storeGroup != null) {
            storeDTO.setStoreGroupName(storeGroup.getStoreGroupName());
            String storeGroupChain = storeGroup.getStoreGroupChain();
            String[] storeIdArr = storeGroupChain.split("_");
            List<Integer> storeGroupIdList = new ArrayList<>();
            for (String s : storeIdArr) {
                if (StringUtils.isNotBlank(s)) {
                    storeGroupIdList.add(Integer.valueOf(s));
                }
            }
            storeDTO.setStoreGroupIdList(storeGroupIdList);
        }
        // 门店照片
        List<TabStorePhoto> tabPhotoList = storePhotoService.listStorePhoto(storeDTO.getEnterpriseId(), storeDTO.getStoreInfoId());
        List<StorePhotoDTO> photoDTOList = EntityUtil.changeEntityListByOrika(StorePhotoDTO.class, tabPhotoList);
        storeDTO.setPhotoList(photoDTOList);
        // 营业时间
        List<TabStoreBusinessTime> tabBusinessTimeList = storeBusinessTimeService.listBusinessTime(store.getStoreInfoId());
        if (CollectionUtils.isNotEmpty(tabBusinessTimeList)) {
            List<StoreBusinessTimeDTO> businessTimeDTOList = EntityUtil.changeEntityListByOrika(StoreBusinessTimeDTO.class, tabBusinessTimeList);
            storeDTO.setBusinessTimeList(businessTimeDTOList);
            storeBusinessTimeService.convertBusinessTime(businessTimeDTOList);
        }
        // 门店品牌
        List<TabStoreBrand> storeBrandList = storeBrandService.listStoreBrandByIds(store.getBrandIds());
        if (CollectionUtils.isNotEmpty(storeBrandList)) {
            List<StoreBrandDTO> brandDTOList = EntityUtil.changeEntityListByOrika(StoreBrandDTO.class, storeBrandList);
            storeDTO.setBrandList(brandDTOList);
            List<Integer> storeBrandIdList = new ArrayList<>();
            String storeBrandNames = "";
            for (StoreBrandDTO storeBrandDTO : brandDTOList) {
                storeBrandIdList.add(storeBrandDTO.getStoreBrandId());
                storeBrandNames += storeBrandDTO.getStoreBrandName() + ",";
            }
            storeDTO.setStoreBrandIdList(storeBrandIdList);
            storeDTO.setStoreBrandNames(StringUtils.isBlank(storeBrandNames) ? "" : storeBrandNames.substring(0, storeBrandNames.length() - 1));
        }

        // 自定义属性
        List<TabStoreExtend> extendList = storeExtendService.listByStoreInfoId(enterpriseId, storeDTO.getStoreInfoId());
        List<StoreExtendDTO> extendDTOList = storeExtendService.convertStoreExtendToDTO(extendList);
        storeDTO.setStoreExtendList(extendDTOList);
        Integer fromEnterpriseId = storeDTO.getFromEnterpriseId();
        if (fromEnterpriseId != null) {
            ServiceResponse<EnterpriseDTO> serviceResponse = enterpriseApiService.getEnterpriseById(fromEnterpriseId);
            if (serviceResponse.isSuccess()) {
                EnterpriseDTO enterpriseDTO = serviceResponse.getResult();
                storeDTO.setFromEnterpriseName(enterpriseDTO.getEnterpriseName());
            }
        }
        return storeDTO;
    }

    @Override
    public ServiceResponse<Void> validateFieldExceptRepeat(String value, StoreFieldDTO storeFieldDTO) {
        Integer storeFieldType = storeFieldDTO.getStoreFieldType();
        String storeFieldDetail = storeFieldDTO.getStoreFieldDetail();
        FieldBase fieldBase;
        if (StoreFieldTypeEnum.TEXT.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, TextField.class);
        } else if (StoreFieldTypeEnum.RADIO.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, RadioField.class);
        } else if (StoreFieldTypeEnum.CHECK.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, CheckField.class);
        } else if (StoreFieldTypeEnum.REAL_NUMBER.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, RealNumberField.class);
        } else if (StoreFieldTypeEnum.DATE.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, DateTypeField.class);
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在该自定义属性类型");
        }
        String msg;
        if (StoreFieldTypeEnum.RADIO.getCode() == storeFieldType || StoreFieldTypeEnum.CHECK.getCode() == storeFieldType) {
            List<TabStoreFieldSelect> fieldSelectList = storeFieldSelectService.listStoreFieldSelect(storeFieldDTO.getStoreFieldId(), null, 1, Integer.MAX_VALUE);
            List<String> nameList = fieldSelectList.stream().map(TabStoreFieldSelect::getStoreFieldSelectName).collect(Collectors.toList());
            msg = fieldBase.validateSelect(value, nameList);
        } else {
            msg = fieldBase.validate(value);
        }

        if (StringUtils.isBlank(msg)) {
            return ServiceResponse.success();
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), storeFieldDTO.getStoreFieldName() + "  " + msg);
        }
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> listStoreOverFlow(Integer enterpriseId, Page<StoreDTO> page, String search) {
        StoreSearchDbDTO searchDbDTO = new StoreSearchDbDTO();
        searchDbDTO.setEnterpriseId(enterpriseId);
        searchDbDTO.setOverflowStatus(com.gic.enterprise.constants.Constants.NORMAL_STATUS);
        searchDbDTO.setSearch(search);
        com.github.pagehelper.Page<StoreDTO> storeDTOS = this.storeService.listStore(searchDbDTO, page.getCurrentPage(), page.getPageSize());
        page = PageHelperUtils.changePageHelperToCurrentPage(storeDTOS, StoreDTO.class);
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse bulkUpdateStore(String storeIds, String value, Integer enterpriseId, int type, Integer operatorId, String operatorName) {
        //type表示操作的类型，1更新门店启用状态，2门店电话，3营业时间，4门店照片（替换），5关联品牌，6批量上传图片（原有基础上新增）
        //7批量修改门店分组
        if (StringUtils.isBlank(storeIds) || StringUtils.isBlank(value)) {
            return EnterpriseServiceResponse.failure(com.gic.enterprise.error.ErrorCode.MISS_PARAMETER);
        }
        try {
            String storeInfoIds;
            if (StringUtils.equals(Constants.ALL, storeIds)) {
                storeInfoIds = "all";
            } else {
                storeInfoIds = storeService.listStoreInfoIdByStoreIds(storeIds, enterpriseId);
            }
            if (StringUtils.isBlank(storeInfoIds)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店不存在或非自有门店");
            }
            String msg = storeStrtegyMap.get(type+"").bulkUpdateStore(storeInfoIds, enterpriseId, value, storeService, operatorId, operatorName);
            List<Integer> storeIdList = new ArrayList<>();
            if (Constants.ALL.equals(storeIds)) {
                this.removeCache(enterpriseId);
            } else {
                String[] storeIdArr = storeIds.split(GlobalInfo.FLAG_COMMA);
                for (String s : storeIdArr) {
                    storeIdList.add(Integer.valueOf(s));
                }
                this.removeCache(enterpriseId, storeIdList);
            }
            if (type == Constants.UPDATE_STATUS || type == Constants.UPDATE_BRAND || type == Constants.UPDATE_GROUP) {
                if (Constants.ALL.equals(storeIds)) {
                    this.bulkAddStoreToIndex(enterpriseId);
                } else {
                    this.bulkAddStoreToIndex(enterpriseId, storeIdList);
                }
            }
            if (StringUtils.isNotBlank(msg)) {
                return ServiceResponse.failure("7777", msg);
            }
//            bulkUpdateLog(storeIds, value, enterpriseId, type, operatorId, operatorName);
        } catch (Exception e) {
            logger.error("",  e);
        }

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<StoreDTO> getByStoreNameAndStoreCode(Integer enterpriseId, Integer regionId, String storeName, String storeCode) {
        StoreDTO store = this.storeService.getByStoreNameAndStoreCode(enterpriseId, regionId, storeName, storeCode);
        if (store != null) {
            return ServiceResponse.success(EntityUtil.changeEntityByOrika(StoreDTO.class, store));
        } else {
            return ServiceResponse.success();
        }
    }

    @Override
    public ServiceResponse addStoreToIndex(Integer enterpriseId, Integer storeId) {
        logger.info("addStoreToIndex:{},{}",enterpriseId,storeId);
        this.refreshCache(enterpriseId, storeId);
        int logId = this.storeIndexLogService.saveStoreIndexLog(enterpriseId, storeId);
        JSONObject json = new JSONObject();
        json.put("enterpriseId", enterpriseId);
        json.put("storeId", storeId);
        json.put("logId", logId);
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            clientInstance.sendMessage("storeIndexMq", json.toJSONString());
        } catch (Exception e) {
            logger.warn("异常：", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse addStoreToIndexMq(String param){
        JSONObject json = JSON.parseObject(param);
        Integer enterpriseId = json.getInteger("enterpriseId");
        Integer storeId = json.getInteger("storeId");
        Integer logId = json.getInteger("logId");
        if(logId == null){
            logId = this.storeIndexLogService.saveStoreIndexLog(enterpriseId, storeId);
        }
        boolean isAdd = true;
        DynamicSearchDTO dynamicSearchDTO = new DynamicSearchDTO();
        JSONObject search = QueryConditionAssemblyUtil.createSimpleQueryNode("id", OperateEnum.OPERATE_EQ, enterpriseId + "_" + storeId);
        dynamicSearchDTO.setEnterpriseId(com.gic.enterprise.constants.Constants.INDEX_ENTERPRISEID);
        dynamicSearchDTO.setColumnCategoryCode(com.gic.enterprise.constants.Constants.ColumnCategoryCode);
        dynamicSearchDTO.setSearchJson(search);
        ServiceResponse<Long> serviceResponse = this.esBusinessOperaApiService.queryDataCount(dynamicSearchDTO);
        if (serviceResponse.isSuccess()) {
            Long result = serviceResponse.getResult();
            if (result != null) {
                if (result > 0) {
                    isAdd = false;
                }
            }
        }
        DynamicDocDTO docDTO = this.buildStoreDocDto(enterpriseId, storeId);
        logger.info("写入es的门店数据：{}", JSON.toJSONString(docDTO, true));
        String message = "success";
        if (docDTO != null) {
            ServiceResponse esResponse = null;
            if (isAdd) {
                esResponse = this.esBusinessOperaApiService.createDataSingle(docDTO, false);
            } else {
                esResponse = this.esBusinessOperaApiService.updateDataSingle(docDTO, false);
            }
            logger.info("门店写入es返回结果：{}", JSON.toJSONString(esResponse));
            if (esResponse.isSuccess()) {
                this.storeIndexLogService.updateStoreIndexLog(logId, message);
                return esResponse;
            }
            message = esResponse.getMessage();
            this.storeIndexLogService.updateStoreIndexLog(logId, message);
            return EnterpriseServiceResponse.failure(esResponse.getCode(), esResponse.getMessage());
        }
        message = "门店不存在";
        this.storeIndexLogService.updateStoreIndexLog(logId, message);
        return null;
    }

    @Override
    public ServiceResponse bulkAddStoreToIndex(Integer enterpriseId) {
        int i = this.storeService.refreshStoreIndex(enterpriseId);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> refreshStoreIndex(Integer enterpriseId, Integer storeGroupId) {
        int i = this.storeService.refreshStoreIndex(enterpriseId, storeGroupId);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse bulkAddStoreToIndex(Integer enterpriseId, List<Integer> storeIdList) {
        for(Integer storeId : storeIdList){
            this.addStoreToIndex(enterpriseId, storeId);
        }
        return ServiceResponse.success();
    }

    private DynamicDocDTO buildStoreDocDto(Integer enterpriseId, Integer storeId) {
        refreshCache(enterpriseId, storeId);
        ServiceResponse<StoreDTO> response = this.getStoreById(enterpriseId, storeId);
        StoreDTO dto = response.getResult();
        if (response.isSuccess() && dto != null && dto.getOverflowStatus() != 1) {
            StoreEsDTO esDTO = new StoreEsDTO();
            esDTO.setAreaIds(Arrays.asList(dto.getProvinceId(), dto.getCityId(), dto.getAreaId()));
            esDTO.setCityId(dto.getCityId());
            esDTO.setCompleteStatus(dto.getCompleteStatus());
            esDTO.setCountyId(dto.getAreaId());
            esDTO.setCreateType(dto.getCreateType());
            esDTO.setEnterpriseId(enterpriseId);
            esDTO.setRealStatus(dto.getRealStatus());
            esDTO.setId(enterpriseId + "_" + storeId);
            esDTO.setProvinceId(dto.getProvinceId());
            esDTO.setProvinces(dto.getProvinces());
            esDTO.setRegionId(dto.getRegionId());
            esDTO.setRegionName(dto.getRegionName());
            esDTO.setStatus(dto.getStatus());
            esDTO.setStoreBrandIdList(dto.getStoreBrandIdList());
            esDTO.setStoreBrands(dto.getStoreBrandNames());
            esDTO.setStoreCode(dto.getStoreCode());
            esDTO.setStoreGroupId(dto.getStoreGroupId());
            esDTO.setStoreGroupIdList(dto.getStoreGroupIdList());
            esDTO.setStoreId(dto.getStoreId());
            esDTO.setStoreInfoId(dto.getStoreInfoId());
            esDTO.setStoreGroupName(dto.getStoreGroupName());
            esDTO.setStoreName(dto.getStoreName());
            esDTO.setStoreType(dto.getStoreType());
            esDTO.setOwnType(dto.getOwnType());
            esDTO.setBusinessTimeList(dto.getBusinessTimeList());
            esDTO.setPhotoList(dto.getPhotoList());
            esDTO.setAddress(dto.getAddress());
            esDTO.setScore(dto.getScore());
            esDTO.setEvaluateCount(dto.getEvaluateCount());
            esDTO.setConactsPhone(dto.getConactsPhone());
            esDTO.setUpdateTime(dto.getUpdateTime().getTime());
            esDTO.setFromEnterpriseId(dto.getFromEnterpriseId());
            esDTO.setFromEnterpriseName(dto.getFromEnterpriseName());
            Double[] location = {Double.valueOf(StringUtils.isBlank(dto.getLongitude())? "0" : dto.getLongitude()),Double.valueOf(StringUtils.isBlank(dto.getLatitude())? "0":dto.getLatitude())};
            JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(esDTO));
            jsonObject.put("location", location);
            List<StoreExtendDTO> storeExtendDTOList = dto.getStoreExtendList();
            for (StoreExtendDTO extendDTO : storeExtendDTOList) {
                String key = "enterprise:store:storeField:" + enterpriseId + ":" + extendDTO.getStoreFieldId();
                Object existObj = RedisUtil.getCache(key);
                if (existObj != null && extendDTO.getStoreFieldType() != StoreFieldTypeEnum.TEXT.getCode()) {
                    if (extendDTO.getStoreFieldType() == StoreFieldTypeEnum.DATE.getCode()) {
                        String value = null;
                        if(StringUtils.isNotBlank(extendDTO.getValue())){
                            value = extendDTO.getValue().replaceAll("-", "").
                                    replaceAll(":", "").replaceAll(" ", "");
                        }
                        jsonObject.put(existObj.toString(), value == null ? 0L : Long.valueOf(value));
                    } else if (extendDTO.getStoreFieldType() == StoreFieldTypeEnum.CHECK.getCode()) {
                        jsonObject.put(existObj.toString(), StringUtils.isBlank(extendDTO.getValue()) ? 0 : Stream.of(extendDTO.getValue().split(",")).map(Long::valueOf).collect(Collectors.toList()));
                    } else if (extendDTO.getStoreFieldType() == StoreFieldTypeEnum.REAL_NUMBER.getCode()) {

                        jsonObject.put(existObj.toString(), StringUtils.isBlank(extendDTO.getValue()) ? 0 : new BigDecimal(extendDTO.getValue()).multiply(new BigDecimal(100000)).longValue());
                    } else {
                        jsonObject.put(existObj.toString(), StringUtils.isBlank(extendDTO.getValue()) ? 0 : Long.valueOf(extendDTO.getValue()));
                    }
                }
            }
            DynamicDocDTO docDTO = new DynamicDocDTO();
            docDTO.setEnterpriseId(com.gic.enterprise.constants.Constants.INDEX_ENTERPRISEID);
            docDTO.setColumnCategoryCode(com.gic.enterprise.constants.Constants.ColumnCategoryCode);
            docDTO.setDocJson(jsonObject);
            return docDTO;
        }
        return null;
    }

    private void removeCache(Integer enterpriseId) {
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RKeys keys = redisClient.getKeys();
        keys.deleteByPattern(Constants.STORE_KEY + enterpriseId + ":*");
    }

    private void removeCache(Integer enterpriseId, List<Integer> storeIds) {
        for (Integer storeId : storeIds) {
            String key = Constants.STORE_KEY + enterpriseId + ":" + storeId;
            RedisUtil.delCache(key);
        }
    }

    private JSONObject buildSearchJson(StoreSearchDTO storeSearchDTO){
        logger.info("查询条件：{}", JSON.toJSONString(storeSearchDTO));
        List<JSONObject> jsonObjectList = new ArrayList<>();
        JSONObject json = null;
        JSONObject enterpriseJson = null;
        json = buildBaseField(storeSearchDTO, jsonObjectList, json);
        json = buildOtherField(storeSearchDTO, jsonObjectList, json);
        if(StringUtils.isNotBlank(storeSearchDTO.getStoreTags())){
            String[] arr = storeSearchDTO.getStoreTags().split(" ");
            List<TabStoreTag> storeTags = this.storeTagService.listByEnterprise(storeSearchDTO.getEnterpriseId(), null);
            List<Integer> storeWidgetList = storeTags.stream().filter(tag -> {
                for (String s : arr) {
                    if (tag.getStoreTagId() == Integer.parseInt(s)) {
                        return true;
                    }
                }
                return false;
            }).map(TabStoreTag::getStoreWidgetId).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(storeWidgetList)){
                List<JSONObject> jsonObjectOrList = new ArrayList<>();
                for (Integer storeResource : storeWidgetList) {
                    String s = this.buildStoreWidgetJson(storeResource, json, jsonObjectOrList, storeSearchDTO.getEnterpriseId());
                }
                JSONObject object = QueryConditionAssemblyUtil.createListQueryNode(jsonObjectOrList, OperateEnum.OPERATE_OR);
                jsonObjectList.add(object);
            }
        }
        String storeWidgetJson = this.buildStoreWidgetJson(storeSearchDTO.getStoreResource() == null ? null : storeSearchDTO.getStoreResource().intValue(), json, jsonObjectList, storeSearchDTO.getEnterpriseId());
        /*****如果存在没有门店权限的门店控件id，直接返回*****/
        if (StringUtils.isNotBlank(storeWidgetJson)) {
            return null;
        }
        if(CollectionUtils.isNotEmpty(storeSearchDTO.getStoreResourceList())){
            for(Integer storeResource : storeSearchDTO.getStoreResourceList()){
                String s = this.buildStoreWidgetJson(storeResource, json, jsonObjectList, storeSearchDTO.getEnterpriseId());
                if(StringUtils.isNotBlank(s)){
                    return null;
                }
            }
        }
        enterpriseJson = QueryConditionAssemblyUtil.createListQueryNode(jsonObjectList, OperateEnum.OPERATE_AND);
        /***build notIn storeId****/
        if(StringUtils.isNotBlank(storeSearchDTO.getStoreIdsOfNot())){
            List<JSONObject> list = new ArrayList<>();
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STOREID.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreIdsOfNot());
            list.add(json);
            JSONObject notJson = QueryConditionAssemblyUtil.createListQueryNode(list, OperateEnum.OPERATE_AND_NOT);
            List<JSONObject> listTmp = new ArrayList<>();
            listTmp.add(enterpriseJson);
            listTmp.add(notJson);
            enterpriseJson = QueryConditionAssemblyUtil.createListQueryNode(listTmp, OperateEnum.OPERATE_AND);
        }
        /****build notIn storeInfoId*****/
        if(StringUtils.isNotBlank(storeSearchDTO.getStoreInfoIdsOfNot())){
            List<JSONObject> list = new ArrayList<>();
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STOREINFOID.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreInfoIdsOfNot());
            list.add(json);
            JSONObject notJson = QueryConditionAssemblyUtil.createListQueryNode(list, OperateEnum.OPERATE_AND_NOT);
            List<JSONObject> listTmp = new ArrayList<>();
            listTmp.add(enterpriseJson);
            listTmp.add(notJson);
            enterpriseJson = QueryConditionAssemblyUtil.createListQueryNode(listTmp, OperateEnum.OPERATE_AND);
        }
        return enterpriseJson;
    }

    private JSONObject buildOtherField(StoreSearchDTO storeSearchDTO, List<JSONObject> jsonObjectList, JSONObject json) {
        if (StringUtils.isNotBlank(storeSearchDTO.getSearch())) {
            JSONObject storeNameSearch = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STORENAME.getField(), OperateEnum.OPERATE_LIKE, storeSearchDTO.getSearch());
            JSONObject storeCodeSearch = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STORECODE.getField(), OperateEnum.OPERATE_LIKE, storeSearchDTO.getSearch());
            List<JSONObject> list = new ArrayList<>();
            list.add(storeCodeSearch);
            list.add(storeNameSearch);
            json = QueryConditionAssemblyUtil.createListQueryNode(list, OperateEnum.OPERATE_OR);
            jsonObjectList.add(json);
        }
        if (StringUtils.isNotBlank(storeSearchDTO.getStoreIds())) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STOREID.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreIds());
            jsonObjectList.add(json);
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getStoreInfoIds())){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STOREINFOID.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreInfoIds());
            jsonObjectList.add(json);
        }
        if(storeSearchDTO.getStoreBrandId() != null){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STOREBRANDIDLIST.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreBrandId());
            jsonObjectList.add(json);
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getSearchJson()) && !Constants.EMPTY_JSON.equals(storeSearchDTO.getSearchJson())){
            ServiceResponse<JSONObject> jsonObjectServiceResponse = this.parseStoreSelectJson(storeSearchDTO.getSearchJson(), storeSearchDTO.getEnterpriseId());
            if(jsonObjectServiceResponse.isSuccess()){
                json = jsonObjectServiceResponse.getResult();
                if (json != null) {
                    jsonObjectList.add(json);
                }
            }
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getAuthSearchJson()) && !Constants.EMPTY_JSON.equals(storeSearchDTO.getAuthSearchJson())){
            ServiceResponse<JSONObject> jsonObjectServiceResponse = this.parseStoreSelectJson(storeSearchDTO.getAuthSearchJson(), storeSearchDTO.getEnterpriseId());
            if(jsonObjectServiceResponse.isSuccess()){
                json = jsonObjectServiceResponse.getResult();
                if (json != null) {
                    jsonObjectList.add(json);
                }
            }
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getLatitude()) && StringUtils.isNotBlank(storeSearchDTO.getLongitude())){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.LOCATION.getField(), OperateEnum.OPERATE_LOCATION, storeSearchDTO.getLatitude()+","+storeSearchDTO.getLongitude());
            jsonObjectList.add(json);
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getStoreGroupIds())){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STOREGROUPIDLIST.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreGroupIds());
            jsonObjectList.add(json);
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getRegionIds())){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.REGIONID.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getRegionIds());
            jsonObjectList.add(json);
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getStoreTypes())){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STORETYPE.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreTypes());
            jsonObjectList.add(json);
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getStoreStatuss())){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STATUS.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreStatuss());
            jsonObjectList.add(json);
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getRealStatuss())){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.REALSTATUS.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getRealStatuss());
            jsonObjectList.add(json);
        }
        if(StringUtils.isNotBlank(storeSearchDTO.getStoreBrandIds())){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STOREBRANDIDLIST.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreBrandIds());
            jsonObjectList.add(json);
        }
        return json;
    }

    private JSONObject buildBaseField(StoreSearchDTO storeSearchDTO, List<JSONObject> jsonObjectList, JSONObject json) {
        JSONObject enterpriseJson;
        if(storeSearchDTO.getEnterpriseId() != null){
            enterpriseJson = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.ENTERPRISEID.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getEnterpriseId());
            jsonObjectList.add(enterpriseJson);
        }
        if (storeSearchDTO.getStoreGroupId() != null) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STOREGROUPIDLIST.getField(), OperateEnum.OPERATE_ARR_CONTAIN, storeSearchDTO.getStoreGroupId());
            jsonObjectList.add(json);
        }
        if(storeSearchDTO.getRegionId() != null){
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.REGIONID.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getRegionId());
            jsonObjectList.add(json);
        }
        if (StringUtils.isNotBlank(storeSearchDTO.getProvinceId())) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.PROVINCEID.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getProvinceId());
            jsonObjectList.add(json);
        }
        if (StringUtils.isNotBlank(storeSearchDTO.getCityId())) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.CITYID.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getCityId());
            jsonObjectList.add(json);
        }
        if (StringUtils.isNotBlank(storeSearchDTO.getCountyId())) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.COUNTYID.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getCountyId());
            jsonObjectList.add(json);
        }
        if (storeSearchDTO.getCreateType() != null) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.CREATETYPE.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getCreateType());
            jsonObjectList.add(json);
        }
        if (storeSearchDTO.getStatus() != null) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STATUS.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getStatus());
            jsonObjectList.add(json);
        }
        //门店真正的上线状态
        if (storeSearchDTO.getRealStatus() != null) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.REALSTATUS.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getRealStatus());
            jsonObjectList.add(json);
        }
        if (storeSearchDTO.getCompleteStatus() != null) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.COMPLETESTATUS.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getCompleteStatus());
            jsonObjectList.add(json);
        }
        if (StringUtils.isNotBlank(storeSearchDTO.getStoreCode())) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.STORECODE.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getStoreCode());
            jsonObjectList.add(json);
        }
        if (storeSearchDTO.getOwnType() != null) {
            json = QueryConditionAssemblyUtil.createSimpleQueryNode(StoreESFieldsEnum.OWNTYPE.getField(), OperateEnum.OPERATE_EQ, storeSearchDTO.getOwnType());
            jsonObjectList.add(json);
        }
        return json;
    }

    private String  buildStoreWidgetJson(Integer storeWidgetId, JSONObject json, List<JSONObject> jsonObjectList, Integer enterpriseId){
        if(storeWidgetId != null){
            StoreWidgetDTO storeWidget = this.storeWidgetService.getStoreWidget(storeWidgetId);
            if(storeWidget != null){
                if(storeWidget.getAuthMode() != null && storeWidget.getAuthMode() == 1
                        && StringUtils.isNotBlank(storeWidget.getSearchParam()) && !"{}".equals(storeWidget.getSearchParam())){
                    ServiceResponse<JSONObject> jsonObjectServiceResponse = this.parseStoreSelectJson(storeWidget.getSearchParam(), enterpriseId);
                    if(jsonObjectServiceResponse.isSuccess()){
                        json = jsonObjectServiceResponse.getResult();
                        if (json != null) {
                            jsonObjectList.add(json);
                        }
                    }
                }
                if(storeWidget.getAuthMode() != null && storeWidget.getAuthMode() == 1
                        && StringUtils.isNotBlank(storeWidget.getAuthSearchParam()) && !"{}".equals(storeWidget.getAuthSearchParam())){
                    ServiceResponse<JSONObject> jsonObjectServiceResponse = this.parseStoreSelectJson(storeWidget.getAuthSearchParam(), enterpriseId);
                    if(jsonObjectServiceResponse.isSuccess()){
                        json = jsonObjectServiceResponse.getResult();
                        if (json != null) {
                            jsonObjectList.add(json);
                        }
                    }
                }
                if(storeWidget.getAuthMode() != null && storeWidget.getAuthMode().equals(Constants.STORE_NONE_AUTH)){
                    return "noAuth";
                }
            }
        }
        return null;
    }

    private static ServiceResponse<String> responseErrorElement(ImportStoreBatchResultEnum error) {
        return ServiceResponse.failure(error.getCode(), error.getResultInfo());
    }

    private void updateStoreToWm(StoreDTO storeDTO) {
        WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = getWeimobPhsicalStore(storeDTO);
        logger.info("实体门店拼接后对象为:{}", JSON.toJSONString(weimobPhysicalStoreDTO));
        if (weimobPhysicalStoreDTO != null) {
            ServiceResponse<List<WmStoreSyncLogDTO>> wmStoreListResult = wmStoreSyncLogApiService
                    .listByStoreId(storeDTO.getStoreId(), storeDTO.getEnterpriseId());
            if (wmStoreListResult.isSuccess()) {
                List<WmStoreSyncLogDTO> wmStoreList = wmStoreListResult.getResult();
                wmStoreList.forEach(wmStore -> {
                    ServiceResponse<Void> weimobResult = this.weimobStoreSiteApiService
                            .updateWeimobPhysicalStore(wmStore.getWmMallStoreId(), storeDTO.getEnterpriseId(), weimobPhysicalStoreDTO);
                    logger.info("门店修改同步到微盟结果：{},店铺ID：{}", JSON.toJSONString(weimobResult), wmStore.getWmMallStoreId());
                });
            }
        }
    }

    private WeimobPhysicalStoreDTO getWeimobPhsicalStore(StoreDTO storeDTO) {
        if (storeDTO.getStatus().intValue() == 1) {
            WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = new WeimobPhysicalStoreDTO();
            //用storeInfoId
            weimobPhysicalStoreDTO.setStoreInfoId(storeDTO.getStoreInfoId());
            weimobPhysicalStoreDTO.setStoreNumber(storeDTO.getStoreCode());
            weimobPhysicalStoreDTO.setStoreName(storeDTO.getStoreName());
            //经纬度
            if (org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getLongitude())) {
                weimobPhysicalStoreDTO.setLongitude(new BigDecimal(storeDTO.getLongitude()));
            }
            if (org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getLatitude())) {
                weimobPhysicalStoreDTO.setLatitude(new BigDecimal(storeDTO.getLatitude()));
            }
            //省市区名称
            String fullArea = org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getProvinces()) ?
                    storeDTO.getProvinces().replaceAll("//", "") : "";
            weimobPhysicalStoreDTO.setAddress(fullArea + storeDTO.getAddress());
            StringBuilder businessTime = new StringBuilder();
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm");
            List<StoreBusinessTimeDTO> businessTimeList = storeDTO.getBusinessTimeList();
            if (CollectionUtils.isNotEmpty(businessTimeList)) {
                businessTimeList.stream().filter(e -> e != null && StringUtils.isNotBlank(e.getWeekday())).map(e -> {
                    String businessHours = simpleDateFormat.format(e.getOpenTime()) + "-" + simpleDateFormat.format(e.getCloseTime());
                    return businessTime(e.getWeekday()) + ":"  + businessHours;
                }).forEach(e -> {
                    businessTime.append(e).append("；");
                });
            }
            weimobPhysicalStoreDTO.setBussinessHours(businessTime.toString());

            WeimobStoreAccountDTO accountDTO = new WeimobStoreAccountDTO();
            ClerkDTO headClerk = getHeadClerk(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
            if (headClerk != null) {
                accountDTO.setManagerLoginAccount(headClerk.getPhoneNumber());
                accountDTO.setManagerName(headClerk.getClerkName());
            }
            if (org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getConactsPhone())) {
                weimobPhysicalStoreDTO.setStoreTel(storeDTO.getConactsPhone());
            } else {
                //店长的号码
                if (headClerk != null) {
                    weimobPhysicalStoreDTO.setStoreTel(headClerk.getPhoneNumber());
                }
            }
            weimobPhysicalStoreDTO.setAccountVo(accountDTO);
            return weimobPhysicalStoreDTO;
        }
        return null;
    }

    private static String businessTime(String week) {
        StringBuilder businessTime = new StringBuilder();
        Arrays.asList(week.split(","))
                .stream()
                .filter(e -> org.apache.commons.lang.StringUtils.isNumeric(e))
                .mapToInt(e -> Integer.parseInt(e))
                .boxed()
                .sorted()
                .collect(Collectors.toList())
                .forEach(e -> {
                    businessTime.append(BUSINESS_TIME_MAP.get(e.toString())).append("、");
                });
        String result = businessTime.toString();
        return result.substring(0, result.length() - 1);
    }

    private ClerkDTO getHeadClerk(Integer enterpriseId, Integer storeId) {
        ServiceResponse<ClerkDTO> clerkResponse = clerkApiService.getHeadClerk(enterpriseId, storeId);
        if (clerkResponse.isSuccess()) {
            return clerkResponse.getResult();
        }
        return null;
    }

}
