package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.enterprise.service.PlatformBrandApiService;
import com.gic.store.dto.StoreBrandDTO;
import com.gic.store.entity.TabStoreBrand;
import com.gic.store.service.StoreBrandApiService;
import com.gic.store.service.StoreBrandService;
import com.gic.store.service.StoreService;
import com.gic.store.utils.MoveType;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @date 2019/6/26
 */
@Service("storeBrandApiService")
@SuppressWarnings("unchecked")
public class StoreBrandApiServiceImpl implements StoreBrandApiService {

    private static final Logger logger = LogManager.getLogger(StoreBrandApiServiceImpl.class);

    @Autowired
    private StoreBrandService storeBrandService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private PlatformBrandApiService platformBrandApiService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;


    @Override
    public ServiceResponse<Integer> saveOrUpdateStoreBrand(StoreBrandDTO storeBrandDTO) {
        Integer storeBrandId;
        int storeBrandName = storeBrandService.countByStoreBrandName(storeBrandDTO.getEnterpriseId(), storeBrandDTO.getStoreBrandName(), storeBrandDTO.getStoreBrandId());
        if (storeBrandName > 0) {
            return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "品牌名称已存在");
        }
        if (storeBrandDTO.getStoreBrandId() == null) {
            // 新增
            storeBrandId = storeBrandService.save(storeBrandDTO);
            storeBrandDTO.setStoreBrandId(storeBrandId);
            addRef(storeBrandDTO);
            enterpriseInitApiService.createStoreBrand(storeBrandDTO.getEnterpriseId());
        } else {
            // 编辑
            storeBrandId = storeBrandDTO.getStoreBrandId();
            Integer updateLine = storeBrandService.update(storeBrandDTO);
            if (updateLine == 0) {
                logger.warn("[storeBrandId ==> {}] 不存在", storeBrandDTO.getStoreBrandId());
                return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "没找到要编辑的门店品牌");
            }
            updateRef(storeBrandDTO);
        }
        return ServiceResponse.success(storeBrandId);
    }

    @Override
    public ServiceResponse<Page<StoreBrandDTO>> listStoreBrand(Integer enterpriseId, String search, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page storeBrandList = storeBrandService.listStoreBrand(enterpriseId, search, pageNum, pageSize);
        Page<StoreBrandDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeBrandList, StoreBrandDTO.class);
        List<StoreBrandDTO> resultList = page.getResult();
        int index = (pageNum - 1) * pageSize + 1;
        for (int i = 0; i < resultList.size(); i++) {
            resultList.get(i).setSeq(index + i);
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<List<StoreBrandDTO>> listAllStoreBrand(Integer enterpriseId, String search) {
        List<TabStoreBrand> storeBrandList = storeBrandService.listAllStoreBrand(enterpriseId, search);
        List<StoreBrandDTO> brandDTOList = EntityUtil.changeEntityListByOrika(StoreBrandDTO.class, storeBrandList);
        return ServiceResponse.success(brandDTOList);
    }

    @Override
    public ServiceResponse deleteAll(Integer enterpriseId) {
        List<TabStoreBrand> storeBrandList = storeBrandService.listAllStoreBrand(enterpriseId, null);
        if (CollectionUtils.isEmpty(storeBrandList)) {
            return ServiceResponse.success();
        }
        String ids = storeBrandList.stream().map(e -> e.getStoreBrandId().toString()).collect(Collectors.joining(","));
        return this.deleteByIds(enterpriseId, ids);
    }

    @Override
    public ServiceResponse deleteByIds(Integer enterpriseId, String ids) {
        List<Integer> notDelList = new ArrayList<>();
        if (StringUtils.isNotBlank(ids)) {
            String[] idArr = ids.split(GlobalInfo.FLAG_COMMA);
            List<Integer> delList = Stream.of(idArr)
                    .map(Integer::parseInt)
                    .filter(id -> {
                        Integer count = storeService.countByBrandId(id);
                        if (count > 0) {
                            notDelList.add(id);
                        } else {
                            TabStoreBrand tabStoreBrand = this.storeBrandService.getById(id);
                            if (Constants.NORMAL_STATUS.equals(tabStoreBrand.getHasRel())) {
                                notDelList.add(id);
                            }
                        }
                        return count == 0;
                    })
                    .collect(Collectors.toList());

            if (CollectionUtils.isEmpty(notDelList)) {
                // 要删除品牌
                storeBrandService.deleteByIds(enterpriseId, delList);
                delRef(enterpriseId, delList);
                return ServiceResponse.success();
            } else {
                // 有不能删除的品牌
                List<TabStoreBrand> storeBrandList = storeBrandService.listStoreBrandByIds(notDelList);
                EntityUtil.changeEntityListByOrika(StoreBrandDTO.class, storeBrandList);
                return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "部分数据在使用中", EntityUtil.changeEntityListByOrika(StoreBrandDTO.class, storeBrandList));
            }
        } else {
            // 没有要删除的品牌
            return ServiceResponse.success();
        }
    }

    @Override
    public ServiceResponse moveStoreBrand(Integer enterpriseId, Integer storeBrandId, Integer moveType) {
        if (MoveType.MOVE_UP.getCode().equals(moveType)) {
            // 上移
            storeBrandService.sortMoveUp(enterpriseId, storeBrandId);
        } else if (MoveType.MOVE_DOWN.getCode().equals(moveType)) {
            // 下移
            storeBrandService.sortMoveDown(enterpriseId, storeBrandId);
        } else if (MoveType.TOPPING.getCode().equals(moveType)) {
            // 置顶
            storeBrandService.sortTopping(enterpriseId, storeBrandId);
        } else if (MoveType.BOTTOM.getCode().equals(moveType)) {
            // 置底
            storeBrandService.sortBottom(enterpriseId, storeBrandId);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse setStoreBrandSort(Integer enterpriseId, Integer storeBrandId, Integer seq) {
        storeBrandService.setStoreBrandSort(enterpriseId, storeBrandId, seq);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<StoreBrandDTO>> listStoreBrandByIds(List<Integer> brandIdList) {
        List<TabStoreBrand> tabStoreBrands = this.storeBrandService.listStoreBrandByIds(brandIdList);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(StoreBrandDTO.class, tabStoreBrands));
    }

    @Override
    public ServiceResponse<List<StoreBrandDTO>> listAllStoreBrandInfoByIds(List<Integer> brandIdList) {
        return ServiceResponse.success(EntityUtil.changeEntityListNew(StoreBrandDTO.class, storeBrandService.listAllStoreBrandInfoByIds(brandIdList)));
    }

    @Override
    public ServiceResponse<StoreBrandDTO> getById(Integer storeBrandId) {
        TabStoreBrand storeBrand = this.storeBrandService.getById(storeBrandId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(StoreBrandDTO.class, storeBrand));
    }

    @Override
    public ServiceResponse<Void> updateRelByOperation(Integer storeBrandId, Integer hasRel) {
        this.storeBrandService.updateRelByOperation(storeBrandId, hasRel);
        return EnterpriseServiceResponse.success();
    }

    private void addRef(StoreBrandDTO storeBrandDTO) {
        ServiceResponse<Integer> serviceResponse = platformBrandApiService.addRef(storeBrandDTO.getEnterpriseId(), storeBrandDTO.getStoreBrandId(), storeBrandDTO.getStoreBrandName(), storeBrandDTO.getStoreBrandCode(), storeBrandDTO.getStoreBrandCategoryCode());
        if (serviceResponse.isSuccess() && serviceResponse.getResult() != null) {
            logger.info("添加关联成功， refId:{}", serviceResponse.getResult());
        } else {
            logger.info("添加关联失败， {}", JSON.toJSONString(storeBrandDTO));
        }
    }

    private void updateRef(StoreBrandDTO storeBrandDTO) {
        platformBrandApiService.updateRef(storeBrandDTO.getEnterpriseId(), storeBrandDTO.getStoreBrandId(),storeBrandDTO.getStoreBrandName(), storeBrandDTO.getStoreBrandCode(), storeBrandDTO.getStoreBrandCategoryCode());
    }

    private void delRef(Integer enterpriseId, List<Integer> delList) {
        platformBrandApiService.delRef(enterpriseId, delList);
    }

}
