package com.gic.store.service.outer.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.gic.store.entity.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.application.center.api.dto.ApplicationResourceDTO;
import com.gic.application.center.api.service.AuthorizeService;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.erp.operation.api.dto.ApiFieldDTO;
import com.gic.erp.operation.api.service.ApiFieldApiService;
import com.gic.gateway.sdk.anno.GatewayExporter;
import com.gic.gateway.sdk.anno.GatewayParams;
import com.gic.store.constant.ClerkGenderEnum;
import com.gic.store.constant.CreateTypeEnum;
import com.gic.store.constant.StoreLogReasonEnum;
import com.gic.store.dto.*;
import com.gic.store.dto.open.*;
import com.gic.store.dto.open.ClerkDTO;
import com.gic.store.dto.open.StoreGroupDTO;
import com.gic.store.service.*;

/**
 * 
 * @ClassName: StoreOpenApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-09 17:54
 */
@Service("storeOpenApiService")
@GatewayExporter
public class StoreOpenApiServiceImpl implements StoreOpenApiService {
    private static final Logger LOGGER = LogManager.getLogger(StoreOpenApiServiceImpl.class);
    @Autowired
    private ApiFieldApiService apiFieldApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private ClerkApiService clerkApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreBrandService storeBrandService;
    @Autowired
    private ProvincesApiService provincesApiService;
    @Autowired
    private StoreFieldService storeFieldService;
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private AuthorizeService authorizeService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreRegionService storeRegionService;
    @Autowired
    private StoreGroupApiService storeGroupApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private ClerkPositionService clerkPositionService;

    @Override
    public ServiceResponse<String> getApiFieldJson(Integer enterpriseId, Integer regionId) {
        ServiceResponse<List<ApiFieldDTO>> result = apiFieldApiService.listApiField(2, enterpriseId,
                Long.parseLong(regionId.toString()));
        if (result.isSuccess()) {
            List<ApiFieldDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return ServiceResponse.success(JSON.toJSONString(list));
            }
        }
        return ServiceResponse.success();
    }

    public Map<String, String> getApiFieldMap(Integer enterpriseId, Integer regionId) {
        ServiceResponse<List<ApiFieldDTO>> result = apiFieldApiService.listApiField(2, enterpriseId, Long.parseLong(regionId.toString()));
        if (result.isSuccess()) {
            List<ApiFieldDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return list.stream().collect(Collectors.toMap(ApiFieldDTO::getFieldCode, ApiFieldDTO::getGicFieldName));
            }
        }
        return null;
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "storeCode", "regionId"})
    public ServiceResponse<String> getStoreDetail(String appId, Integer enterpriseId, String storeCode, Integer regionId, Integer storeInfoId) {
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if(storeInfoId == null){
            if(regionId == null || StringUtils.isBlank(storeCode)){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域或者门店code不能为空");
            }
            StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, regionId, storeCode);
            if (store == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
            }
            storeInfoId = store.getStoreInfoId();
        }
        if (!hasAuth(appId, enterpriseId, storeInfoId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        StoreDTO store = storeApiService.getStoreByStoreInfoId(enterpriseId, storeInfoId).getResult();
        if (store == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
        }
        return ServiceResponse.success(JSON.toJSONString(store));
    }

    @Override
    @GatewayParams(query = {"appId","enterpriseId"}, body = {"storeOpenDTO"})
    public ServiceResponse<String> addStore(String appId, Integer enterpriseId, StoreOpenDTO storeOpenDTO) {
        LOGGER.info("对外API新增门店参数：{}", JSON.toJSONString(storeOpenDTO));
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        StoreDTO storeDTO = EntityUtil.changeEntityByJSON(StoreDTO.class, storeOpenDTO);
        storeDTO.setEnterpriseId(enterpriseId);
        if (StringUtils.isBlank(storeOpenDTO.getStoreName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店名称不能为空");
        }
        if(storeOpenDTO.getStoreInfoId() == null){
            if(storeOpenDTO.getRegionId() == null || StringUtils.isBlank(storeOpenDTO.getStoreCode())){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域或者门店code不能为空");
            }
        }
        if (storeOpenDTO.getStoreBrandId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店招品牌不能为空");
        }
        // 校验权限
        if (!hasAuth(appId, enterpriseId, storeOpenDTO.getStoreInfoId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        // 校验门店类型
        ServiceResponse<String> x2 = praseStoreType(enterpriseId, storeOpenDTO, storeDTO);
        if (x2 != null) {
            return x2;
        }
        // 校验门店状态
        ServiceResponse<String> x1 = validErpStatus(enterpriseId, storeOpenDTO, storeDTO);
        if (x1 != null) {
            return x1;
        }
        if (StringUtils.isNotBlank(storeOpenDTO.getGroupName())) {
            TabStoreGroup group = storeGroupService.getStoreGroupByName(storeOpenDTO.getGroupName(), enterpriseId);
            if (group == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店分组不存在");
            } else {
                storeDTO.setStoreGroupId(group.getStoreGroupId());
            }
        }
        TabStoreBrand brand = storeBrandService.getById(storeOpenDTO.getStoreBrandId());
        if (brand == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店招品牌不存在");
        }
        storeDTO.setBrandIds(brand.getStoreBrandId().toString());
        ServiceResponse<String> x = parseArea(storeOpenDTO, storeDTO);
        if (x != null) {
            return x;
        }
        // 营业时间
        if (parsebusinessTime(storeOpenDTO, storeDTO)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "输入的营业时间有误");
        }
        // 门店照片
        parsePhoto(storeOpenDTO, storeDTO);
        // 自定义属性
        ServiceResponse<String> fieldResponse = parseCustomField(enterpriseId, storeOpenDTO, storeDTO);
        if (fieldResponse != null) {
            return fieldResponse;
        }
        storeDTO.setReason(StoreLogReasonEnum.EXTERNAL.getCode());
        storeDTO.setCreateType(CreateTypeEnum.API.getCode());
        if (storeDTO.getStoreExtendList() == null) {
            storeDTO.setStoreExtendList(Collections.emptyList());
        }
        // 保存门店
//        return null;
        if(storeOpenDTO.getStoreInfoId() != null){
            Integer storeId = storeService.getStoreIdByStoreInfoId(enterpriseId, storeOpenDTO.getStoreInfoId());
            storeDTO.setStoreId(storeId);
        }
        ServiceResponse<StoreDTO> serviceResponse = storeApiService.saveOrUpdate(storeDTO);
        if (!serviceResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        } else {
            return EnterpriseServiceResponse.success();
        }
    }

    private void parsePhoto(StoreOpenDTO storeOpenDTO, StoreDTO storeDTO) {
        if (StringUtils.isNotBlank(storeOpenDTO.getPhotos())) {
            List<String> photoList = JSON.parseArray(storeOpenDTO.getPhotos(), String.class);
            List<StorePhotoDTO> photoResultList = new ArrayList<>();
            for (String imageUrl : photoList) {
                StorePhotoDTO storePhotoDTO = new StorePhotoDTO();
                storePhotoDTO.setImageUrl(imageUrl);
                photoResultList.add(storePhotoDTO);
            }
            storeDTO.setPhotoList(photoResultList);
        }
    }

    private ServiceResponse<String> praseStoreType(Integer enterpriseId, StoreOpenDTO storeOpenDTO, StoreDTO storeDTO) {
        if (storeOpenDTO.getStoreType() != null) {
            ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreType(enterpriseId);
            if (serviceResponse.isSuccess()) {
                List<StoreDictDTO> list = serviceResponse.getResult();
                Integer erpType = storeOpenDTO.getStoreType();
                String storeTypeTmp = erpType.toString();
                for (StoreDictDTO storeDictDTO : list) {
                    if (storeDictDTO.getValue().equals(storeTypeTmp)) {
                        storeDTO.setStoreType(Integer.valueOf(storeDictDTO.getValue()));
                        break;
                    }
                }
            } else {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "调用门店类型接口失败");
            }
            // 没有匹配上
            if (storeDTO.getStoreType() == null) {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "门店类型不存在");
            }
        }
        return null;
    }

    private ServiceResponse<String> validErpStatus(Integer enterpriseId, StoreOpenDTO storeOpenDTO, StoreDTO storeDTO) {
        if (storeOpenDTO.getErpStatus() != null) {
            ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreStatus(enterpriseId);
            if (serviceResponse.isSuccess()) {
                List<StoreDictDTO> list = serviceResponse.getResult();
                Integer erpStatus = storeOpenDTO.getErpStatus();
                String storeStatusType = erpStatus.toString();
                for (StoreDictDTO storeDictDTO : list) {
                    if (storeDictDTO.getValue().equals(storeStatusType)) {
                        storeDTO.setStoreType(Integer.valueOf(storeDictDTO.getValue()));
                        break;
                    }
                }
            } else {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "调用门店状态接口失败");
            }
            // 没有匹配上
            if (storeDTO.getStoreType() == null) {
                return EnterpriseServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "门店状态不存在");
            }
        }
        return null;
    }

    private boolean parsebusinessTime(StoreOpenDTO storeOpenDTO, StoreDTO storeDTO) {
        if (StringUtils.isNotBlank(storeOpenDTO.getBusinessTime())) {
            try {
                List<BusinessTimeOpenDTO> businessTimeList = JSON.parseArray(storeOpenDTO.getBusinessTime(), BusinessTimeOpenDTO.class);
                List<StoreBusinessTimeDTO> businessTimeResultList = new ArrayList<>();
                for (BusinessTimeOpenDTO businessTimeOpenDTO : businessTimeList) {
                    StoreBusinessTimeDTO businessTimeDTO = new StoreBusinessTimeDTO();
                    businessTimeDTO.setWeekday(businessTimeOpenDTO.getWeekday());
                    businessTimeDTO.setOpenTime(DateUtil.strToDate(businessTimeOpenDTO.getOpenTime(), DateUtil.FORMAT_MINUTE_TIME));
                    businessTimeDTO.setCloseTime(DateUtil.strToDate(businessTimeOpenDTO.getCloseTime(), DateUtil.FORMAT_MINUTE_TIME));
                    businessTimeResultList.add(businessTimeDTO);
                }
                storeDTO.setBusinessTimeList(businessTimeResultList);
            } catch (Exception e) {
                LOGGER.info("输入的营业时间有误:{}", storeOpenDTO.getBusinessTime(), e);
                return true;
            }
        }
        return false;
    }

    private ServiceResponse<String> parseCustomField(Integer enterpriseId, StoreOpenDTO storeOpenDTO, StoreDTO storeDTO) {
        List<StoreFieldDTO> fieldList = storeFieldService.listStoreFieldByRegionId(storeOpenDTO.getRegionId());
        String customField = storeOpenDTO.getCustomField();
        if (StringUtils.isBlank(customField)) {
            customField = "[]";
        }
        List<JSONObject> customFieldList = JSON.parseArray(customField, JSONObject.class);
        if (CollectionUtils.isNotEmpty(customFieldList)) {

            Map<String, String> apiFieldMap = getApiFieldMap(enterpriseId, storeOpenDTO.getRegionId());
            if (apiFieldMap == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "获取gic端和对外API之间门店自定义字段的匹配数据失败");
            }
            for (JSONObject jsonObject : customFieldList) {
                String fieldCode = jsonObject.getString("fieldCode");
                String fieldName = apiFieldMap.get(fieldCode);
                jsonObject.put("fieldName", fieldName);
            }

            if (CollectionUtils.isNotEmpty(fieldList)) {
                List<StoreExtendDTO> extendDTOList = new ArrayList<>();
                for (int i = 0; i < fieldList.size(); i++) {
                    StoreExtendDTO storeExtendDTO = new StoreExtendDTO();
                    storeExtendDTO.setStoreFieldId(fieldList.get(i).getStoreFieldId());
                    storeExtendDTO.setStoreFieldType(fieldList.get(i).getStoreFieldType());
                    storeExtendDTO.setStoreFieldName(fieldList.get(i).getStoreFieldName());
                    storeExtendDTO.setStoreFieldCode(fieldList.get(i).getStoreFieldCode());
                    for (JSONObject storeFieldObject : customFieldList) {
                        if (fieldList.get(i).getStoreFieldName().equals(storeFieldObject.getString("fieldName"))) {
                            storeExtendDTO.setValue(storeFieldObject.getString("fieldValue"));
                            break;
                        }
                    }
                    storeExtendDTO.setSort(fieldList.get(i).getSort());
                    // 验证
                    ServiceResponse fieldResponse = storeApiService.validateFieldExceptRepeat(storeExtendDTO.getValue(), fieldList.get(i));
                    if (!fieldResponse.isSuccess()) {
                        return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), fieldResponse.getMessage());
                    }
                    extendDTOList.add(storeExtendDTO);
                }
                storeDTO.setStoreExtendList(extendDTOList);
            } else {
                storeDTO.setStoreExtendList(Collections.emptyList());
            }
        }
        return null;
    }

    private ServiceResponse<String> parseArea(StoreOpenDTO storeOpenDTO, StoreDTO storeDTO) {
        if(StringUtils.isNotBlank(storeOpenDTO.getProvince()) && StringUtils.isNotBlank(storeOpenDTO.getCity()) && StringUtils.isNotBlank(storeOpenDTO.getCounty())) {
            ServiceResponse<List<ProvinceDTO>> provinceResponse = provincesApiService.selectAllProvince();
            ServiceResponse<List<CityDTO>> cityResponse = provincesApiService.selectAllCity();
            ServiceResponse<List<CountyDTO>> countyResponse = provincesApiService.selectAllCounty();
            List<ProvinceDTO> provinceDTOList = provinceResponse.getResult();
            String finalProvince = storeOpenDTO.getProvince();
            ProvinceDTO provinceDTO = provinceDTOList.stream().filter(e -> StringUtils.equals(e.getProvinceName(), finalProvince)).findFirst().orElse(null);
            List<CityDTO> cityDTOList = cityResponse.getResult();
            String finalCity = storeOpenDTO.getCity();
            CityDTO cityDTO = cityDTOList.stream().filter(e -> StringUtils.equals(e.getCityName(), finalCity)).findFirst().orElse(null);
            List<CountyDTO> countyDTOList = countyResponse.getResult();
            String finalCounty = storeOpenDTO.getCounty();
            CountyDTO countyDTO = countyDTOList.stream().filter(e -> StringUtils.equals(e.getCountyName(), finalCounty)).findFirst().orElse(null);
            if (countyDTO == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "输入的区县信息有误");
            }
            if (cityDTO == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "输入的市信息有误");
            }
            if (provinceDTO == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "输入的省信息有误");
            }
            storeDTO.setProvinceId(provinceDTO.getProvinceId());
            storeDTO.setCityId(cityDTO.getCityId());
            storeDTO.setAreaId(countyDTO.getCountyId());
            storeDTO.setProvinces(provinceDTO.getProvinceName() + "/" + cityDTO.getCityName() + "/" + countyDTO.getCountyName());
        }
        return null;
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId"}, body = {"storeOpenDTO"})
    public ServiceResponse<String> editStore(String appId, Integer enterpriseId, StoreOpenDTO storeOpenDTO) {
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }

        if (StringUtils.isBlank(storeOpenDTO.getStoreCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code不能为空");
        }
        if (storeOpenDTO.getRegionId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        if(storeOpenDTO.getStoreInfoId() == null){
            if(storeOpenDTO.getRegionId() == null || StringUtils.isBlank(storeOpenDTO.getStoreCode())){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域或者门店code不能为空");
            }
            StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, storeOpenDTO.getRegionId(), storeOpenDTO.getStoreCode());
            if (store == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
            }
            storeOpenDTO.setStoreInfoId(store.getStoreInfoId());
        }
        // 校验权限
        if (!hasAuth(appId, enterpriseId, storeOpenDTO.getStoreInfoId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }

        return this.addStore(appId, enterpriseId, storeOpenDTO);
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "regionId", "pageNum", "pageSize"})
    public ServiceResponse<String> pageStore(String appId, Integer enterpriseId, Integer regionId, Integer pageNum, Integer pageSize) {
        if (enterpriseId == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户id不能为空");
        }
        if (regionId == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setRegionId(regionId);

        // 校验权限
        if (!hasAuth(appId, enterpriseId, null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        String storeIds = getStoreIds(appId, enterpriseId, regionId).toString();
        if (StringUtils.isBlank(storeIds)) {
            return ServiceResponse.success(JSON.toJSONString(new Page()));
        }
        storeSearchDTO.setStoreIds(storeIds);

        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }

        ServiceResponse<Page<StoreDTO>> serviceResponse = this.storeApiService.listStore(storeSearchDTO, pageNum, pageSize);
        if (!serviceResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<StoreDTO> page = serviceResponse.getResult();
        return ServiceResponse.success(JSON.toJSONString(page));
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "regionId", "storeCode", "clerkCode", "storeInfoId"})
    public ServiceResponse<String> getClerkDetail(String appId, Integer enterpriseId, Integer regionId, String storeCode, String clerkCode, Integer storeInfoId) {
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if(storeInfoId == null){
            if(regionId == null || StringUtils.isBlank(storeCode)){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域或者门店code不能为空");
            }
            StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, regionId, storeCode);
            if (store == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
            }
            storeInfoId = store.getStoreInfoId();
            storeCode = store.getStoreCode();
        }
        if (!hasAuth(appId, enterpriseId, storeInfoId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        if (StringUtils.isBlank(clerkCode)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购code不能为空");
        }
        TabClerk clerk = this.clerkService.getClerkByCode(enterpriseId, clerkCode, storeInfoId);
        if (clerk == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购代码错误，无此导购");
        }
        ClerkDetailDTO result = EntityUtil.changeEntityNew(ClerkDetailDTO.class, clerk);
        result.setStoreCode(storeCode);
        return ServiceResponse.success(JSON.toJSONString(result));
    }

    @Override
    @GatewayParams(query = { "appId", "enterpriseId"}, body = {"clerkInfo"})
    public ServiceResponse<Void> addClerk(String appId, Integer enterpriseId, ClerkDTO clerkInfo) {
        LOGGER.info("对外API新增导购参数:{}", JSON.toJSONString(clerkInfo));
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if(clerkInfo.getStoreInfoId() == null){
            if(clerkInfo.getRegionId() == null || StringUtils.isBlank(clerkInfo.getStoreCode())){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域或者门店code不能为空");
            }
            StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, clerkInfo.getRegionId(), clerkInfo.getStoreCode());
            if (store == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
            }
            clerkInfo.setStoreInfoId(store.getStoreInfoId());
        }

        if (!hasAuth(appId, enterpriseId, clerkInfo.getStoreInfoId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        if (StringUtils.isBlank(clerkInfo.getClerkCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购code不能为空");
        }
        if (StringUtils.isBlank(clerkInfo.getClerkName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购名称不能为空");
        }
        Integer gender = clerkInfo.getClerkGender();
        boolean genderIllegal = gender == null || (gender != ClerkGenderEnum.MALE.getCode() && gender != ClerkGenderEnum.FEMALE.getCode());
        if (genderIllegal) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购性别值非法");
        }
        String phone = clerkInfo.getPhoneNumber();
        if (StringUtils.isBlank(phone)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号不能为空");
        }
        if (phone.trim().length() != com.gic.store.constant.Constants.CLERK_IMPORT_PHONE_LENGTH) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号非法");
        }

        com.gic.store.dto.ClerkDTO dto = EntityUtil.changeEntityNew(com.gic.store.dto.ClerkDTO.class, clerkInfo);
        dto.setNationcode(Constants.NATION_CODE);
        dto.setEnterpriseId(enterpriseId);
        dto.setPositionId(clerkInfo.getClerkPositionId());
        //接口会验证
        // 权限 、 手机号重复 、 导购名称重复 、导购code重复
        return clerkApiService.saveOrUpdate(dto);
    }

    @Override
    @GatewayParams(query = { "appId", "enterpriseId"}, body = {"clerkInfo"})
    public ServiceResponse<Void> editClerk(String appId, Integer enterpriseId, ClerkDTO clerkInfo) {
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if(clerkInfo.getStoreInfoId() == null){
            if(clerkInfo.getRegionId() == null || StringUtils.isBlank(clerkInfo.getStoreCode())){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域或者门店code不能为空");
            }
            StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, clerkInfo.getRegionId(), clerkInfo.getStoreCode());
            if (store == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
            }
            clerkInfo.setStoreInfoId(store.getStoreInfoId());
        }

        if (!hasAuth(appId, enterpriseId, clerkInfo.getStoreInfoId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        if (StringUtils.isBlank(clerkInfo.getClerkCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购code不能为空");
        }
        TabClerk clerk = this.clerkService.getClerkByCode(enterpriseId, clerkInfo.getClerkCode(), clerkInfo.getStoreInfoId());
        if (clerk == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购code错误，无此导购");
        }

        if (StringUtils.isBlank(clerkInfo.getClerkName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购名称不能为空");
        }
        Integer gender = clerkInfo.getClerkGender();
        boolean genderIllegal = gender == null || (gender != ClerkGenderEnum.MALE.getCode() && gender != ClerkGenderEnum.FEMALE.getCode());
        if (genderIllegal) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购性别值非法");
        }
        String phone = clerkInfo.getPhoneNumber();
        if (StringUtils.isBlank(phone)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号不能为空");
        }
        if (phone.trim().length() != com.gic.store.constant.Constants.CLERK_IMPORT_PHONE_LENGTH) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号非法");
        }

        com.gic.store.dto.ClerkDTO dto = EntityUtil.changeEntityNew(com.gic.store.dto.ClerkDTO.class, clerkInfo);
        dto.setNationcode(Constants.NATION_CODE);
        dto.setEnterpriseId(enterpriseId);
        dto.setClerkId(clerk.getClerkId());
        //接口会验证
        // 权限 、 手机号重复 、 导购名称重复 、导购code重复
        return clerkApiService.saveOrUpdate(dto);
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "storeCode", "regionId", "pageNum", "pageSize"})
    public ServiceResponse<String> pageClerk(String appId, Integer enterpriseId, String storeCode, Integer regionId, Integer storeInfoId,
                                             Integer pageNum, Integer pageSize) {
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        if (regionId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不能为空");
        }
        if (!hasAuth(appId, enterpriseId, storeInfoId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
        }
        StringBuilder storeIds = new StringBuilder();
        if (StringUtils.isBlank(storeCode)) {
            //门店域有权限，但是域下面的门店不一定都有权限，进行过滤
            storeIds = getStoreIds(appId, enterpriseId, regionId);
            if (StringUtils.isBlank(storeIds.toString())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有门店权限");
            }
        } else {
            StoreDTO store = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, regionId, storeCode);
            if (store == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店code或者域数据错误，无此门店");
            }
            storeIds.append(store.getStoreId());
        }
        //查询门店下所有的导购
        ClerkSearchDTO clerkSearchDTO = new ClerkSearchDTO();
        clerkSearchDTO.setPageSize(pageSize);
        clerkSearchDTO.setCurrentPage(pageNum);
        clerkSearchDTO.setStoreIds(storeIds.toString());
        ServiceResponse<Page<ClerkStoreListDTO>> clerkResult = clerkApiService.listClerk(enterpriseId, clerkSearchDTO, pageNum, pageSize);
        if (clerkResult.isSuccess()) {
            return ServiceResponse.success(JSON.toJSONString(clerkResult));
        }
        return ServiceResponse.failure(clerkResult.getCode(), clerkResult.getMessage());
    }

    @Override
    @GatewayParams(query = {"enterpriseId"})
    public ServiceResponse<List<ClerkPositionDTO>> listClerkPosition(Integer enterpriseId){
        com.github.pagehelper.Page<TabClerkPosition> positions = this.clerkPositionService.listClerkPosition(enterpriseId, null, 1, 1000);
        List<TabClerkPosition> result = positions.getResult();
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ClerkPositionDTO.class, result));
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId"})
    public ServiceResponse<String> listRegion(String appId, Integer enterpriseId) {
        LOGGER.info("appId:{}, enterpriseId:{}", appId, enterpriseId);
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        List<StoreRegionDTO> regionList = storeRegionService.listStoreRegion(enterpriseId, null, null);
        if (CollectionUtils.isNotEmpty(regionList)) {
            return ServiceResponse.success(JSON.toJSONString(EntityUtil.changeEntityListByJSON(StoreRegionOpenDTO.class, regionList)));
        } else {
            return ServiceResponse.success();
        }
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId"})
    public ServiceResponse<String> listBrand(String appId, Integer enterpriseId) {
        LOGGER.info("appId:{}, enterpriseId:{}", appId, enterpriseId);
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        List<TabStoreBrand> storeBrandList = storeBrandService.listAllStoreBrand(enterpriseId, null);
        if (CollectionUtils.isNotEmpty(storeBrandList)) {
            return ServiceResponse.success(JSON.toJSONString(EntityUtil.changeEntityListByJSON(StoreBrandOpenDTO.class, storeBrandList)));
        } else {
            return ServiceResponse.success();
        }
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "regionId"})
    public ServiceResponse<String> listField(String appId, Integer enterpriseId, Integer regionId) {
        LOGGER.info("appId:{}, enterpriseId:{}", appId, enterpriseId);
        //参数校验
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，无此商户");
        }
        StoreRegionDTO region = storeRegionService.getById(enterpriseId, regionId);
        if (region == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不存在");
        }

        List<StoreFieldDTO> fieldList = storeFieldService.listStoreFieldByRegionId(regionId);
        if (CollectionUtils.isNotEmpty(fieldList)) {
            return ServiceResponse.success(JSON.toJSONString(EntityUtil.changeEntityListByJSON(StoreFieldOpenDTO.class, fieldList)));
        } else {
            return ServiceResponse.success();
        }
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId"})
    public ServiceResponse<String> listStoreGroup(String appId, Integer enterpriseId) {
        //查询全部
        List<TabStoreGroup> list = this.storeGroupService.listStoreGroupByIds(null, enterpriseId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            List<com.gic.store.dto.open.StoreGroupDTO> result = EntityUtil.changeEntityListNew(com.gic.store.dto.open.StoreGroupDTO.class, list);
            return ServiceResponse.success(JSON.toJSONString(result, true));
        }
        return ServiceResponse.success();
    }

    @Override
    @GatewayParams(query = { "appId", "enterpriseId"}, body = {"storeGroupInfo"})
    public ServiceResponse<String> saveStoreGroup(String appId, Integer enterpriseId, StoreGroupDTO storeGroupInfo) {
        com.gic.store.dto.StoreGroupDTO storeGroupDTO = EntityUtil.changeEntityNew(com.gic.store.dto.StoreGroupDTO.class, storeGroupInfo);
        //确保是新增入口
        storeGroupDTO.setStoreGroupId(null);
        storeGroupDTO.setEnterpriseId(enterpriseId);
        ServiceResponse<Integer> result = storeGroupApiService.save(storeGroupDTO);
        if (result.isSuccess()) {
            return ServiceResponse.success();
        }
        return ServiceResponse.failure("1", result.getMessage());
    }

    @Override
    @GatewayParams(query = { "appId", "enterpriseId", "storeGroupId", "storeGroupName"})
    public ServiceResponse<String> editStoreGroup(String appId, Integer enterpriseId, Integer storeGroupId, String storeGroupName) {
        com.gic.store.dto.StoreGroupDTO storeGroupDTO = new com.gic.store.dto.StoreGroupDTO();
        storeGroupDTO.setStoreGroupId(storeGroupId);
        storeGroupDTO.setStoreGroupName(storeGroupName);
        ServiceResponse<Integer> result = storeGroupApiService.updateStoreGroupName(storeGroupDTO);
        if (result.isSuccess()) {
            return ServiceResponse.success();
        }
        return ServiceResponse.failure("1", result.getMessage());
    }

    @Override
    @GatewayParams(query = {"appId", "enterpriseId", "storeGroupId"})
    public ServiceResponse<String> deleteStoreGroup(String appId, Integer enterpriseId, Integer storeGroupId) {
        ServiceResponse<String> result = storeGroupApiService.remove(storeGroupId);
        if (result.isSuccess()) {
            return ServiceResponse.success();
        }
        return ServiceResponse.failure("1", result.getMessage());
    }


    /**
     * 验证权限门店相关权限
     * @param appId
     * @param enterpriseId
     * @param storeInfoId
     * @return
     */
    private boolean hasAuth(String appId, Integer enterpriseId, Integer storeInfoId) {
        Map<Integer, Integer> result = getAuthByAppId(appId, enterpriseId);
        if (MapUtils.isEmpty(result)) {
            return true;
        }
        if(result.containsKey(storeInfoId)){
            return true;
        }
        return false;
    }

    /**
     * 获取有权限的门店域的有权限的所有门店，前提是门店域有权限
     * @param appId
     * @param enterpriseId
     * @param regionId
     * @return
     */
    private StringBuilder getStoreIds(String appId, Integer enterpriseId, Integer regionId) {
        StringBuilder storeIds = new StringBuilder();
        Map<Integer, Integer> result = getAuthByAppId(appId, enterpriseId);
        if (MapUtils.isNotEmpty(result)) {
            result.keySet().forEach(id -> {
                storeIds.append(id).append(" ");
            });
        }
        return storeIds;
    }

    /**
     * 获取资源组门店资源
     * @param appId
     * @param enterpriseId
     * @return
     */
    private Map<Integer, Integer> getAuthByAppId(String appId, Integer enterpriseId) {
        Map<Integer, Integer> result = new HashMap<>();
        result.put(-99, -99);
        ServiceResponse<ApplicationResourceDTO> res = authorizeService.findResourceByEnterpriseIdAndAppId(Long.valueOf(enterpriseId), appId);
        LOGGER.info("应用资源组内容：{}", JSON.toJSONString(res));
        if (res.isSuccess()) {
            ApplicationResourceDTO resourceDTO = res.getResult();
            if (resourceDTO != null) {
                Integer resourceId = resourceDTO.getResourceId();
                if (resourceId == null) {
                    result.clear();
                    return result;
                }
                ServiceResponse<ResourceGroupDTO> resourceResult = resourceGroupApiService.getResourceGroup(resourceId);
                if (resourceResult.isSuccess() && resourceResult.getResult() != null) {
                    LOGGER.info("对外API权限校验结果-门店资源ID:{}", JSON.toJSONString(resourceResult));
                    Long storeResourceId = resourceResult.getResult().getStoreResource();
                    //查询门店资源组所有的门店权限
                    ServiceResponse<Page<StoreDTO>> storeResult = storeWidgetApiService
                            .listStoreByStoreWidgetId(enterpriseId, storeResourceId.intValue(), 1, 10000);
                    if (storeResult.isSuccess()) {
                        Page<StoreDTO> page = storeResult.getResult();
                        if (CollectionUtils.isEmpty(page.getResult())) {
                            LOGGER.info("没有权限");
                        }
                        List<StoreDTO> list = page.getResult();
                        if (CollectionUtils.isNotEmpty(list)) {
                            result = list.stream().collect(Collectors.toMap(StoreDTO::getStoreInfoId, StoreDTO::getStoreInfoId));
                            return result;
                        }
                    } else {
                        LOGGER.info("对外API权限校验结果-门店资源获取:{}", JSON.toJSONString(storeResult));
                    }
                }
            } else {
                result.clear();
                return result;
            }
            LOGGER.info("对外API权限校验结果-资源ID:{}", JSON.toJSONString(res));
        } else {
            LOGGER.info("对外API权限校验结果:{}", JSON.toJSONString(res));
        }
        LOGGER.info("应用的门店权限：{}", JSON.toJSONString(result));
        return result;
    }
}
