package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.CollectionUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreRegionDTO;
import com.gic.store.entity.TabStoreRegionRel;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreRegionApiService;
import com.gic.store.service.StoreRegionRelService;
import com.gic.store.service.StoreRegionService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/6/24
 */
@Service("storeRegionApiService")
@SuppressWarnings("unchecked")
public class StoreRegionApiServiceImpl implements StoreRegionApiService {

    @Autowired
    private StoreRegionService storeRegionService;
    @Autowired
    private StoreRegionRelService storeRegionRelService;
    @Autowired
    private StoreFieldApiService storeFieldApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> saveOrUpdateStoreRegion(Integer enterpriseId, String regionCode, String regionName, Integer regionId) {
        StoreRegionDTO storeRegionDTO = new StoreRegionDTO();
        storeRegionDTO.setRegionName(regionName);
        storeRegionDTO.setRegionCode(regionCode);
        if (regionId == null) {
            // 新增
            int nameCount = storeRegionService.countByRegionName(enterpriseId, regionName, null);
            if (nameCount > 0) {
                return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "域名称已存在");
            }
            regionId = storeRegionService.save(enterpriseId, regionCode, regionName);
            storeRegionRelService.saveByOwner(enterpriseId, regionId);
            enterpriseInitApiService.createStoreRegion(enterpriseId);
        }else {
            /*
             修改
            判断门店域是自己的 还是授权过来的
             */
            boolean own = storeRegionRelService.validIsOwn(enterpriseId, regionId);
            if (!own) {
                // 不是自己的门店域
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不能修改没有权限的门店域");
            }
            int nameCount = storeRegionService.countByRegionName(enterpriseId, regionName, regionId);
            if (nameCount > 0) {
                return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "域name已存在");
            }
            int update = storeRegionService.update(regionId, regionName);
            if (update == 0) {
                return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
            }
        }
        return ServiceResponse.success(regionId);
    }

    @Override
    public ServiceResponse<Page<StoreRegionDTO>> listStoreRegion(Integer enterpriseId, String search, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<StoreRegionDTO> list = storeRegionService.listStoreRegion(enterpriseId, search, pageNum, pageSize);
        if (CollectionUtils.isNotEmpty(list)) {
            List<Integer> fromEnterpriseIds = list.stream().map(StoreRegionDTO::getFromEnterpriseId).collect(Collectors.toList());
            ServiceResponse<List<EnterpriseDTO>> serviceResponse = enterpriseApiService.listEnterpriseByIds(fromEnterpriseIds);
            if (serviceResponse.isSuccess()) {
                List<EnterpriseDTO> enterpriseList = serviceResponse.getResult();
                Map<Integer, EnterpriseDTO> dtoMap = CollectionUtil.toMap(enterpriseList, "enterpriseId");
                for (StoreRegionDTO storeRegionDTO : list) {
                    EnterpriseDTO dto = dtoMap.get(storeRegionDTO.getFromEnterpriseId());
                    storeRegionDTO.setShareEnterpriseName(dto.getEnterpriseName());
                }
            }
        }
        Page<StoreRegionDTO> page = PageHelperUtils.changePageHelperToCurrentPage(list, StoreRegionDTO.class);
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<List<StoreRegionDTO>> listStoreRegion(Integer enterpriseId, String search) {
        List<StoreRegionDTO> list = storeRegionService.listStoreRegion(enterpriseId, search, null);
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<StoreRegionDTO>> listStoreRegion(Integer enterpriseId, String search, Integer ownType) {
        List<StoreRegionDTO> list = storeRegionService.listStoreRegion(enterpriseId, search, ownType);
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<StoreRegionDTO>> listOwnerStoreRegion(Integer enterpriseId) {
        List<StoreRegionDTO> list = storeRegionService.listStoreRegion(enterpriseId, null, StoreOwnTypeEnum.OWNER.getCode());
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<StoreRegionDTO> getStoreRegionByRegion(Integer enterpriseId, Integer regionId) {
        return ServiceResponse.success(EntityUtil.changeEntityByOrika(StoreRegionDTO.class, storeRegionService.getById(enterpriseId, regionId)));
    }

    @Override
    public ServiceResponse<List<StoreRegionDTO>> listStoreRegionByIds(Integer enterpriseId, List<Integer> storeRegionIdList) {
        List<StoreRegionDTO> list = this.storeRegionService.listStoreRegionByIds(enterpriseId, storeRegionIdList);
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<Integer> authorizeRegionToOther(Integer ownerEnterpriseId, Integer unionEnterpriseId,
                                                           List<Integer> newStoreRegionIdList, Integer authType) {
        if (newStoreRegionIdList == null) {
            newStoreRegionIdList = Collections.emptyList();
        }
        // 新的region是不是自有商户所具有权限的
        List<StoreRegionDTO> list = storeRegionService.listStoreRegionByIds(ownerEnterpriseId, newStoreRegionIdList, StoreOwnTypeEnum.OWNER.getCode());

        if (CollectionUtils.isEmpty(list) || list.size() != newStoreRegionIdList.size()) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }

        // 查询所有联合商户 已经具有的自有商户权限
        List<Integer> oldRelIdList = storeRegionRelService.listByAuthorize(ownerEnterpriseId, unionEnterpriseId);
        Set<Integer> newRelSet = new HashSet<>(newStoreRegionIdList);
        // 要删除的关联
        List<Integer> delRelList = oldRelIdList.stream().filter(e -> !newRelSet.contains(e)).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(delRelList) && authType != 1) {  //重新授权时不需要删除门店域
            storeRegionRelService.delByRegionIds(unionEnterpriseId, delRelList, StoreOwnTypeEnum.OTHER.getCode());
        }
        for (Integer regionId : newStoreRegionIdList) {
            TabStoreRegionRel storeRegionRel = storeRegionRelService.getByRegionIdAndEnterpriseId(regionId, unionEnterpriseId, StoreOwnTypeEnum.OTHER.getCode());
            if (storeRegionRel == null) {
                // 插入
                storeRegionRelService.saveByOther(ownerEnterpriseId, unionEnterpriseId, regionId);
            } else if (storeRegionRel.getDeleteFlag() == 1) {
                // 状态更新为0 delete_flag = 0
                storeRegionRelService.updateStatusByOther(unionEnterpriseId, regionId, 0);
            }
        }
        // 调用门店自定义属性方法
        storeFieldApiService.authStoreField(ownerEnterpriseId, unionEnterpriseId, newStoreRegionIdList);

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> unAuthorizeRegion(Integer toEnterpriseId, Integer storeRegionId) {
        storeRegionRelService.updateStatusByOther(toEnterpriseId, storeRegionId, GlobalInfo.DATA_STATUS_DELETE);
        // TODO 调用删除门店自定义属性的方法

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<StoreRegionDTO> getStoreRegionByRegionName(Integer enterpriseId, String regionName) {
        StoreRegionDTO regionDTO = storeRegionService.getStoreRegionByRegionName(enterpriseId, regionName);
        return EnterpriseServiceResponse.success(regionDTO);
    }

    @Override
    public ServiceResponse<String> relationErp(Integer enterpriseId, Integer storeRegionId) {
        String result = this.storeRegionService.relationErp(enterpriseId, storeRegionId);
        if(StringUtils.isNotBlank(result)){
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), result);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> delRegion(Integer enterpriseId, Integer regionId) {
        Integer integer = this.storeRegionService.delRegion(enterpriseId, regionId);
        return ServiceResponse.success(integer);
    }

}
