package com.gic.store.service.outer.impl;

import java.util.Collections;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.UserApiService;
import com.gic.enterprise.exception.CommonException;
import com.gic.store.dto.StoreResourceDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreResourceApiService;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.store.utils.resource.AuthDataHandleUtils;

/**
 * 门店选择器资源过滤
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/5/26 10:54 AM 
 */
@Service("storeResourceApiService")
public class StoreResourceApiServiceImpl implements StoreResourceApiService{
    private static final Logger LOGGER = LogManager.getLogger(StoreResourceApiServiceImpl.class);
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;

    @Override
    public ServiceResponse<List<StoreResourceDTO>> getAuthDataFromStoreWidget(Integer userId, String esField) {
        UserDTO userDTO = getUser(userId);
        List<StoreResourceDTO> list;
        if (userDTO.getSuperAdmin() == 1) {
            //超管
            list = AuthDataHandleUtils.getAuthData(userDTO.getEnterpriseId(), esField);
            return ServiceResponse.success(list);
        }
        ServiceResponse<com.gic.auth.dto.StoreResourceDTO> storeResource = resourceGroupApiService.getStoreResourceByUserId(userId);
        boolean hasStoreWidgetId = storeResource.isSuccess() && storeResource.getResult() != null
                && storeResource.getResult().getStoreResourceId() != null;
        if (!hasStoreWidgetId) {
            LOGGER.info("该子管理员没有配置资源组门店选择器权限");
            return ServiceResponse.success(getAuthDataWhenNo());
        }
        Integer storeWidgetId = storeResource.getResult().getStoreResourceId();
        ServiceResponse<StoreWidgetDTO> storeWidget = storeWidgetApiService.getStoreWidget(storeWidgetId);
        if (storeWidget.isSuccess()) {
            //todo 目前做了品牌，待补充其他类型
            return ServiceResponse.success(AuthDataHandleUtils.getUserData(storeWidget.getResult(), esField));
        } else {
            LOGGER.info("该子管理员没有配置资源组门店选择器权限");
            return ServiceResponse.success(getAuthDataWhenNo());
        }
    }

    private UserDTO getUser(Integer userId) {
        ServiceResponse<UserDTO> response = userApiService.getUserInfoById(userId);
        if (!response.isSuccess()) {
            throw new CommonException(response.getCode(), response.getMessage());
        }
        return response.getResult();
    }

    /**
     * 没有资源的时候
     * @return
     */
    private static List<StoreResourceDTO> getAuthDataWhenNo() {
        return Collections.emptyList();
    }

}
