package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.store.dto.StoreWidgetLogDTO;
import com.gic.store.dto.StoreWidgetLogSaveDTO;
import com.gic.store.entity.TabStoreWidgetLog;
import com.gic.store.service.StoreWidgetLogApiService;
import com.gic.store.service.StoreWidgetLogService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * 门店选择器操作日志
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/13 3:19 PM 
 */
@Service("storeWidgetLogApiService")
public class StoreWidgetLogApiServiceImpl implements StoreWidgetLogApiService{
    @Autowired
    private StoreWidgetLogService storeWidgetLogService;
    @Autowired
    private UserApiService userApiService;

    @Override
    public ServiceResponse<Integer> saveStoreWidgetLog(StoreWidgetLogSaveDTO storeWidgetLog) {
        //valid param
        if (storeWidgetLog.getStoreWidgetId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店选择器ID不能为空");
        }
        if (storeWidgetLog.getOperationId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "操作人不能为空");
        }
        if (storeWidgetLog.getLogType() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "日志类型不能为空");
        }
        if (storeWidgetLog.getOperationType() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "操作类型不能为空");
        }

        StoreWidgetLogDTO storeWidgetLogDTO = EntityUtil.changeEntityNew(StoreWidgetLogDTO.class, storeWidgetLog);
        //插入操作人的部分冗余字段数据
        ServiceResponse<UserDTO> userResponse = userApiService.getUserInfoById(storeWidgetLog.getOperationId());
        if (userResponse.isSuccess()) {
            UserDTO userDTO = userResponse.getResult();
            storeWidgetLogDTO.setOperationName(userDTO.getUserName())
            .setOperationPosition(userDTO.getUserPosition())
            .setOperationPositionText(getUserPositionText(userDTO.getUserPosition()))
            .setEnterpriseId(userDTO.getEnterpriseId());

            Integer id = storeWidgetLogService.saveStoreWidgetLog(storeWidgetLogDTO);
            return ServiceResponse.success(id);
        }
        return ServiceResponse.failure(userResponse.getCode(), userResponse.getMessage());
    }

    @Override
    public ServiceResponse<Integer> tmpInsert(List<StoreWidgetLogDTO> list) {
        return ServiceResponse.success(storeWidgetLogService.tmpInsert(list));
    }

    @Override
    public ServiceResponse<Void> tmpUpdateByStoreWidgetId(Integer oldStoreWidgetId, Integer newStoreWidgetId) {
        int updateNum = storeWidgetLogService.tmpUpdateByStoreWidgetId(oldStoreWidgetId, newStoreWidgetId);
        if (updateNum > 0) {
            return ServiceResponse.success();
        }
        return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "无数据变更");
    }

    @Override
    public ServiceResponse<Page<StoreWidgetLogDTO>> pageStoreWidgetLog(Integer storeWidgetId, Integer pageNum, Integer pageSize) {
        return ServiceResponse.success(
                PageHelperUtils
                .changePageHelperToCurrentPage(storeWidgetLogService.pageStoreWidgetLog(storeWidgetId, pageNum, pageSize), StoreWidgetLogDTO.class
        ));
    }

    @Override
    public ServiceResponse<List<StoreWidgetLogDTO>> listStoreWidgetLog(Integer storeWidgetId) {
        List<TabStoreWidgetLog> list = storeWidgetLogService.listStoreWidgetLog(storeWidgetId);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(StoreWidgetLogDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<StoreWidgetLogDTO> getLastOperationInfo(Integer storeWidgetId) {
        TabStoreWidgetLog record = storeWidgetLogService.getLastOperationInfo(storeWidgetId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "无数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(StoreWidgetLogDTO.class, record));
    }

    private String getUserPositionText(Integer userPosition) {
        if (userPosition == null) {
            return "--";
        }
        switch (userPosition) {
            case 1:
                return "CRM经理";
            case 2:
                return "商城运营经理";
            case 3:
                return "IT经理";
            default:
                return "--";
        }
    }
}
