package com.gic.store.strategy.impl;

import com.gic.spark.api.service.SparkJobApiService;
import com.gic.store.constant.Constants;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDbDTO;
import com.gic.store.entity.TabStoreBrand;
import com.gic.store.service.StoreBrandService;
import com.gic.store.service.StoreLogService;
import com.gic.store.service.StoreService;
import com.gic.store.strategy.BulkUpdateStoreStrtegy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 
 * @ClassName: StoreBrandUpdateStrategyImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-09 17:53
 */
@Component("5")
public class StoreBrandUpdateStrategyImpl implements BulkUpdateStoreStrtegy {

    @Autowired
    private StoreLogService storeLogService;
    @Autowired
    private StoreBrandService storeBrandService;
    @Autowired
    private SparkJobApiService sparkJobApiService;

    @Override
    public String bulkUpdateStore(String storeInfoIds, Integer enterpriseId, String value, StoreService storeService, Integer operatorId, String operatorName) {
        List<Integer> storeInfoIdList;
        List<StoreDTO> storeList;
        if (Constants.ALL.equals(storeInfoIds)) {
            StoreSearchDbDTO storeSearchDbDTO = new StoreSearchDbDTO();
            storeSearchDbDTO.setOverflowStatus(0);
            storeList = storeService.listStore(storeSearchDbDTO, 1, Integer.MAX_VALUE);
            storeInfoIdList = storeService.listAllStoreInfoId(enterpriseId);
        } else {
            storeInfoIdList = new ArrayList<>();
            String[] storeIdArr = storeInfoIds.split(",");
            for (String s : storeIdArr) {
                storeInfoIdList.add(Integer.valueOf(s));
            }
            StoreSearchDbDTO storeSearchDbDTO = new StoreSearchDbDTO();
            storeSearchDbDTO.setOverflowStatus(0);
            storeSearchDbDTO.setStoreInfoIdList(storeInfoIdList);
            storeList = storeService.listStore(storeSearchDbDTO, 1, Integer.MAX_VALUE);
        }
        List<BrandUpdateDTO> brandUpdateList = new ArrayList<>(storeList.size());
        for (StoreDTO storeDTO : storeList) {
            brandUpdateList.add(new BrandUpdateDTO(storeDTO.getStoreInfoId(), Integer.valueOf(storeDTO.getBrandIds())));
        }
        int i = storeService.bulkUpdateStoreBrands(storeInfoIds, enterpriseId, value);

        List<TabStoreBrand> storeBrandList = storeBrandService.listStoreBrandByIds(value);
        String content = Optional.ofNullable(storeBrandList).orElse(Collections.emptyList()).stream().map(TabStoreBrand::getStoreBrandName).collect(Collectors.joining(","));
        storeLogService.saveBulkStoreInfoLog(storeInfoIdList, enterpriseId, operatorId, operatorName, "门店品牌变更为【" + content + "】");

        Integer newBrand = Integer.valueOf(value);
        for (BrandUpdateDTO brandUpdateDTO : brandUpdateList) {
            sparkJobApiService.updateStoreBrand(enterpriseId, brandUpdateDTO.storeInfoId, brandUpdateDTO.oldStoreBrand, newBrand);
        }

        return null;
    }

    class BrandUpdateDTO{
        public Integer storeInfoId;
        public Integer oldStoreBrand;

        public BrandUpdateDTO(Integer storeInfoId, Integer oldStoreBrand) {
            this.storeInfoId = storeInfoId;
            this.oldStoreBrand = oldStoreBrand;
        }

        @Override
        public String toString() {
            return "BrandUpdateDTO{" +
                    "storeInfoId=" + storeInfoId +
                    ", oldStoreBrand=" + oldStoreBrand +
                    '}';
        }
    }

}
