package com.gic.store.utils.custom;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.common.utils.ReflectUtils;
import org.apache.dubbo.validation.Validator;
import org.apache.dubbo.validation.support.jvalidation.JValidator;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.ValidationException;
import java.util.Set;

/**
 * @author guojx
 * @date 2019/7/10 11:37 AM
 */
public class CustomValidator extends JValidator implements Validator{
    private final Class<?> clazz;

    public CustomValidator(URL url) {
        super(url);
        this.clazz = ReflectUtils.forName(url.getServiceInterface());
    }

    @Override
    public void validate(String methodName, Class<?>[] parameterTypes, Object[] arguments) throws Exception {

        try{
            super.validate(methodName,parameterTypes,arguments);
        }catch (ConstraintViolationException e){
            //防止客户端进行验证报错
            //throw new ValidationException(constraintMessage(clazz.getName(),methodName,e.getConstraintViolations()));
        }
    }

    private String constraintMessage(String className,String methodName,Set<ConstraintViolation<?>> violations){
        JSONObject json = new JSONObject();
        json.put("service",className);
        json.put("method",methodName);

        JSONArray details = new JSONArray();
        for(ConstraintViolation violation : violations){
            JSONObject detail = new JSONObject();
            detail.put("bean",violation.getRootBean().getClass().getName());
            detail.put("property",violation.getPropertyPath().toString());
            detail.put("message",violation.getMessage());
            details.add(detail);
        }
        json.put("details",details);
        return json.toString();
    }
}
