package com.gic.store.utils.resource;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.dto.StoreResourceDTO;
import com.gic.store.dto.StoreWidgetDTO;

import java.util.*;

/**
 * 查询超管的权限
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/5/26 11:31 AM 
 */
public class AuthDataHandleUtils {
    private static Map<String, AbstractAuthDataUtils> map = new HashMap<>(16);
    public final static String STORE_FIELD_KEY = "storeFieldKey";

    static {
        //门店分组不用过滤
        map.put(StoreESFieldsEnum.STOREBRANDIDLIST.getField(), new BrandAbstractAuthData());
    }

    /**
     * 查询超管数据
     * @param enterpriseId
     * @param esField
     * @return
     */
    public static List<StoreResourceDTO> getAuthData(Integer enterpriseId, String esField) {
        return map.get(esField).getAuthData(enterpriseId);
    }

    /**
     * 子管理员权限数据
     * @param storeWidget 门店选择器记录
     * @param esField
     * @return
     */
    public static List<StoreResourceDTO> getUserData(StoreWidgetDTO storeWidget, String esField) {
        Set<String> authIdSet = getAuthData(storeWidget, esField);
        return map.get(esField).auth(authIdSet);
    }

    protected static Set<String> getAuthData(StoreWidgetDTO storeWidget, String field) {
        Set<String> authRegionSet = new HashSet<>();

        String searchParam = storeWidget.getSearchParam();
        JSONArray jsonArr = JSON.parseArray(searchParam);
        if (jsonArr.isEmpty()) {
            return authRegionSet;
        }
        JSONObject json = jsonArr.getJSONObject(0);
        JSONArray list = json.getJSONArray("list");

        Set<String> storeFieldKey = StoreESFieldsEnum.getStoreFieldKey();

        //是否是自定义字段类型的数据
        //因为这两者解析不一致
        boolean isStoreField = isStoreField(field);

        for (int i = 0, len = list.size(); i < len; i ++) {
            JSONObject o = list.getJSONObject(i).getJSONObject("data");
            String key = o.getString("key");

            o.put("value", o.getString("value").replaceAll(",", " "));

            if (isStoreField) {
                //如果是自定义字段，应该存key，而不是value
                if (storeFieldKey.contains(key)) {
                    authRegionSet.add(key);
                }
                continue;
            }
            if (key.equals(field)) {
                String[] arr = o.getString("value").split(" ");
                for(String s : arr){
                    authRegionSet.add(s);
                }
            }
        }
        return authRegionSet;
    }

    private static boolean isStoreField(String field) {
        if (AuthDataHandleUtils.STORE_FIELD_KEY.equals(field)) {
            return true;
        }
        return false;
    }
}
