package com.gic.store.utils.resource;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreBrandDTO;
import com.gic.store.dto.StoreResourceDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreBrandApiService;
import com.gic.store.utils.ApplicationContextUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 品牌数据
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/5/26 11:18 AM 
 */
public class BrandAbstractAuthData extends AbstractAuthDataUtils {

    @Override
    public List<StoreResourceDTO> getAuthData(Integer enterpriseId) {
        StoreBrandApiService storeBrandApiService = ApplicationContextUtils.getBean("storeBrandApiService");
        ServiceResponse<List<StoreBrandDTO>> response = storeBrandApiService.listAllStoreBrand(enterpriseId, null);
        return getData(response);
    }

    @Override
    public List<StoreResourceDTO> auth(Set<String> authIdSet) {
        StoreBrandApiService storeBrandApiService = ApplicationContextUtils.getBean("storeBrandApiService");
        ServiceResponse<List<StoreBrandDTO>> response = storeBrandApiService.listAllStoreBrandInfoByIds(authIdSet.stream()
                .filter(e -> StringUtils.isNotBlank(e))
                .mapToInt(e -> Integer.valueOf(e)).boxed()
                .collect(Collectors.toList()));
        return getData(response);
    }

    private List<StoreResourceDTO> getData(ServiceResponse<List<StoreBrandDTO>> response) {
        if (response.isSuccess()) {
            List<StoreBrandDTO> list = response.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                List<StoreResourceDTO> result = list.stream().map(e -> new StoreResourceDTO().setId(e.getStoreBrandId().toString())
                        .setName(e.getStoreBrandName()).setOwnType(e.getType()).setEnterpriseId(e.getEnterpriseId())).collect(Collectors.toList());
                return result;
            }
        }
        return Collections.EMPTY_LIST;
    }
}
