package com.gic.store.utils.widget;

import java.util.List;
import java.util.stream.Collectors;

import com.gic.auth.dto.UserDTO;
import com.gic.auth.dto.UserResourceDTO;
import com.gic.auth.service.UserApiService;
import com.gic.auth.service.UserResourceApiService;
import com.gic.store.service.StoreWidgetService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.dto.StoreWidgetLogDTO;
import com.gic.store.dto.StoreWidgetLogSaveDTO;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.store.service.StoreWidgetLogApiService;

/**
 * 门店选择器日志工具
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/29 3:43 PM 
 */
@Component
public class StoreWidgetLogUtils {
    private static Logger logger = LogManager.getLogger(StoreWidgetLogUtils.class);
    @Autowired
    private StoreWidgetLogApiService storeWidgetLogApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private StoreWidgetService storeWidgetService;

    /**
     * 写门店选择器日志
     * @param storeWidgetDTO
     * @param newStoreWidgetId
     */
    public void saveStoreWidgetLog(StoreWidgetDTO storeWidgetDTO, Integer newStoreWidgetId) {
        Integer storeWidgetId = storeWidgetDTO.getStoreWidgetId();
        //如果是编辑，则会有值，用于判断创建还是编辑
        StoreWidgetDTO oldStoreWidget = getOldStoreWidget(storeWidgetId);
        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO;

        if (oldStoreWidget == null) {
            logSaveDTO = buildStoreWidgetLogWhenCreate(storeWidgetDTO, newStoreWidgetId);
        } else {
            //copy旧门店选择器ID上挂载的日志到新的ID上面
            copyStoreWidgetLog(storeWidgetId, newStoreWidgetId);
            //新门店选择器ID上新增新的日志
            logSaveDTO = buildStoreWidgetLogWhenUpdate(oldStoreWidget, storeWidgetDTO, newStoreWidgetId);
        }
        //写日志
        storeWidgetLogApiService.saveStoreWidgetLog(logSaveDTO);
    }

    /**
     * 因为门店选择器机制总是新增，但是权限范围数据前端不传，所以需要后端自己从老记录里拿
     * @param storeWidgetDTO
     * @return
     */
    public void setAuthSearchParamDataFromOld(StoreWidgetDTO storeWidgetDTO) {
        StoreWidgetDTO oldStoreWidget = getOldStoreWidget(storeWidgetDTO.getStoreWidgetId());
        if (oldStoreWidget != null) {
            storeWidgetDTO.setAuthSearchParam(oldStoreWidget.getAuthSearchParam());
            storeWidgetDTO.setAuthScreenSimpleData(oldStoreWidget.getAuthScreenSimpleData());
            storeWidgetDTO.setAuthScreenDetailData(oldStoreWidget.getAuthScreenDetailData());
        }
    }

    /**
     * 只更新权限范围数据的日志
     * @param storeWidgetId
     * @param userId
     */
    public void saveStoreWidgetWhenEditAuth(Integer storeWidgetId, Integer userId) {
        StoreWidgetDTO oldStoreWidget = getOldStoreWidget(storeWidgetId);
        if (oldStoreWidget != null) {
            //操作日志对象组装
            StoreWidgetDTO storeWidgetDTO = new StoreWidgetDTO();
            setLoginUserResource(userId, storeWidgetDTO);
            storeWidgetDTO.setUserId(userId);
            storeWidgetDTO.setAuthChangeType(true);
            StoreWidgetLogSaveDTO logSaveDTO = buildStoreWidgetLogWhenUpdate(oldStoreWidget, storeWidgetDTO, storeWidgetId);
            //写日志
            storeWidgetLogApiService.saveStoreWidgetLog(logSaveDTO);
        }
    }

    /**
     * 设置登录管理员的门店资源权限（更改当前权限范围操作/管理员业务创建的时候）
     * @param userId
     * @param storeWidgetDTO
     */
    public void setLoginUserResource(Integer userId, StoreWidgetDTO storeWidgetDTO) {
        logger.info("设置登录管理员的门店资源权限参数：{}", userId);
        //子管理员创建/更新权限范围数据，写入管理员管辖权限数据
        if (userId != null) {
            ServiceResponse<UserDTO> userRes = userApiService.getUserInfoById(userId);
            if (userRes.isSuccess()) {
                UserDTO  user = userRes.getResult();
                if (user.getSuperAdmin() != 1) {
                    //子管理员
                    ServiceResponse<UserResourceDTO> resourceRes = userResourceApiService.getResourceByUserId(userId);
                    if (resourceRes.isSuccess()) {
                        UserResourceDTO userResource = resourceRes.getResult();
                        if (userResource != null) {
                            Integer storeWidgetId = userResource.getResourceId();
                            StoreWidgetDTO storeWidgetAuth = storeWidgetService.getStoreWidget(storeWidgetId);
                            if (storeWidgetAuth != null) {
                                storeWidgetDTO.setAuthSearchParam(storeWidgetAuth.getSearchParam());
                                storeWidgetDTO.setAuthScreenSimpleData(storeWidgetAuth.getScreenSimpleData());
                                storeWidgetDTO.setAuthScreenDetailData(storeWidgetAuth.getScreenDetailData());
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * copy旧门店选择器ID上挂载的日志到新的ID上面
     * @param oldStoreWidgetId
     * @param newStoreWidgetId
     */
    private void copyStoreWidgetLog(Integer oldStoreWidgetId, Integer newStoreWidgetId) {
        //查询旧uuid上面挂载的日志数据
        ServiceResponse<List<StoreWidgetLogDTO>> response = storeWidgetLogApiService.listStoreWidgetLog(oldStoreWidgetId);
        if (response.isSuccess()) {
            List<StoreWidgetLogDTO> list = response.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                //复制数据到新uuid上面
                List<StoreWidgetLogDTO> copyList = list.stream().map(e ->
                        e.setStoreWidgetId(newStoreWidgetId)
                                .setStoreWidgetLogId(null)).collect(Collectors.toList());
                //
                storeWidgetLogApiService.tmpInsert(copyList);
            }
        }
    }


    private StoreWidgetDTO getOldStoreWidget(Integer storeWidgetId) {
        if (storeWidgetId == null) {
            return null;
        }
        ServiceResponse<StoreWidgetDTO> oldStoreWidgetResponse = storeWidgetApiService.getStoreWidget(storeWidgetId);
        if (!oldStoreWidgetResponse.isSuccess()) {
            logger.info("根据门店选择器ID:{}，查询不到记录", storeWidgetId);
            return null;
        }
        return oldStoreWidgetResponse.getResult();
    }

    /**
     * 组装日志对象-编辑场景
     * @param oldStoreWidget 老数据
     * @param newStoreWidgetDTO 前端传入的参数
     * @param newStoreWidgetId 新增门店选择器产生的ID
     * @return
     */
    private static StoreWidgetLogSaveDTO buildStoreWidgetLogWhenUpdate(StoreWidgetDTO oldStoreWidget, StoreWidgetDTO newStoreWidgetDTO,
                                                                       Integer newStoreWidgetId) {
        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO = buildCommon(newStoreWidgetId, newStoreWidgetDTO.getUserId());

        //编辑
        //编辑类型
        logSaveDTO.setOperationType(2);
        //是否是权限变更范围类型
        if (newStoreWidgetDTO.isAuthChangeType()) {
            //编辑筛选条件类型
            logSaveDTO.setLogType(2);

            logSaveDTO.setSearchParamBefore(oldStoreWidget.getSearchParam());
            logSaveDTO.setSearchParamTextBefore(oldStoreWidget.getScreenDetailData());
            logSaveDTO.setSearchParamSimpleBefore(oldStoreWidget.getScreenSimpleData());

            logSaveDTO.setSearchParamAfter(newStoreWidgetDTO.getSearchParam());
            logSaveDTO.setSearchParamSimpleAfter(newStoreWidgetDTO.getScreenSimpleData());
            logSaveDTO.setSearchParamTextAfter(newStoreWidgetDTO.getScreenDetailData());
        } else {
            //权限变更范围类型
            logSaveDTO.setLogType(1);
            //记录变更前的权限范围数据
            logSaveDTO.setSearchParamBefore(oldStoreWidget.getAuthSearchParam());
            logSaveDTO.setSearchParamTextBefore(oldStoreWidget.getAuthScreenDetailData());
            logSaveDTO.setSearchParamSimpleBefore(oldStoreWidget.getAuthScreenSimpleData());

            logSaveDTO.setSearchParamAfter(newStoreWidgetDTO.getAuthSearchParam());
            logSaveDTO.setSearchParamSimpleAfter(newStoreWidgetDTO.getAuthScreenSimpleData());
            logSaveDTO.setSearchParamTextAfter(newStoreWidgetDTO.getAuthScreenDetailData());

        }
        return logSaveDTO;
    }

    /**
     * 组装日志对象-创建场景
     * @param storeWidgetDTO 前端传入的参数
     * @param newStoreWidgetId 新增门店选择器产生的ID
     * @return
     */
    private static StoreWidgetLogSaveDTO buildStoreWidgetLogWhenCreate(StoreWidgetDTO storeWidgetDTO, Integer newStoreWidgetId) {
        String authSearchParam = storeWidgetDTO.getAuthSearchParam();
        String searchParam = storeWidgetDTO.getSearchParam();

        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO = buildCommon(newStoreWidgetId, storeWidgetDTO.getUserId());

        //创建
        logSaveDTO.setOperationType(1);
        if (storeWidgetDTO.isAuthChangeType()) {
            //权限变更范围类型
            logSaveDTO.setLogType(1);
            logSaveDTO.setSearchParamAfter(authSearchParam);
            logSaveDTO.setSearchParamTextAfter(storeWidgetDTO.getAuthScreenDetailData());
            logSaveDTO.setSearchParamSimpleAfter(storeWidgetDTO.getAuthScreenSimpleData());
        } else {
            //编辑筛选条件类型
            logSaveDTO.setLogType(2);
            logSaveDTO.setSearchParamAfter(searchParam);
            logSaveDTO.setSearchParamTextAfter(storeWidgetDTO.getScreenDetailData());
            logSaveDTO.setSearchParamSimpleAfter(storeWidgetDTO.getScreenSimpleData());
        }
        return logSaveDTO;
    }

    private static StoreWidgetLogSaveDTO buildCommon(Integer newStoreWidgetId, Integer userId) {
        StoreWidgetLogSaveDTO logSaveDTO = new StoreWidgetLogSaveDTO();
        logSaveDTO.setOperationId(userId);
        logSaveDTO.setStoreWidgetId(newStoreWidgetId);
        return logSaveDTO;
    }
}
