package com.gic.store.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.constant.Constants;
import com.gic.store.constant.GenderEnum;
import com.gic.store.constant.StoreImportEnum;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.*;
import com.gic.store.service.*;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.utils.ExcelUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zhiwj
 * @date 2019/7/17
 */
@RestController
public class ClerkImportController {

    private static final Logger logger = LogManager.getLogger(ClerkImportController.class);

    @Autowired
    private ClerkImportApiService clerkImportApiService;

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private DictApiService dictApiService;
    @Autowired
    private ClerkPositionApiService clerkPositionApiService;
    @Autowired
    private StoreRegionApiService storeRegionApiService;

    /**
     * 下载模板
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @RequestMapping("/clerk-import-template-download")
    @HeaderSignIgnore
    public Object download(HttpServletRequest request, HttpServletResponse response) throws Exception {
        String fileName = " 导购资料导入模板.xlsx";

        OutputStream fo = null;
        InputStream in = null;
        try {
            response.setContentType("text/html;charset=gbk");
            response.setHeader("Content-Disposition",
                    "attachment; filename=" + new String(fileName.getBytes("gbk"), "ISO-8859-1"));

            fo = response.getOutputStream();
            in = this.getClass().getClassLoader().getResourceAsStream("excel/clerk_batch_import.xlsx");
            byte[] b = new byte[1024];
            int len = 0;
            while ((len = in.read(b)) != -1) {
                fo.write(b, 0, len);
            }
            fo.flush();
        } catch (Exception e) {
            logger.info("下载错误", e);
        } finally {
            if (fo != null) {
                fo.close();
            }
            if (in != null) {
                in.close();
            }
        }
        return RestResponse.success();
    }

    /**
     * 批量导入
     * @param request
     * @param file
     * @param pageQO
     * @return
     * @throws IOException
     */
    @RequestMapping("/clerk-tmp-file-upload")
    @HeaderSignIgnore
    public RestResponse fileUpload(HttpServletRequest request, MultipartFile file, PageQO pageQO) throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (file == null || file.isEmpty()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "文件不存在");
        }
        // 判断是否有正在导入的任务
        ServiceResponse<Boolean> importingStoreResponse = clerkImportApiService.hasImportingClerk(enterpriseId);
        if (importingStoreResponse.getResult()) {
            return RestResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "当前企业下有正在导入导购的任务，请稍后重试！");
        }
        if (file.getSize() > 0) {
            String fileName = file.getOriginalFilename();
            String uuId = request.getParameter("uuId");
            String repeatStoreCodeConfig = request.getParameter("repeatClerkCodeConfig");
            if (StringUtils.isBlank(repeatStoreCodeConfig)) {
                repeatStoreCodeConfig = "0";
            }
            //如果门店code重复，是否替换更新
            boolean isReplace = "1".equals(repeatStoreCodeConfig);
            if (fileName.endsWith(Constants.XLSX) || fileName.endsWith(Constants.XLS)) {
                try {
                    Workbook workbook = ExcelUtils.getWorkbookFromUpload(file.getInputStream(), fileName);
                    List<Map<Integer, String>> data = ExcelUtils.readExcel(6, fileName.substring(fileName.lastIndexOf(".") + 1), workbook, null);
                    if (CollectionUtils.isNotEmpty(data) && data.get(0).size() != Constants.CLERK_IMPORT_EXCEL_SIZE) {
                        return RestResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "总列数与模板不一致");
                    }
                    clerkImportApiService.delClerkTmp(enterpriseId);
                    int logCount = 0;
                    StringBuilder sb = new StringBuilder();
                    for (Map<Integer, String> row : data) {
                        ClerkTmpDTO bean = this.validateData(row, enterpriseId, isReplace);
                        try {
                            bean.setSignKey(uuId);
                            // 保存数据到临时表
                            bean.setOperatorId(UserDetailUtils.getUserDetail().getUserId());
                            bean.setOperatorName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
                            clerkImportApiService.save(bean);
                        } catch (Exception e) {
                            logger.warn(e);
                        }
                        if (logCount <= 3) {
                            sb.append(bean.getStoreName()).append(",");
                        }
                        logCount++;
                    }

                    if (sb.length() > 0 && logCount > 0) {
                        sb.deleteCharAt(sb.length() - 1);
                        sb.append("等").append(logCount).append("名导购");
                    }
                    LogUtils.createLog("批量导入", sb.toString());

                    if (clerkImportApiService.hasImportingClerk(enterpriseId).getResult()) {
                        // userId
                        Integer userId = UserDetailUtils.getUserDetail().getUserInfo().getUserId();
                        // 开队列
                        String r = clerkImportApiService.importDataToClerk(enterpriseId, userId.toString(), uuId, isReplace).getResult();
                        if (StringUtils.isNotBlank(r)) {
                            logger.warn("队列错误, {}", r);
                            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "网络错误");
                        }
                    } else {
                        Page page = clerkImportApiService.listClerkTmp(enterpriseId, false, pageQO.getCurrentPage(), pageQO.getPageSize()).getResult();
                        Map<Object, Object> result = new HashMap<>(8);
                        result.put("errorData", page);
                        int errorRows = page.getTotalCount();
                        result.put("total", errorRows);
                        result.put("success", 0);
                        result.put("error", errorRows);
                        return RestResponse.success(result);
                    }
                } catch (Exception e) {
                    logger.warn("excel验证不通过", e);
                    return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "excel验证不通过");
                }
            } else {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "选择xlsx格式文件上传");
            }
        } else {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "选择xlsx格式文件上传");
        }
        LogUtils.createLog("导入导购", "导购");
        return RestResponse.success();
    }

    /**
     * 查看进度
     * @param request
     * @param page
     * @return
     */
    @RequestMapping("/get_clerk_batch_process")
    public Object getStoreBatchProcess(HttpServletRequest request, PageQO page) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        String uuId = request.getParameter("uuId");
        Map<String, Object> data = clerkImportApiService.getClerkBatchProcess(enterpriseId, uuId, page.getCurrentPage(), page.getPageSize()).getResult();
        return RestResponse.success(data);
    }

    /**
     * 错误日志查看
     * @param page
     * @return
     */
    @RequestMapping("/clerk-import-load-error-data")
    public Object loadErrorData(PageQO page) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Map<String, Object> result = new HashMap<String, Object>();
        List<ClerkTmpDTO> successList = clerkImportApiService.listClerkTmp(enterpriseId, true).getResult();
        ServiceResponse<Page> serviceResponse = clerkImportApiService.pageError(enterpriseId, page.getCurrentPage(), page.getPageSize());
        List<ClerkTmpDTO> totalList = clerkImportApiService.listAllClerkTmp(enterpriseId).getResult();
        Page pageResponse = serviceResponse.getResult();
        result.put("errorData", pageResponse);
        result.put("total", totalList == null ? 0 : totalList.size());
        result.put("success", successList == null ? 0 : successList.size());
        result.put("error", pageResponse.getResult().size());
        return RestResponse.success(result);
    }

    /**
     * 导出错误日志
     * @param request
     * @param response
     * @return
     * @throws IOException
     */
    @RequestMapping("/export-clerk-error-data")
    public RestResponse exportErrorData(HttpServletRequest request, HttpServletResponse response) throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        String fileName = "导购导入错误记录.xlsx";
        List<ClerkTmpDTO> listErrorData = clerkImportApiService.listClerkTmp(enterpriseId, false).getResult();
        if (!listErrorData.isEmpty()) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmm");
            fileName = sdf.format(listErrorData.get(0).getUpdateTime()) + fileName;
            List<List<String>> errorList = clerkImportApiService.listError(enterpriseId).getResult();
            List<String> titleList = new ArrayList<>();
            Collections.addAll(titleList, "序号", "导购名", "导购代码", "门店名", "门店代码", "职位", "性别", "区号", "手机号", "是否店长", "错误信息");
            List<String> descriptionList = new ArrayList<>();
            Collections.addAll(descriptionList, "", "必填，不能超过20个字", "必填，不能超过20个字，为导购的唯一识别码", "必填，不能超过20个字", "必填,不能超过20个字", "非必填，不能超过30个字", "\"男\"或\"女\"", "区号，如不填默认为 +86", "必填，不能超过20个字", "必填， 是或否", "");

            Workbook workbook = ExcelUtils.getWorkbook("导购导入错误记录", titleList, descriptionList, errorList, null);
            byte[] bytesName = fileName.getBytes("UTF-8");
            fileName = new String(bytesName, "ISO-8859-1");
            response.setHeader("content-disposition", "attachment;fileName=" + fileName);

            OutputStream out = response.getOutputStream();
            workbook.write(out);
            out.close();
            return RestResponse.success();
        } else {
            return RestResponse.failure(ErrorCode.NOTEXISTS.getCode(), "没有错误日志");
        }
    }


    private ClerkTmpDTO validateData(Map<Integer, String> row, Integer enterpriseId, boolean isReplace) {
        boolean validate = true;
        ClerkTmpDTO bean = new ClerkTmpDTO();
        bean.setEnterpriseId(enterpriseId);
        bean.setErrorMessage(StoreImportEnum.WAIT.getCode());
        // 验证导购名
        String clerkName = row.get(1);
        validate = validClerkName(validate, bean, clerkName);
        // 验证导购code
        String clerkCode = row.get(2);
        if (validate) {
            if (StringUtils.isBlank(clerkCode)) {
                bean.setErrorMessage("导购代码不能为空");
                validate = false;
            }
        }
        clerkCode = clerkCode.trim();
        validate = validClerkCode(validate, bean, clerkCode);
        // 门店域
        String regionName = row.get(3);
        if (validate) {
            if (StringUtils.isBlank(regionName)) {
                bean.setErrorMessage("门店域不能为空");
                validate = false;
            }
        }
        regionName = regionName.trim();
        bean.setRegionName(regionName);
        if (validate) {
            if (this.lengthValidate(regionName, Constants.CLERK_IMPORT_REGION_LENGTH)) {
                bean.setErrorMessage("门店域超过20个字");
                validate = false;
            }
        }
        StoreRegionDTO regionDTO = storeRegionApiService.getStoreRegionByRegionName(enterpriseId, regionName).getResult();
        if (validate) {
            if (regionDTO == null) {
                bean.setErrorMessage("门店域不存在");
                validate = false;
            }
        }
        // 验证门店name
        String storeName = row.get(4);
        if (validate) {
            if (StringUtils.isBlank(storeName)) {
                bean.setErrorMessage("门店名不能为空");
                validate = false;
            }
        }
        storeName = storeName.trim();
        bean.setStoreName(storeName);
        validate = validStoreName(validate, bean, storeName);
        // 验证门店code
        String storeCode = row.get(5);
        if (validate) {
            if (StringUtils.isBlank(storeCode)) {
                bean.setErrorMessage("门店代码不能为空");
                validate = false;
            }
        }
        storeCode = storeCode.trim();
        validate = validStoreCode(validate, bean, storeCode);
        // 验证门店是否存在
        validate = validateStoreExist(enterpriseId, isReplace, validate, bean, clerkCode, regionDTO, storeName, storeCode);
        // 验证岗位
        validate = validPositionName(row, enterpriseId, validate, bean, storeCode);
        // 验证性别
        validate = validGender(row, validate, bean);
        // 验证区号
        validate = validNationcode(row, validate, bean);
        // 验证手机号
        validPhone(row, validate, bean);
        return bean;
    }

    private void validPhone(Map<Integer, String> row, boolean validate, ClerkTmpDTO bean) {
        String phoneNumber = row.get(9);
        bean.setPhoneNumber(phoneNumber);
        if (validate) {
            if (StringUtils.isBlank(phoneNumber)) {
                bean.setErrorMessage("手机号不能为空");
            }
        }
        bean.setPhoneNumber(phoneNumber.trim());
    }

    private boolean validClerkCode(boolean validate, ClerkTmpDTO bean, String clerkCode) {
        bean.setClerkCode(clerkCode);
        if (validate) {
            if (this.lengthValidate(clerkCode, Constants.CLERK_IMPORT_CODE_LENGTH)) {
                bean.setErrorMessage("导购代码超过20个字");
                validate = false;
            }
        }
        return validate;
    }

    private boolean validClerkName(boolean validate, ClerkTmpDTO bean, String clerkName) {
        if (StringUtils.isBlank(clerkName)) {
            bean.setErrorMessage("导购名称不能为空");
            validate = false;
        }
        clerkName = clerkName.trim();
        bean.setClerkName(clerkName);
        if (validate) {
            if (this.lengthValidate(clerkName, Constants.CLERK_IMPORT_NAME_LENGTH)) {
                bean.setErrorMessage("导购名称长度超过20个字");
                validate = false;
            }
        }
        return validate;
    }

    private boolean validStoreName(boolean validate, ClerkTmpDTO bean, String storeName) {
        if (validate) {
            if (this.lengthValidate(storeName, Constants.CLERK_IMPORT_STORE_NAME_LENGTH)) {
                bean.setErrorMessage("门店名超过20个字");
                validate = false;
            }
        }
        return validate;
    }

    private boolean validStoreCode(boolean validate, ClerkTmpDTO bean, String storeCode) {
        bean.setStoreCode(storeCode);
        if (validate) {
            if (this.lengthValidate(storeCode, Constants.CLERK_IMPORT_STORE_CODE_LENGTH)) {
                bean.setErrorMessage("门店代码超过20个字");
                validate = false;
            }
        }
        return validate;
    }

    private boolean validateStoreExist(Integer enterpriseId, boolean isReplace, boolean validate, ClerkTmpDTO bean, String clerkCode, StoreRegionDTO regionDTO, String storeName, String storeCode) {
        if (validate) {
            StoreDTO storeDTO = storeApiService.getByStoreNameAndStoreCode(enterpriseId, regionDTO.getRegionId(), storeName, storeCode).getResult();
            if (storeDTO == null) {
                bean.setErrorMessage("门店不存在");
                validate = false;
            } else if (StoreOwnTypeEnum.OWNER.getCode() != storeDTO.getOwnType()) {
                bean.setErrorMessage("不能在被授权门店下导入导购");
                validate = false;
            }else if (!isReplace) {
                // 验证code唯一
                if (clerkImportApiService.validateClerkCode(enterpriseId, storeDTO.getStoreInfoId(), clerkCode).getResult()) {
                    bean.setErrorMessage("导购编码重复");
                    validate = false;
                }
            }
        }
        return validate;
    }

    private boolean validPositionName(Map<Integer, String> row, Integer enterpriseId, boolean validate, ClerkTmpDTO bean, String storeCode) {
        String positionName = row.get(6);
        if (validate) {
            if (StringUtils.isNotBlank(positionName) && this.lengthValidate(storeCode, Constants.CLERK_IMPORT_POSITION_NAME_LENGTH)) {
                bean.setErrorMessage("角色超过20个字");
                validate = false;
            }
        }
        positionName = positionName.trim();
        ServiceResponse<ClerkPositionDTO> positionResp = clerkPositionApiService.getByName(enterpriseId, positionName);
        if (validate) {
            ClerkPositionDTO positionDTO = positionResp.getResult();
            if (positionDTO == null) {
                bean.setErrorMessage("角色不存在");
                validate = false;
            }
        }
        bean.setPositionName(positionName);
        return validate;
    }

    private boolean validGender(Map<Integer, String> row, boolean validate, ClerkTmpDTO bean) {
        String gender = row.get(7);
        bean.setClerkGender(gender);
        if (validate) {
            if (StringUtils.isBlank(gender)) {
                bean.setErrorMessage("性别不能为空");
                validate = false;
            }
        }
        if (validate) {
            GenderEnum genderEnum = GenderEnum.parseMessage(gender);
            if (genderEnum == null) {
                bean.setErrorMessage("不存在该性别: " + gender);
            }
        }
        return validate;
    }

    private boolean validNationcode(Map<Integer, String> row, boolean validate, ClerkTmpDTO bean) {
        String nationcode = row.get(8);
        if (StringUtils.isBlank(nationcode)) {
            nationcode = "+86";
        }
        nationcode = nationcode.trim();
        if (Constants.PLUS_CHAR != nationcode.charAt(0)) {
            nationcode = Constants.PLUS + nationcode;
        }
        bean.setNationcode(nationcode);
        if (validate) {
            if (this.lengthValidate(nationcode, Constants.CLERK_IMPORT_NATION_CODE_LENGTH)) {
                bean.setErrorMessage("区号超过10个字");
                validate = false;
            }
        }
        GicDictDTO dict = this.dictApiService.getDictByCode(Constants.NATION_CODE_DICT, nationcode).getResult();
        if (dict == null) {
            bean.setErrorMessage("区号错误");
            validate = false;
        }
        return validate;
    }

    private boolean lengthValidate(String value, int len) {
        String valueTmp = value.replaceAll("[^\\x00-\\xff]", "**");
        return valueTmp.length() / 2.0 > (double) len;
    }
}
