package com.gic.store.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.constant.Constants;
import com.gic.store.dto.ClerkPositionDTO;
import com.gic.store.service.ClerkPositionApiService;
import com.gic.store.web.qo.clerk.ClerkPositionQO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-16 14:29
 */
@RestController
public class ClerkPositionController {

    @Autowired
    private ClerkPositionApiService clerkPositionApiService;

    @RequestMapping("/save-clerk-position")
    public RestResponse saveClerkPosition(@Validated(ClerkPositionQO.SaveValidView.class) ClerkPositionQO clerkPositionQO) {
        clerkPositionQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Void> serviceResponse = clerkPositionApiService.save(EntityUtil.changeEntityByJSON(ClerkPositionDTO.class, clerkPositionQO));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-clerk-position")
    public RestResponse updateClerkPosition(@Validated(ClerkPositionQO.EditValidView.class) ClerkPositionQO clerkPositionQO) {
        clerkPositionQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Void> serviceResponse = clerkPositionApiService.update(EntityUtil.changeEntityByJSON(ClerkPositionDTO.class, clerkPositionQO));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-by-clerk-position-id")
    public RestResponse getByClerkPositionId(Integer clerkPositionId) {
        ServiceResponse<ClerkPositionDTO> serviceResponse = clerkPositionApiService.getByClerkPositionId(clerkPositionId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/delete-by-clerk-position-id")
    public RestResponse deleteByClerkPositionId(Integer clerkPositionId) {
        ServiceResponse<Void> serviceResponse = clerkPositionApiService.delete(clerkPositionId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-clerk-position")
    public RestResponse listClerkPosition(String search, Integer currentPage, Integer pageSize) {
        ServiceResponse<Page<ClerkPositionDTO>> serviceResponse = clerkPositionApiService.listClerkPosition(UserDetailUtils.getUserDetail().getEnterpriseId(), search, currentPage, pageSize);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-all-clerk-position")
    public RestResponse listAllClerkPosition(String search) {
        ServiceResponse<List<ClerkPositionDTO>> serviceResponse = clerkPositionApiService.listAllClerkPosition(UserDetailUtils.getUserDetail().getEnterpriseId(), search);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/set-clerk-position-sort")
    public RestResponse setClerkPositionSort(Integer clerkPositionId, Integer seq) {
        if (seq < Constants.CLERK_POSITION_TOP) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "系统默认角色无法调整排序");
        }
        ServiceResponse<Void> serviceResponse = clerkPositionApiService.setSort(UserDetailUtils.getUserDetail().getEnterpriseId(), clerkPositionId, seq);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

}
