package com.gic.store.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.dto.PlatformBrandDTO;
import com.gic.enterprise.service.PlatformBrandApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.dto.StoreBrandDTO;
import com.gic.store.service.StoreBrandApiService;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.qo.storebrand.StoreBrandQO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @date 2019/6/27
 */
@RestController
public class StoreBrandController {

    @Autowired
    private StoreBrandApiService storeBrandApiService;
    @Autowired
    private PlatformBrandApiService platformBrandApiService;

    @RequestMapping("/list-platform-brand")
    public RestResponse listPlatformBrand(String branName){
        ServiceResponse<List<PlatformBrandDTO>> serviceResponse = platformBrandApiService.listPlatformBrand(branName);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }


    @RequestMapping("/save-store-brand")
    public RestResponse saveStoreBrand(@Validated(StoreBrandQO.SaveValidView.class) StoreBrandQO storeBrandQO) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        StoreBrandDTO storeBrandDTO = EntityUtil.changeEntity(StoreBrandDTO.class, storeBrandQO);
        storeBrandDTO.setEnterpriseId(enterpriseId);
        ServiceResponse<Integer> serviceResponse = storeBrandApiService.saveOrUpdateStoreBrand(storeBrandDTO);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog((storeBrandQO.getStoreBrandId() == null ? "新增" : "修改") + "门店品牌", storeBrandQO.getStoreBrandName());
            return RestResponse.success();
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/page-store-brand")
    public RestResponse pageStoreBrand(String search, PageQO pageQO) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Page<StoreBrandDTO>> serviceResponse = storeBrandApiService.listStoreBrand(enterpriseId, search, pageQO.getCurrentPage(), pageQO.getPageSize());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-store-brand")
    public RestResponse listStoreBrand(String search) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreBrandDTO>> serviceResponse = storeBrandApiService.listAllStoreBrand(enterpriseId, search);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/delete-store-brand-batch")
    public RestResponse deleteByIds(String ids) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse serviceResponse = storeBrandApiService.deleteByIds(enterpriseId, ids);
        if (serviceResponse.isSuccess()) {
            if (StringUtils.isNotBlank(ids)) {
                ServiceResponse<List<StoreBrandDTO>> response = storeBrandApiService.listStoreBrandByIds(Stream.of(ids.split(GlobalInfo.FLAG_COMMA)).map(Integer::valueOf).collect(Collectors.toList()));
                String logName = Optional.ofNullable(response.getResult())
                        .orElse(Collections.emptyList())
                        .stream()
                        .map(StoreBrandDTO::getStoreBrandName)
                        .collect(Collectors.joining(","));
                LogUtils.createLog("删除门店品牌", logName);
            }
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage(), serviceResponse.getResult());
        }
    }
    @RequestMapping("/delete-all-store-brand-batch")
    public RestResponse delete() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse serviceResponse = storeBrandApiService.deleteAll(enterpriseId);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog("删除所有", "门店品牌");
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage(), serviceResponse.getResult());
        }
    }

    @RequestMapping("/move-store-brand")
    public RestResponse moveStoreBrand(@Validated(StoreBrandQO.MoveValidView.class) StoreBrandQO storeBrandQO) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse serviceResponse = storeBrandApiService.moveStoreBrand(enterpriseId, storeBrandQO.getStoreBrandId(), storeBrandQO.getMoveType());
        if (serviceResponse.isSuccess()) {
            ServiceResponse<StoreBrandDTO> service = storeBrandApiService.getById(storeBrandQO.getStoreBrandId());
            StoreBrandDTO result = service.getResult();
            LogUtils.createLog("设定序号", result.getStoreBrandName());
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/set-store-brand-sort")
    public RestResponse setStoreBrandSort(@Validated({StoreBrandQO.SetSortValidView.class}) StoreBrandQO storeBrandQO) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse serviceResponse = storeBrandApiService.setStoreBrandSort(enterpriseId, storeBrandQO.getStoreBrandId(), storeBrandQO.getSeq());
        if (serviceResponse.isSuccess()) {
            ServiceResponse<StoreBrandDTO> service = storeBrandApiService.getById(storeBrandQO.getStoreBrandId());
            StoreBrandDTO result = service.getResult();
            LogUtils.createLog("设定序号", result.getStoreBrandName());
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

}
