package com.gic.store.web.controller;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.gic.auth.constant.SignConstants;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.service.EnterpriseApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.constant.StoreFieldDateEnum;
import com.gic.store.constant.StoreFieldTextEnum;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.dto.StoreRegionDTO;
import com.gic.store.service.StoreDictApiService;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreFieldSelectApiService;
import com.gic.store.service.StoreRegionApiService;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.qo.storefield.StoreFieldQO;
import com.gic.store.web.qo.storefield.StoreFieldSelectQO;
import com.gic.store.web.vo.storefield.StoreFieldEditVO;
import com.gic.store.web.vo.storefield.StoreFieldRegionVO;
import com.gic.store.web.vo.storefield.StoreFieldVO;

/**
  * @ClassName: StoreFieldController 
  * @Description: 自定义字段 controller
  * @author guojuxing 
  * @date 2019/7/31 10:06 AM 
  */
@RestController
@RequestMapping("/store-field")
public class StoreFieldController {

    private static final Logger logger = LoggerFactory.getLogger(StoreFieldController.class);

    @Autowired
    private StoreFieldApiService storeFieldApiService;
    @Autowired
    private StoreFieldSelectApiService storeFieldSelectApiService;

    @Autowired
    private StoreRegionApiService storeRegionApiService;
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;

    /**
      * 字段类型下拉数据列表
      * @Title: listStoreFieldType 
      * @Description: 字段类型下拉数据列表
       * @author guojuxing
      * @param  
      * @return  com.gic.commons.webapi.reponse.RestResponse 
       */
    @RequestMapping("/list-store-field-type")
    public RestResponse listStoreFieldType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldTypeEnum typeEnum : StoreFieldTypeEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/list-store-field-date-type")
    public RestResponse listStoreFieldDateType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldDateEnum typeEnum : StoreFieldDateEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    /**
     * 文本类型
     * @return
     */
    @RequestMapping("/list-store-field-text-type")
    public RestResponse listStoreFieldTextType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldTextEnum typeEnum : StoreFieldTextEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/save-store-field")
    public RestResponse save(@Validated({ StoreFieldQO.SaveValidView.class,
            StoreFieldQO.FieldTypeValidView.class }) StoreFieldQO storeFieldQO) {
        String regions = storeFieldQO.getStoreRegionIds();
        String[] regionArr = regions.split(",");
        StringBuilder region = new StringBuilder("_");
        for (String str : regionArr) {
            if (StringUtils.isBlank(str)) {
                continue;
            }
            if (!StringUtils.isNumeric(str)) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域有误");
            }
            region.append(Integer.parseInt(str));
            region.append("_");
        }
        storeFieldQO.setStoreRegionId(region.toString());
        ServiceResponse result = storeFieldApiService.save(transferQoToDTO(storeFieldQO));
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_NEW + "自定义属性字段",
                storeFieldQO.getStoreFieldName());
    }

    @RequestMapping("/edit-store-field")
    public RestResponse edit(@Validated({ StoreFieldQO.SaveValidView.class,
            StoreFieldQO.RemoveValidView.class }) StoreFieldQO storeFieldQO) {
        String regions = storeFieldQO.getStoreRegionIds();
        String[] regionArr = regions.split(",");
        StringBuilder region = new StringBuilder("_");
        for (String str : regionArr) {
            region.append(str);
            region.append("_");
        }
        storeFieldQO.setStoreRegionId(region.toString());
        ServiceResponse result = storeFieldApiService.edit(transferQoToDTO(storeFieldQO));
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_EDIT + "自定义属性字段",
                storeFieldQO.getStoreFieldName());
    }

    @RequestMapping("/get-store-field")
    public RestResponse getStoreField(@Validated({ StoreFieldQO.RemoveValidView.class }) StoreFieldQO storeFieldQO) {
        ServiceResponse<StoreFieldDTO> result = storeFieldApiService.getStoreField(storeFieldQO.getStoreFieldId());
        if (result.isSuccess()) {
            StoreFieldEditVO vo = EntityUtil.changeEntityNew(StoreFieldEditVO.class, result.getResult());
            ServiceResponse<Map<Integer, String>> storeFieldIdMapResult = storeFieldApiService
                    .storeFieldIdMapRegionId(UserDetailUtils.getUserDetail().getEnterpriseId());
            if (storeFieldIdMapResult.isSuccess()) {
                String regionId = storeFieldIdMapResult.getResult().get(vo.getStoreFieldId());
                String tempRegion = regionId.replaceAll("_", ",");
                vo.setStoreRegionIds(tempRegion);
                return RestResponse.success(vo);
            } else {
                return EnterpriseRestResponse.failure(storeFieldIdMapResult);
            }
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/remove-store-field")
    public RestResponse removeStoreField(@Validated({ StoreFieldQO.RemoveValidView.class }) StoreFieldQO storeFieldQO) {
        ServiceResponse result = storeFieldApiService.delete(storeFieldQO.getStoreFieldId());
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_DELETE + "自定义属性字段",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list-store-field-by-region-id")
    public RestResponse listStoreFieldByRegionId(Integer regionId) {
        if (regionId == null) {
            RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "域ID不能为空");
        }
        ServiceResponse<List<StoreFieldDTO>> result = storeFieldApiService.listStoreFieldByRegionId(regionId);
        if (result.isSuccess()) {
            return RestResponse.success(EntityUtil.changeEntityListNew(StoreFieldRegionVO.class, result.getResult()));
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/remove-store-field-batch")
    public RestResponse removeStoreFieldBatch(boolean isDeleteAll, String storeFieldIds) {
        if (isDeleteAll) {
            ServiceResponse result = storeFieldApiService.deleteAll(UserDetailUtils.getUserDetail().getEnterpriseId());
            if (result.isSuccess()) {
                return RestResponse.success(result.getResult());
            } else {
                return RestResponse.failure(result.getCode(), result.getMessage());
            }
        } else {
            if (StringUtils.isBlank(storeFieldIds)) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键值不能为空");
            }
            String[] ids = storeFieldIds.split(",");
            List<Integer> storeFieldIdList = new ArrayList<>();
            for (String id : ids) {
                storeFieldIdList.add(Integer.parseInt(id));
            }
            ServiceResponse result = storeFieldApiService.deleteBatch(storeFieldIdList);
            return OperationResultUtils.operationResult(result, "批量删除自定义属性字段",
                    OperationResultUtils.getOperationObject(result));
        }

    }

    @RequestMapping("/list-store-field")
    public RestResponse listStoreField(String search, PageQO pageQO, String regionIds) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Map<Integer, String>> storeFieldIdMapResult = storeFieldApiService
                .storeFieldIdMapRegionId(enterpriseId);
        //
        Set<String> searchSet = new HashSet<>(Arrays.asList(Optional.ofNullable(regionIds).orElse("-9999").split(SignConstants.COMMA)));
        if (storeFieldIdMapResult.isSuccess()) {
            Map<Integer, String> storeFieldIdMapRegionIds = storeFieldIdMapResult.getResult();
            List<Integer> storeFieldIdList = Optional.ofNullable(storeFieldIdMapRegionIds).orElse(new HashMap<>(2))
                    .entrySet()
                    .stream()
                    .filter(e -> {
                        Set<String> valueSet = new HashSet<>(Arrays.asList(e.getValue().split(SignConstants.UNDERLINE)));
                        return searchSet
                                .stream()
                                .anyMatch(regionId -> "".equals(regionId) || "-9999".equals(regionId) || valueSet.contains(regionId));
                    })
                    .map(e -> e.getKey())
                    .collect(Collectors.toList());


            ServiceResponse<Page<StoreFieldDTO>> result = storeFieldApiService.listStoreField(
                    storeFieldIdList.size() < 1 ? Stream.of(-9999).collect(Collectors.toList()) : storeFieldIdList,
                    UserDetailUtils.getUserDetail().getEnterpriseId(), search, pageQO.getCurrentPage(),
                    pageQO.getPageSize());
            if (result.isSuccess()) {
                ServiceResponse<List<StoreRegionDTO>> serviceResponse = storeRegionApiService
                        .listStoreRegion(enterpriseId, null);
                List<StoreRegionDTO> regionDTOList = serviceResponse.getResult();
                Map<Integer, String> regionMap = new HashMap<>(16);
                for (StoreRegionDTO regionDTO : regionDTOList) {
                    regionMap.put(regionDTO.getRegionId(), regionDTO.getRegionName());
                }

                //转 vo
                Page<StoreFieldDTO> resultPage = result.getResult();
                List<StoreFieldDTO> storeFieldDTOList = resultPage.getResult();
                List<StoreFieldVO> voList = EntityUtil.changeEntityListNew(StoreFieldVO.class, storeFieldDTOList);

                //列表/筛选属性支持 列表数据
                ServiceResponse<List<String>> storeFieldDictValue = storeDictApiService
                        .listStoreFieldDictValue(enterpriseId);
                List<String> storeFieldDictValueList = null;
                if (storeFieldDictValue.isSuccess()) {
                    storeFieldDictValueList = storeFieldDictValue.getResult();
                }

                if (voList != null) {
                    Map<Integer, String> enterpriseIdMapName = getEnterpriseIdMapName(storeFieldDTOList.stream()
                            .filter(e -> e.getOwnType() == 1)
                            .mapToInt(e -> e.getEnterpriseId())
                            .boxed()
                            .collect(Collectors.toList()));

                    for (StoreFieldVO storeFieldVO : voList) {
                        //共享商户名称
                        storeFieldVO.setOwnerEntName(enterpriseIdMapName.get(storeFieldVO.getEnterpriseId()));

                        String regionId = storeFieldIdMapRegionIds.get(storeFieldVO.getStoreFieldId());
                        storeFieldVO.setRegionName(getRegionNameByRegionId(regionId, regionMap));
                        if (storeFieldDictValueList != null
                                && storeFieldDictValueList.contains(storeFieldVO.getStoreFieldId().toString())) {
                            storeFieldVO.setIsFilter(1);
                        }
                    }
                }
                Page<StoreFieldVO> page = EntityUtil.changeEntityNew(Page.class, resultPage);
                page.setResult(voList);
                return RestResponse.success(page);
            } else {
                return RestResponse.failure(result.getCode(), result.getMessage());
            }
        } else {
            return EnterpriseRestResponse.failure(storeFieldIdMapResult);
        }
    }

    @RequestMapping("/swap-store-field")
    public RestResponse swapStoreField(@Validated({ StoreFieldQO.SwapValidView.class,
            StoreFieldQO.RemoveValidView.class }) StoreFieldQO storeFieldQO) {
        int upOrDown = storeFieldQO.getUpOrDown();
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse result;
        switch (upOrDown) {
        case StoreGroupConstant.SORT_UP:
            result = storeFieldApiService.upOrDown(storeFieldQO.getStoreFieldId(), true, enterpriseId);
            break;
        case StoreGroupConstant.SORT_DOWN:
            result = storeFieldApiService.upOrDown(storeFieldQO.getStoreFieldId(), false, enterpriseId);
            break;
        case StoreGroupConstant.SORT_BOTTOM:
            result = storeFieldApiService.sortBottom(storeFieldQO.getStoreFieldId(), enterpriseId);
            break;
        case StoreGroupConstant.SORT_TOPPING:
            result = storeFieldApiService.sortTopping(storeFieldQO.getStoreFieldId(), enterpriseId);
            break;
        default:
            result = null;
        }
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/set-store-field-sort-value")
    public RestResponse setStoreFieldSortValue(@Validated({ StoreFieldQO.SetSortValueValidView.class,
            StoreFieldQO.RemoveValidView.class }) StoreFieldQO storeFieldQO) {
        ServiceResponse result = storeFieldApiService.setStoreFieldSort(storeFieldQO.getStoreFieldId(),
                storeFieldQO.getSortValue(), UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(result, "排序自定义属性字段",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/save-store-field-select")
    public RestResponse saveFieldValue(String storeFieldSelectName, String storeFieldSelectCode, Integer storeFieldId) {
        StoreFieldSelectDTO dto = new StoreFieldSelectDTO();
        dto.setStoreFieldSelectCode(storeFieldSelectCode);
        dto.setStoreFieldSelectName(storeFieldSelectName);
        dto.setStoreFieldId(storeFieldId);
        return OperationResultUtils.operationResult(storeFieldSelectApiService.save(dto),
                OperationResultUtils.LOG_NEW + "自定义属性值", storeFieldSelectName);
    }

    @RequestMapping("/edit-store-field-select")
    public RestResponse editStoreFieldSelect(
            @Validated({ StoreFieldSelectQO.EditValidView.class }) StoreFieldSelectQO storeFieldSelectQO) {
        ServiceResponse result = storeFieldSelectApiService.edit(transferSelectQoToDTO(storeFieldSelectQO));
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_EDIT + "自定义属性值",
                storeFieldSelectQO.getStoreFieldSelectName());
    }

    @RequestMapping("/swap-store-field-select")
    public RestResponse swapStoreFieldSelect(@Validated({ StoreFieldSelectQO.SwapValidView.class,
            StoreFieldSelectQO.RemoveValidView.class }) StoreFieldSelectQO storeFieldSelectQO) {
        int upOrDown = storeFieldSelectQO.getUpOrDown();
        ServiceResponse result;
        switch (upOrDown) {
        case StoreGroupConstant.SORT_UP:
            result = storeFieldSelectApiService.upOrDown(storeFieldSelectQO.getStoreFieldSelectId(), true);
            break;
        case StoreGroupConstant.SORT_DOWN:
            result = storeFieldSelectApiService.upOrDown(storeFieldSelectQO.getStoreFieldSelectId(), false);
            break;
        case StoreGroupConstant.SORT_BOTTOM:
            result = storeFieldSelectApiService.sortBottom(storeFieldSelectQO.getStoreFieldSelectId());
            break;
        case StoreGroupConstant.SORT_TOPPING:
            result = storeFieldSelectApiService.sortTopping(storeFieldSelectQO.getStoreFieldSelectId());
            break;
        default:
            result = null;
        }
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/list-store-field-select")
    public RestResponse listStoreFieldSelect(
            @Validated({ StoreFieldSelectQO.ListValidView.class }) StoreFieldSelectQO storeFieldSelectQO,
            PageQO pageQO) {
        ServiceResponse<Page<StoreFieldSelectDTO>> result = storeFieldSelectApiService.listStoreFieldSelect(
                storeFieldSelectQO.getStoreFieldId(), storeFieldSelectQO.getSearch(), pageQO.getCurrentPage(),
                pageQO.getPageSize());
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/set-store-field-select-sort-value")
    public RestResponse setStoreFieldSelectSortValue(@Validated({ StoreFieldSelectQO.SetSortValueValidView.class,
            StoreFieldSelectQO.RemoveValidView.class }) StoreFieldSelectQO storeFieldSelectQO) {
        ServiceResponse result = storeFieldSelectApiService
                .setStoreFieldSelectSort(storeFieldSelectQO.getStoreFieldSelectId(), storeFieldSelectQO.getSortValue());
        return OperationResultUtils.operationResult(result, "排序自定义属性值",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/delete-store-field-select")
    public RestResponse editStoreFieldSelect(String storeFieldSelectIds) {
        if (StringUtils.isBlank(storeFieldSelectIds)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "storeFieldSelectIds:不能为空");
        }
        ServiceResponse result = storeFieldSelectApiService.delete(storeFieldSelectIds);
        return OperationResultUtils.operationResult(result, "批量删除自定义属性值",
                OperationResultUtils.getOperationObject(result));
    }

    private StoreFieldDTO transferQoToDTO(StoreFieldQO storeFieldQO) {
        StoreFieldDTO storeFieldDTO = EntityUtil.changeEntityNew(StoreFieldDTO.class, storeFieldQO);
        storeFieldDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return storeFieldDTO;
    }

    private StoreFieldSelectDTO transferSelectQoToDTO(StoreFieldSelectQO storeFieldSelectQO) {
        StoreFieldSelectDTO storeFieldDTO = EntityUtil.changeEntityNew(StoreFieldSelectDTO.class, storeFieldSelectQO);
        return storeFieldDTO;
    }

    private String getRegionNameByRegionId(String regionId, Map<Integer, String> regionMap) {
        String[] regionIdArr = regionId.split("_");
        StringBuilder result = new StringBuilder();
        for (String str : regionIdArr) {
            if (StringUtils.isBlank(str)) {
                continue;
            }
            result.append(regionMap.get(Integer.parseInt(str)));
            result.append("/");
        }
        return result.toString().substring(0, result.toString().length() - 1);
    }

    private boolean validStoreFieldSelectSaveParam(String storeFieldSelectNames, String storeFieldSelectCodes) {
        if (StringUtils.isBlank(storeFieldSelectCodes)) {
            return true;
        }
        if (StringUtils.isBlank(storeFieldSelectNames)) {
            return true;
        }
        String[] nameArr = storeFieldSelectNames.split(",");
        String[] codeArr = storeFieldSelectCodes.split(",");
        if (nameArr.length != codeArr.length) {
            return true;
        }
        return isRepeatCode(codeArr) || isRepeatCode(nameArr);
    }

    private boolean isRepeatCode(String[] codeArr) {
        Map<String, String> tempMap = new HashMap<>(16);
        for (String code : codeArr) {
            if (tempMap.containsKey(code)) {
                //如果包含了，说明重复
                return true;
            }
            tempMap.put(code, "1");
        }
        return false;
    }

    private Map<Integer, String> getEnterpriseIdMapName(List<Integer> enterpriseIdList) {
        Map<Integer, String> map = new HashMap<>();
        ServiceResponse<List<EnterpriseDTO>> response = enterpriseApiService.listEnterpriseByIds(enterpriseIdList);
        if (response.isSuccess()) {
            List<EnterpriseDTO> list = response.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                map = list.stream().collect(Collectors.toMap(EnterpriseDTO::getEnterpriseId, EnterpriseDTO::getEnterpriseName));
            }
        }
        return map;
    }
}
