package com.gic.store.web.controller;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreRegionDTO;
import com.gic.store.dto.StoreStatusSettingDTO;
import com.gic.store.service.StoreRegionApiService;
import com.gic.store.service.StoreStatusSettingApiService;
import com.gic.store.web.vo.StoreStatusSettingVO;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * @ClassName: StoreStatusSettingController
 * @Description:  
 * @author zhiwj
 * @date 2020-09-09 19:18
 */
@RestController
public class StoreStatusSettingController {
    private Logger logger = Logger.getLogger(StoreStatusSettingController.class);

    @Autowired
    private StoreStatusSettingApiService storeStatusSettingApiService;
    @Autowired
    private StoreRegionApiService storeRegionApiService;

    @RequestMapping("storestatus-setting")
    public RestResponse saveStoreStatusSetting(Integer regionId, Integer days, Integer value){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if(regionId == null || days == null || value == null){
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        StoreStatusSettingDTO dto = new StoreStatusSettingDTO();
        dto.setEnterpriseId(enterpriseId);
        dto.setDays(days);
        dto.setRegionId(regionId);
        dto.setValue(value);
        ServiceResponse serviceResponse = this.storeStatusSettingApiService.saveSetting(dto);
        if(serviceResponse.isSuccess()){
            return EnterpriseRestResponse.success(serviceResponse.getResult());
        }
        return EnterpriseRestResponse.failure(serviceResponse);
    }

    @RequestMapping("list-storestatus-setting")
    public RestResponse listStoreStatusSetting(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreStatusSettingDTO>> serviceResponse = this.storeStatusSettingApiService.listSetting(enterpriseId);
        if(serviceResponse.isSuccess()){
            List<StoreStatusSettingVO> list = new ArrayList<>();
            for(StoreStatusSettingDTO dto : serviceResponse.getResult()){
                StoreStatusSettingVO vo = new StoreStatusSettingVO();
                vo.setDays(dto.getDays());
                vo.setEnterpriseId(enterpriseId);
                vo.setRegionId(dto.getRegionId());
                ServiceResponse<StoreRegionDTO> response = this.storeRegionApiService.getStoreRegionByRegion(enterpriseId, dto.getRegionId());
                if(response.isSuccess()){
                    if(response.getResult() != null){
                        vo.setRegionName(response.getResult().getRegionName());
                    }
                }
                vo.setValue(dto.getValue());
                list.add(vo);
            }
            ServiceResponse<List<StoreRegionDTO>> response = this.storeRegionApiService.listStoreRegion(enterpriseId, null, StoreOwnTypeEnum.OWNER.getCode());
            if(response.isSuccess()){
                List<StoreRegionDTO> storeRegionDTOList = response.getResult();
                List<StoreStatusSettingVO> notSetting = new ArrayList<>();
                for(StoreRegionDTO dto : storeRegionDTOList){
                    boolean exists = false;
                    for(StoreStatusSettingVO vo : list){
                        if(dto.getRegionId().intValue() == vo.getRegionId().intValue()){
                            exists = true;
                        }
                    }
                    if(!exists){
                        StoreStatusSettingVO vo = new StoreStatusSettingVO();
                        //默认给一个30天
                        vo.setDays(30);
                        vo.setEnterpriseId(enterpriseId);
                        vo.setRegionId(dto.getRegionId());
                        vo.setRegionName(dto.getRegionName());
                        vo.setValue(2);
                        notSetting.add(vo);
                    }
                }
                list.addAll(notSetting);
            }
            return RestResponse.success(list);
        }
        return EnterpriseRestResponse.failure(serviceResponse);
    }

    @RequestMapping("delete-storestatus-setting")
    public RestResponse deleteStoreStatusSetting(Integer id){
        if(id == null){
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse serviceResponse = this.storeStatusSettingApiService.delete(enterpriseId, id);
        if(serviceResponse.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(serviceResponse);
    }
}
